@@ -6,15 +6,13 @@
 #ifndef mozilla_a11y_RootAccessible_h__
 #define mozilla_a11y_RootAccessible_h__
 
-#include "nsCaretAccessible.h"
+#include "HyperTextAccessible.h"
 #include "DocAccessibleWrap.h"
 
-
-#include "nsHashtable.h"
-#include "nsCaretAccessible.h"
-#include "nsIDocument.h"
 #include "nsIDOMEventListener.h"
 
+class nsIDocument;
+
 namespace mozilla {
 namespace a11y {
 
@@ -36,12 +34,11 @@ public:
 
   // Accessible
   virtual mozilla::a11y::ENameValueFlag Name(nsString& aName);
-  virtual Relation RelationByType(PRUint32 aType);
+  virtual Relation RelationByType(RelationType aType) MOZ_OVERRIDE;
   virtual mozilla::a11y::role NativeRole();
-  virtual PRUint64 NativeState();
+  virtual uint64_t NativeState();
 
   // RootAccessible
-  nsCaretAccessible* GetCaretAccessible();
 
   /**
    * Notify that the sub document presshell was activated.
@@ -77,20 +74,17 @@ protected:
     void HandleTreeInvalidatedEvent(nsIDOMEvent* aEvent,
                                     XULTreeAccessible* aAccessible);
 
-    PRUint32 GetChromeFlags();
+    uint32_t GetChromeFlags();
 #endif
-
-    nsRefPtr<nsCaretAccessible> mCaretAccessible;
 };
 
+inline RootAccessible*
+Accessible::AsRoot()
+{
+  return IsRoot() ? static_cast<mozilla::a11y::RootAccessible*>(this) : nullptr;
+}
+
 } // namespace a11y
 } // namespace mozilla
 
-inline mozilla::a11y::RootAccessible*
-Accessible::AsRoot()
-{
-  return mFlags & eRootAccessible ?
-    static_cast<mozilla::a11y::RootAccessible*>(this) : nsnull;
-}
-
 #endif
