@@ -7,6 +7,7 @@
 #include "HTMLListAccessible.h"
 
 #include "DocAccessible.h"
+#include "nsAccUtils.h"
 #include "Role.h"
 #include "States.h"
 
@@ -46,13 +47,12 @@ HTMLLIAccessible::
   HTMLLIAccessible(nsIContent* aContent, DocAccessible* aDoc) :
   HyperTextAccessibleWrap(aContent, aDoc), mBullet(nullptr)
 {
-  mFlags |= eHTMLListItemAccessible;
+  mType = eHTMLLiType;
 
   nsBlockFrame* blockFrame = do_QueryFrame(GetFrame());
   if (blockFrame && blockFrame->HasBullet()) {
     mBullet = new HTMLListBulletAccessible(mContent, mDoc);
-    if (!Document()->BindToDocument(mBullet, nullptr))
-      mBullet = nullptr;
+    Document()->BindToDocument(mBullet, nullptr);
   }
 }
 
@@ -98,6 +98,33 @@ HTMLLIAccessible::GetBounds(int32_t* aX, int32_t* aY,
   return NS_OK;
 }
 
+int32_t
+HTMLLIAccessible::FindOffset(int32_t aOffset, nsDirection aDirection,
+                             nsSelectionAmount aAmount,
+                             EWordMovementType aWordMovementType)
+{
+  Accessible* child = GetChildAtOffset(aOffset);
+  if (!child)
+    return -1;
+
+  if (child != mBullet) {
+    if (aDirection == eDirPrevious &&
+        (aAmount == eSelectBeginLine || aAmount == eSelectLine))
+      return 0;
+
+    return HyperTextAccessible::FindOffset(aOffset, aDirection,
+                                           aAmount, aWordMovementType);
+  }
+
+  if (aDirection == eDirPrevious)
+    return 0;
+
+  if (aAmount == eSelectEndLine || aAmount == eSelectLine)
+    return CharacterCount();
+
+  return nsAccUtils::TextLength(child);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // HTMLLIAccessible: public
 
@@ -112,9 +139,8 @@ HTMLLIAccessible::UpdateBullet(bool aHasBullet)
   DocAccessible* document = Document();
   if (aHasBullet) {
     mBullet = new HTMLListBulletAccessible(mContent, mDoc);
-    if (document->BindToDocument(mBullet, nullptr)) {
-      InsertChildAt(0, mBullet);
-    }
+    document->BindToDocument(mBullet, nullptr);
+    InsertChildAt(0, mBullet);
   } else {
     RemoveChild(mBullet);
     document->UnbindFromDocument(mBullet);
@@ -145,7 +171,7 @@ HTMLListBulletAccessible::
   HTMLListBulletAccessible(nsIContent* aContent, DocAccessible* aDoc) :
   LeafAccessible(aContent, aDoc)
 {
-  mFlags |= eSharedNode;
+  mStateFlags |= eSharedNode;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
