@@ -4,23 +4,30 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "SVGPathData.h"
+
+#include "gfx2DGlue.h"
 #include "gfxPlatform.h"
+#include "mozilla/gfx/2D.h"
+#include "mozilla/gfx/Types.h"
+#include "mozilla/gfx/Point.h"
+#include "mozilla/RefPtr.h"
 #include "nsError.h"
 #include "nsString.h"
-#include "nsSVGElement.h"
 #include "nsSVGPathDataParser.h"
 #include "nsSVGPathGeometryElement.h" // for nsSVGMark
 #include <stdarg.h>
-#include "string.h"
 #include "SVGContentUtils.h"
 #include "SVGPathSegUtils.h"
+#include "gfxContext.h"
+#include <algorithm>
 
 using namespace mozilla;
+using namespace mozilla::gfx;
 
 static bool IsMoveto(uint16_t aSegType)
 {
-  return aSegType == nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS ||
-         aSegType == nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL;
+  return aSegType == PATHSEG_MOVETO_ABS ||
+         aSegType == PATHSEG_MOVETO_REL;
 }
 
 nsresult
@@ -164,8 +171,8 @@ SVGPathData::GetDistancesFromOriginToEndsOfVisibleSegments(nsTArray<double> *aOu
     // this case an equal amount of time is spent on each path segment,
     // except on moveto segments which are jumped over immediately.
 
-    if (i == 0 || (segType != nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS &&
-                   segType != nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL)) {
+    if (i == 0 || (segType != PATHSEG_MOVETO_ABS &&
+                   segType != PATHSEG_MOVETO_REL)) {
       if (!aOutput->AppendElement(state.length)) {
         return false;
       }
@@ -200,7 +207,7 @@ SVGPathData::GetPathSegAtLength(float aDistance) const
 
   NS_ABORT_IF_FALSE(i == mData.Length(), "Very, very bad - mData corrupt");
 
-  return NS_MAX(0U, segIndex - 1); // -1 because while loop takes us 1 too far
+  return std::max(0U, segIndex - 1); // -1 because while loop takes us 1 too far
 }
 
 /**
@@ -237,16 +244,296 @@ ApproximateZeroLengthSubpathSquareCaps(const gfxPoint &aPoint, gfxContext *aCtx)
   aCtx->MoveTo(aPoint);
 }
 
+static void
+ApproximateZeroLengthSubpathSquareCaps(const Point& aPoint,
+                                       DrawTarget* aDT,
+                                       PathBuilder* aPB)
+{
+  // Cairo's fixed point fractional part is 8 bits wide, so its device space
+  // coordinate granularity is 1/256 pixels. However, to prevent user space
+  // |aPoint| and |aPoint + tinyAdvance| being rounded to the same device
+  // coordinates, we double this for |tinyAdvance|:
+
+  Matrix currentTransform = aDT->GetTransform();
+  currentTransform.Invert();
+  Size tinyAdvance = currentTransform * Size(2.0/256.0, 0.0);
+
+  aPB->MoveTo(aPoint);
+  aPB->LineTo(aPoint + Point(tinyAdvance.width, tinyAdvance.height));
+  aPB->MoveTo(aPoint);
+}
+
+#define MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS_TO_DT               \
+  do {                                                                        \
+    if (capsAreSquare && !subpathHasLength && subpathContainsNonArc &&        \
+        SVGPathSegUtils::IsValidType(prevSegType) &&                          \
+        (!IsMoveto(prevSegType) ||                                            \
+         segType == PATHSEG_CLOSEPATH)) {                                     \
+      ApproximateZeroLengthSubpathSquareCaps(segStart, aDT, builder);         \
+    }                                                                         \
+  } while(0)
+
 #define MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS                     \
   do {                                                                        \
     if (capsAreSquare && !subpathHasLength && subpathContainsNonArc &&        \
         SVGPathSegUtils::IsValidType(prevSegType) &&                          \
         (!IsMoveto(prevSegType) ||                                            \
-         segType == nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH)) {                   \
+         segType == PATHSEG_CLOSEPATH)) {                                     \
       ApproximateZeroLengthSubpathSquareCaps(segStart, aCtx);                 \
     }                                                                         \
   } while(0)
 
+TemporaryRef<Path>
+SVGPathData::ConstructPath(DrawTarget *aDT,
+                           FillRule aFillRule,
+                           CapStyle aCapStyle) const
+{
+  if (mData.IsEmpty() || !IsMoveto(SVGPathSegUtils::DecodeType(mData[0]))) {
+    return nullptr; // paths without an initial moveto are invalid
+  }
+
+  RefPtr<PathBuilder> builder = aDT->CreatePathBuilder(aFillRule);
+
+  bool capsAreSquare = aCapStyle == CAP_SQUARE;
+  bool subpathHasLength = false;  // visual length
+  bool subpathContainsNonArc = false;
+
+  uint32_t segType     = PATHSEG_UNKNOWN;
+  uint32_t prevSegType = PATHSEG_UNKNOWN;
+  Point pathStart(0.0, 0.0); // start point of [sub]path
+  Point segStart(0.0, 0.0);
+  Point segEnd;
+  Point cp1, cp2;            // previous bezier's control points
+  Point tcp1, tcp2;          // temporaries
+
+  // Regarding cp1 and cp2: If the previous segment was a cubic bezier curve,
+  // then cp2 is its second control point. If the previous segment was a
+  // quadratic curve, then cp1 is its (only) control point.
+
+  uint32_t i = 0;
+  while (i < mData.Length()) {
+    segType = SVGPathSegUtils::DecodeType(mData[i++]);
+    uint32_t argCount = SVGPathSegUtils::ArgCountForType(segType);
+
+    switch (segType)
+    {
+    case PATHSEG_CLOSEPATH:
+      // set this early to allow drawing of square caps for "M{x},{y} Z":
+      subpathContainsNonArc = true;
+      MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS_TO_DT;
+      segEnd = pathStart;
+      builder->Close();
+      break;
+
+    case PATHSEG_MOVETO_ABS:
+      MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS_TO_DT;
+      pathStart = segEnd = Point(mData[i], mData[i+1]);
+      builder->MoveTo(segEnd);
+      subpathHasLength = false;
+      subpathContainsNonArc = false;
+      break;
+
+    case PATHSEG_MOVETO_REL:
+      MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS_TO_DT;
+      pathStart = segEnd = segStart + Point(mData[i], mData[i+1]);
+      builder->MoveTo(segEnd);
+      subpathHasLength = false;
+      subpathContainsNonArc = false;
+      break;
+
+    case PATHSEG_LINETO_ABS:
+      segEnd = Point(mData[i], mData[i+1]);
+      builder->LineTo(segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_LINETO_REL:
+      segEnd = segStart + Point(mData[i], mData[i+1]);
+      builder->LineTo(segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_CURVETO_CUBIC_ABS:
+      cp1 = Point(mData[i], mData[i+1]);
+      cp2 = Point(mData[i+2], mData[i+3]);
+      segEnd = Point(mData[i+4], mData[i+5]);
+      builder->BezierTo(cp1, cp2, segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart || segEnd != cp1 || segEnd != cp2);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_CURVETO_CUBIC_REL:
+      cp1 = segStart + Point(mData[i], mData[i+1]);
+      cp2 = segStart + Point(mData[i+2], mData[i+3]);
+      segEnd = segStart + Point(mData[i+4], mData[i+5]);
+      builder->BezierTo(cp1, cp2, segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart || segEnd != cp1 || segEnd != cp2);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_CURVETO_QUADRATIC_ABS:
+      cp1 = Point(mData[i], mData[i+1]);
+      // Convert quadratic curve to cubic curve:
+      tcp1 = segStart + (cp1 - segStart) * 2 / 3;
+      segEnd = Point(mData[i+2], mData[i+3]); // set before setting tcp2!
+      tcp2 = cp1 + (segEnd - cp1) / 3;
+      builder->BezierTo(tcp1, tcp2, segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart || segEnd != cp1);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_CURVETO_QUADRATIC_REL:
+      cp1 = segStart + Point(mData[i], mData[i+1]);
+      // Convert quadratic curve to cubic curve:
+      tcp1 = segStart + (cp1 - segStart) * 2 / 3;
+      segEnd = segStart + Point(mData[i+2], mData[i+3]); // set before setting tcp2!
+      tcp2 = cp1 + (segEnd - cp1) / 3;
+      builder->BezierTo(tcp1, tcp2, segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart || segEnd != cp1);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_ARC_ABS:
+    case PATHSEG_ARC_REL:
+    {
+      Point radii(mData[i], mData[i+1]);
+      segEnd = Point(mData[i+5], mData[i+6]);
+      if (segType == PATHSEG_ARC_REL) {
+        segEnd += segStart;
+      }
+      if (segEnd != segStart) {
+        if (radii.x == 0.0f || radii.y == 0.0f) {
+          builder->LineTo(segEnd);
+        } else {
+          nsSVGArcConverter converter(segStart, segEnd, radii, mData[i+2],
+                                      mData[i+3] != 0, mData[i+4] != 0);
+          while (converter.GetNextSegment(&cp1, &cp2, &segEnd)) {
+            builder->BezierTo(cp1, cp2, segEnd);
+          }
+        }
+      }
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart);
+      }
+      break;
+    }
+
+    case PATHSEG_LINETO_HORIZONTAL_ABS:
+      segEnd = Point(mData[i], segStart.y);
+      builder->LineTo(segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_LINETO_HORIZONTAL_REL:
+      segEnd = segStart + Point(mData[i], 0.0f);
+      builder->LineTo(segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_LINETO_VERTICAL_ABS:
+      segEnd = Point(segStart.x, mData[i]);
+      builder->LineTo(segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_LINETO_VERTICAL_REL:
+      segEnd = segStart + Point(0.0f, mData[i]);
+      builder->LineTo(segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
+      cp1 = SVGPathSegUtils::IsCubicType(prevSegType) ? segStart * 2 - cp2 : segStart;
+      cp2 = Point(mData[i],   mData[i+1]);
+      segEnd = Point(mData[i+2], mData[i+3]);
+      builder->BezierTo(cp1, cp2, segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart || segEnd != cp1 || segEnd != cp2);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
+      cp1 = SVGPathSegUtils::IsCubicType(prevSegType) ? segStart * 2 - cp2 : segStart;
+      cp2 = segStart + Point(mData[i], mData[i+1]);
+      segEnd = segStart + Point(mData[i+2], mData[i+3]);
+      builder->BezierTo(cp1, cp2, segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart || segEnd != cp1 || segEnd != cp2);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
+      cp1 = SVGPathSegUtils::IsQuadraticType(prevSegType) ? segStart * 2 - cp1 : segStart;
+      // Convert quadratic curve to cubic curve:
+      tcp1 = segStart + (cp1 - segStart) * 2 / 3;
+      segEnd = Point(mData[i], mData[i+1]); // set before setting tcp2!
+      tcp2 = cp1 + (segEnd - cp1) / 3;
+      builder->BezierTo(tcp1, tcp2, segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart || segEnd != cp1);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
+      cp1 = SVGPathSegUtils::IsQuadraticType(prevSegType) ? segStart * 2 - cp1 : segStart;
+      // Convert quadratic curve to cubic curve:
+      tcp1 = segStart + (cp1 - segStart) * 2 / 3;
+      segEnd = segStart + Point(mData[i], mData[i+1]); // changed before setting tcp2!
+      tcp2 = cp1 + (segEnd - cp1) / 3;
+      builder->BezierTo(tcp1, tcp2, segEnd);
+      if (!subpathHasLength) {
+        subpathHasLength = (segEnd != segStart || segEnd != cp1);
+      }
+      subpathContainsNonArc = true;
+      break;
+
+    default:
+      NS_NOTREACHED("Bad path segment type");
+      return nullptr; // according to spec we'd use everything up to the bad seg anyway
+    }
+    i += argCount;
+    prevSegType = segType;
+    segStart = segEnd;
+  }
+
+  NS_ABORT_IF_FALSE(i == mData.Length(), "Very, very bad - mData corrupt");
+  NS_ABORT_IF_FALSE(prevSegType == segType,
+                    "prevSegType should be left at the final segType");
+
+  MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS_TO_DT;
+
+  return builder->Finish();
+}
+
 void
 SVGPathData::ConstructPath(gfxContext *aCtx) const
 {
@@ -258,8 +545,8 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
   bool subpathHasLength = false;  // visual length
   bool subpathContainsNonArc = false;
 
-  uint32_t segType     = nsIDOMSVGPathSeg::PATHSEG_UNKNOWN;
-  uint32_t prevSegType = nsIDOMSVGPathSeg::PATHSEG_UNKNOWN;
+  uint32_t segType     = PATHSEG_UNKNOWN;
+  uint32_t prevSegType = PATHSEG_UNKNOWN;
   gfxPoint pathStart(0.0, 0.0); // start point of [sub]path
   gfxPoint segStart(0.0, 0.0);
   gfxPoint segEnd;
@@ -277,7 +564,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
 
     switch (segType)
     {
-    case nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH:
+    case PATHSEG_CLOSEPATH:
       // set this early to allow drawing of square caps for "M{x},{y} Z":
       subpathContainsNonArc = true;
       MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS;
@@ -285,7 +572,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       aCtx->ClosePath();
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS:
+    case PATHSEG_MOVETO_ABS:
       MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS;
       pathStart = segEnd = gfxPoint(mData[i], mData[i+1]);
       aCtx->MoveTo(segEnd);
@@ -293,7 +580,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = false;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL:
+    case PATHSEG_MOVETO_REL:
       MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS;
       pathStart = segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
       aCtx->MoveTo(segEnd);
@@ -301,7 +588,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = false;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_ABS:
+    case PATHSEG_LINETO_ABS:
       segEnd = gfxPoint(mData[i], mData[i+1]);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -310,7 +597,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_REL:
+    case PATHSEG_LINETO_REL:
       segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -319,7 +606,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_ABS:
+    case PATHSEG_CURVETO_CUBIC_ABS:
       cp1 = gfxPoint(mData[i], mData[i+1]);
       cp2 = gfxPoint(mData[i+2], mData[i+3]);
       segEnd = gfxPoint(mData[i+4], mData[i+5]);
@@ -330,7 +617,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_REL:
+    case PATHSEG_CURVETO_CUBIC_REL:
       cp1 = segStart + gfxPoint(mData[i], mData[i+1]);
       cp2 = segStart + gfxPoint(mData[i+2], mData[i+3]);
       segEnd = segStart + gfxPoint(mData[i+4], mData[i+5]);
@@ -341,7 +628,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_ABS:
+    case PATHSEG_CURVETO_QUADRATIC_ABS:
       cp1 = gfxPoint(mData[i], mData[i+1]);
       // Convert quadratic curve to cubic curve:
       tcp1 = segStart + (cp1 - segStart) * 2 / 3;
@@ -354,7 +641,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_REL:
+    case PATHSEG_CURVETO_QUADRATIC_REL:
       cp1 = segStart + gfxPoint(mData[i], mData[i+1]);
       // Convert quadratic curve to cubic curve:
       tcp1 = segStart + (cp1 - segStart) * 2 / 3;
@@ -367,32 +654,39 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_ARC_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_ARC_REL:
+    case PATHSEG_ARC_ABS:
+    case PATHSEG_ARC_REL:
     {
       gfxPoint radii(mData[i], mData[i+1]);
       segEnd = gfxPoint(mData[i+5], mData[i+6]);
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_ARC_REL) {
+      if (segType == PATHSEG_ARC_REL) {
         segEnd += segStart;
       }
       if (segEnd != segStart) {
         if (radii.x == 0.0f || radii.y == 0.0f) {
           aCtx->LineTo(segEnd);
         } else {
-          nsSVGArcConverter converter(segStart, segEnd, radii, mData[i+2],
+          nsSVGArcConverter converter(ToPoint(segStart), ToPoint(segEnd),
+                                      ToPoint(radii), mData[i+2],
                                       mData[i+3] != 0, mData[i+4] != 0);
-          while (converter.GetNextSegment(&cp1, &cp2, &segEnd)) {
-            aCtx->CurveTo(cp1, cp2, segEnd);
+          Point cp1, cp2, segEnd_;
+          while (converter.GetNextSegment(&cp1, &cp2, &segEnd_)) {
+            aCtx->CurveTo(ThebesPoint(cp1), ThebesPoint(cp2), ThebesPoint(segEnd_));
           }
+          segEnd = ThebesPoint(segEnd_);
         }
       }
       if (!subpathHasLength) {
+        // Round to make sure the current comparison doesn't fail due to
+        // precision issues:
+        // XXX kill after all code is converted to float precision
+        segStart = ThebesPoint(ToPoint(segStart));
         subpathHasLength = (segEnd != segStart);
       }
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_ABS:
+    case PATHSEG_LINETO_HORIZONTAL_ABS:
       segEnd = gfxPoint(mData[i], segStart.y);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -401,7 +695,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_REL:
+    case PATHSEG_LINETO_HORIZONTAL_REL:
       segEnd = segStart + gfxPoint(mData[i], 0.0f);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -410,7 +704,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_ABS:
+    case PATHSEG_LINETO_VERTICAL_ABS:
       segEnd = gfxPoint(segStart.x, mData[i]);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -419,7 +713,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_REL:
+    case PATHSEG_LINETO_VERTICAL_REL:
       segEnd = segStart + gfxPoint(0.0f, mData[i]);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -428,7 +722,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
       cp1 = SVGPathSegUtils::IsCubicType(prevSegType) ? segStart * 2 - cp2 : segStart;
       cp2 = gfxPoint(mData[i],   mData[i+1]);
       segEnd = gfxPoint(mData[i+2], mData[i+3]);
@@ -439,7 +733,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
       cp1 = SVGPathSegUtils::IsCubicType(prevSegType) ? segStart * 2 - cp2 : segStart;
       cp2 = segStart + gfxPoint(mData[i], mData[i+1]);
       segEnd = segStart + gfxPoint(mData[i+2], mData[i+3]);
@@ -450,7 +744,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
       cp1 = SVGPathSegUtils::IsQuadraticType(prevSegType) ? segStart * 2 - cp1 : segStart;
       // Convert quadratic curve to cubic curve:
       tcp1 = segStart + (cp1 - segStart) * 2 / 3;
@@ -463,7 +757,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
       cp1 = SVGPathSegUtils::IsQuadraticType(prevSegType) ? segStart * 2 - cp1 : segStart;
       // Convert quadratic curve to cubic curve:
       tcp1 = segStart + (cp1 - segStart) * 2 / 3;
@@ -492,8 +786,8 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
   MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS;
 }
 
-already_AddRefed<gfxFlattenedPath>
-SVGPathData::ToFlattenedPath(const gfxMatrix& aMatrix) const
+already_AddRefed<gfxPath>
+SVGPathData::ToPath(const gfxMatrix& aMatrix) const
 {
   nsRefPtr<gfxContext> tmpCtx =
     new gfxContext(gfxPlatform::GetPlatform()->ScreenReferenceSurface());
@@ -502,7 +796,7 @@ SVGPathData::ToFlattenedPath(const gfxMatrix& aMatrix) const
   ConstructPath(tmpCtx);
   tmpCtx->IdentityMatrix();
 
-  return tmpCtx->GetFlattenedPath();
+  return tmpCtx->CopyPath();
 }
 
 static double
@@ -535,7 +829,7 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
   float pathStartAngle = 0.0f;
 
   // info on previous segment:
-  uint16_t prevSegType = nsIDOMSVGPathSeg::PATHSEG_UNKNOWN;
+  uint16_t prevSegType = PATHSEG_UNKNOWN;
   gfxPoint prevSegEnd(0.0, 0.0);
   float prevSegEndAngle = 0.0f;
   gfxPoint prevCP; // if prev seg was a bezier, this was its last control point
@@ -552,14 +846,14 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
 
     switch (segType) // to find segStartAngle, segEnd and segEndAngle
     {
-    case nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH:
+    case PATHSEG_CLOSEPATH:
       segEnd = pathStart;
       segStartAngle = segEndAngle = AngleOfVectorF(segEnd - segStart);
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL:
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS) {
+    case PATHSEG_MOVETO_ABS:
+    case PATHSEG_MOVETO_REL:
+      if (segType == PATHSEG_MOVETO_ABS) {
         segEnd = gfxPoint(mData[i], mData[i+1]);
       } else {
         segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
@@ -571,9 +865,9 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       i += 2;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_REL:
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_LINETO_ABS) {
+    case PATHSEG_LINETO_ABS:
+    case PATHSEG_LINETO_REL:
+      if (segType == PATHSEG_LINETO_ABS) {
         segEnd = gfxPoint(mData[i], mData[i+1]);
       } else {
         segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
@@ -582,11 +876,11 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       i += 2;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_REL:
+    case PATHSEG_CURVETO_CUBIC_ABS:
+    case PATHSEG_CURVETO_CUBIC_REL:
     {
       gfxPoint cp1, cp2; // control points
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_ABS) {
+      if (segType == PATHSEG_CURVETO_CUBIC_ABS) {
         cp1 = gfxPoint(mData[i],   mData[i+1]);
         cp2 = gfxPoint(mData[i+2], mData[i+3]);
         segEnd = gfxPoint(mData[i+4], mData[i+5]);
@@ -608,11 +902,11 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_REL:
+    case PATHSEG_CURVETO_QUADRATIC_ABS:
+    case PATHSEG_CURVETO_QUADRATIC_REL:
     {
       gfxPoint cp1, cp2; // control points
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_ABS) {
+      if (segType == PATHSEG_CURVETO_QUADRATIC_ABS) {
         cp1 = gfxPoint(mData[i],   mData[i+1]);
         segEnd = gfxPoint(mData[i+2], mData[i+3]);
       } else {
@@ -626,15 +920,15 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_ARC_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_ARC_REL:
+    case PATHSEG_ARC_ABS:
+    case PATHSEG_ARC_REL:
     {
       double rx = mData[i];
       double ry = mData[i+1];
       double angle = mData[i+2];
       bool largeArcFlag = mData[i+3] != 0.0f;
       bool sweepFlag = mData[i+4] != 0.0f;
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_ARC_ABS) {
+      if (segType == PATHSEG_ARC_ABS) {
         segEnd = gfxPoint(mData[i+5], mData[i+6]);
       } else {
         segEnd = segStart + gfxPoint(mData[i+5], mData[i+6]);
@@ -729,9 +1023,9 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_REL:
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_ABS) {
+    case PATHSEG_LINETO_HORIZONTAL_ABS:
+    case PATHSEG_LINETO_HORIZONTAL_REL:
+      if (segType == PATHSEG_LINETO_HORIZONTAL_ABS) {
         segEnd = gfxPoint(mData[i++], segStart.y);
       } else {
         segEnd = segStart + gfxPoint(mData[i++], 0.0f);
@@ -739,9 +1033,9 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       segStartAngle = segEndAngle = AngleOfVectorF(segEnd - segStart);
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_REL:
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_ABS) {
+    case PATHSEG_LINETO_VERTICAL_ABS:
+    case PATHSEG_LINETO_VERTICAL_REL:
+      if (segType == PATHSEG_LINETO_VERTICAL_ABS) {
         segEnd = gfxPoint(segStart.x, mData[i++]);
       } else {
         segEnd = segStart + gfxPoint(0.0f, mData[i++]);
@@ -749,13 +1043,13 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       segStartAngle = segEndAngle = AngleOfVectorF(segEnd - segStart);
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
     {
       gfxPoint cp1 = SVGPathSegUtils::IsCubicType(prevSegType) ?
                        segStart * 2 - prevCP : segStart;
       gfxPoint cp2;
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_ABS) {
+      if (segType == PATHSEG_CURVETO_CUBIC_SMOOTH_ABS) {
         cp2 = gfxPoint(mData[i], mData[i+1]);
         segEnd = gfxPoint(mData[i+2], mData[i+3]);
       } else {
@@ -775,13 +1069,13 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
     {
       gfxPoint cp1 = SVGPathSegUtils::IsQuadraticType(prevSegType) ?
                        segStart * 2 - prevCP : segStart;
       gfxPoint cp2;
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS) {
+      if (segType == PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS) {
         segEnd = gfxPoint(mData[i], mData[i+1]);
       } else {
         segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
@@ -802,31 +1096,33 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
 
     // Set the angle of the mark at the start of this segment:
     if (aMarks->Length()) {
-      nsSVGMark &mark = aMarks->ElementAt(aMarks->Length() - 1);
+      nsSVGMark &mark = aMarks->LastElement();
       if (!IsMoveto(segType) && IsMoveto(prevSegType)) {
         // start of new subpath
         pathStartAngle = mark.angle = segStartAngle;
       } else if (IsMoveto(segType) && !IsMoveto(prevSegType)) {
         // end of a subpath
-        if (prevSegType != nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH)
+        if (prevSegType != PATHSEG_CLOSEPATH)
           mark.angle = prevSegEndAngle;
       } else {
-        if (!(segType == nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH &&
-              prevSegType == nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH))
+        if (!(segType == PATHSEG_CLOSEPATH &&
+              prevSegType == PATHSEG_CLOSEPATH))
           mark.angle = SVGContentUtils::AngleBisect(prevSegEndAngle, segStartAngle);
       }
     }
 
     // Add the mark at the end of this segment, and set its position:
     if (!aMarks->AppendElement(nsSVGMark(static_cast<float>(segEnd.x),
-                                         static_cast<float>(segEnd.y), 0))) {
+                                         static_cast<float>(segEnd.y),
+                                         0.0f,
+                                         nsSVGMark::eMid))) {
       aMarks->Clear(); // OOM, so try to free some
       return;
     }
 
-    if (segType == nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH &&
-        prevSegType != nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH) {
-      aMarks->ElementAt(aMarks->Length() - 1).angle =
+    if (segType == PATHSEG_CLOSEPATH &&
+        prevSegType != PATHSEG_CLOSEPATH) {
+      aMarks->LastElement().angle =
         //aMarks->ElementAt(pathStartIndex).angle =
         SVGContentUtils::AngleBisect(segEndAngle, pathStartAngle);
     }
@@ -838,8 +1134,12 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
 
   NS_ABORT_IF_FALSE(i == mData.Length(), "Very, very bad - mData corrupt");
 
-  if (aMarks->Length() &&
-      prevSegType != nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH)
-    aMarks->ElementAt(aMarks->Length() - 1).angle = prevSegEndAngle;
+  if (aMarks->Length()) {
+    if (prevSegType != PATHSEG_CLOSEPATH) {
+      aMarks->LastElement().angle = prevSegEndAngle;
+    }
+    aMarks->LastElement().type = nsSVGMark::eEnd;
+    aMarks->ElementAt(0).type = nsSVGMark::eStart;
+  }
 }
 
