@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set ts=8 sts=4 et sw=4 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,26 +12,27 @@
  * see http://developer.mozilla.org/en/docs/XUL
  */
 
-#include "jsapi.h"
-#include "jsfriendapi.h"
 #include "nsXULContentSink.h"
+
+#include "jsfriendapi.h"
+
 #include "nsCOMPtr.h"
 #include "nsForwardReference.h"
+#include "nsHTMLStyleSheet.h"
 #include "nsIContentSink.h"
+#include "nsIDocument.h"
 #include "nsIDOMEventListener.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMXULDocument.h"
-#include "nsIDocument.h"
 #include "nsIFormControl.h"
-#include "nsHTMLStyleSheet.h"
 #include "nsINameSpaceManager.h"
 #include "nsINodeInfo.h"
 #include "nsIScriptContext.h"
-#include "nsIScriptRuntime.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIServiceManager.h"
 #include "nsIURL.h"
-#include "nsIViewManager.h"
+#include "nsParserBase.h"
+#include "nsViewManager.h"
 #include "nsIXULDocument.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsLayoutCID.h"
@@ -53,6 +55,7 @@
 #include "nsXMLContentSink.h"
 #include "nsIConsoleService.h"
 #include "nsIScriptError.h"
+#include "nsContentTypeParser.h"
 
 #ifdef PR_LOGGING
 static PRLogModuleInfo* gLog;
@@ -61,7 +64,7 @@ static PRLogModuleInfo* gLog;
 //----------------------------------------------------------------------
 
 XULContentSinkImpl::ContextStack::ContextStack()
-    : mTop(nsnull), mDepth(0)
+    : mTop(nullptr), mDepth(0)
 {
 }
 
@@ -139,20 +142,30 @@ XULContentSinkImpl::ContextStack::Clear()
     cur = next;
   }
 
-  mTop = nsnull;
+  mTop = nullptr;
   mDepth = 0;
 }
 
+void
+XULContentSinkImpl::ContextStack::Traverse(nsCycleCollectionTraversalCallback& aCb)
+{
+  nsCycleCollectionTraversalCallback& cb = aCb;
+  for (ContextStack::Entry* tmp = mTop; tmp; tmp = tmp->mNext) {
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mNode)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mChildren)
+  }
+}
+
 //----------------------------------------------------------------------
 
 
 XULContentSinkImpl::XULContentSinkImpl()
-    : mText(nsnull),
+    : mText(nullptr),
       mTextLength(0),
       mTextSize(0),
       mConstrainSize(true),
       mState(eInProlog),
-      mParser(nsnull)
+      mParser(nullptr)
 {
 
 #ifdef PR_LOGGING
@@ -170,16 +183,37 @@ XULContentSinkImpl::~XULContentSinkImpl()
     NS_ASSERTION(mContextStack.Depth() == 0, "Context stack not empty?");
     mContextStack.Clear();
 
-    PR_FREEIF(mText);
+    moz_free(mText);
 }
 
 //----------------------------------------------------------------------
 // nsISupports interface
 
-NS_IMPL_ISUPPORTS3(XULContentSinkImpl,
-                   nsIXMLContentSink,
-                   nsIContentSink,
-                   nsIExpatSink)
+NS_IMPL_CYCLE_COLLECTION_CLASS(XULContentSinkImpl)
+
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(XULContentSinkImpl)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mNodeInfoManager)
+  tmp->mContextStack.Clear();
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mPrototype)
+  NS_IF_RELEASE(tmp->mParser);
+NS_IMPL_CYCLE_COLLECTION_UNLINK_END
+
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(XULContentSinkImpl)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mNodeInfoManager)
+  tmp->mContextStack.Traverse(cb);
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPrototype)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_RAWPTR(mParser)
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
+
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(XULContentSinkImpl)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIXMLContentSink)
+  NS_INTERFACE_MAP_ENTRY(nsIXMLContentSink)
+  NS_INTERFACE_MAP_ENTRY(nsIExpatSink)
+  NS_INTERFACE_MAP_ENTRY(nsIContentSink)
+NS_INTERFACE_MAP_END
+
+NS_IMPL_CYCLE_COLLECTING_ADDREF(XULContentSinkImpl)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(XULContentSinkImpl)
 
 //----------------------------------------------------------------------
 // nsIContentSink interface
@@ -204,7 +238,7 @@ XULContentSinkImpl::DidBuildModel(bool aTerminated)
     nsCOMPtr<nsIDocument> doc = do_QueryReferent(mDocument);
     if (doc) {
         doc->EndLoad();
-        mDocument = nsnull;
+        mDocument = nullptr;
     }
 
     // Drop our reference to the parser to get rid of a circular
@@ -260,7 +294,7 @@ nsresult
 XULContentSinkImpl::Init(nsIDocument* aDocument,
                          nsXULPrototypeDocument* aPrototype)
 {
-    NS_PRECONDITION(aDocument != nsnull, "null ptr");
+    NS_PRECONDITION(aDocument != nullptr, "null ptr");
     if (! aDocument)
         return NS_ERROR_NULL_POINTER;
     
@@ -302,9 +336,9 @@ XULContentSinkImpl::Init(nsIDocument* aDocument,
 //
 
 bool
-XULContentSinkImpl::IsDataInBuffer(PRUnichar* buffer, PRInt32 length)
+XULContentSinkImpl::IsDataInBuffer(PRUnichar* buffer, int32_t length)
 {
-    for (PRInt32 i = 0; i < length; ++i) {
+    for (int32_t i = 0; i < length; ++i) {
         if (buffer[i] == ' ' ||
             buffer[i] == '\t' ||
             buffer[i] == '\n' ||
@@ -362,7 +396,7 @@ XULContentSinkImpl::FlushText(bool aCreateTextNode)
             text->mValue.Trim(" \t\n\r");
 
         // hook it up
-        nsPrototypeArray* children = nsnull;
+        nsPrototypeArray* children = nullptr;
         rv = mContextStack.GetTopChildren(&children);
         if (NS_FAILED(rv)) return rv;
 
@@ -381,7 +415,7 @@ nsresult
 XULContentSinkImpl::NormalizeAttributeString(const PRUnichar *aExpatName,
                                              nsAttrName &aName)
 {
-    PRInt32 nameSpaceID;
+    int32_t nameSpaceID;
     nsCOMPtr<nsIAtom> prefix, localName;
     nsContentUtils::SplitExpatName(aExpatName, getter_AddRefs(prefix),
                                    getter_AddRefs(localName), &nameSpaceID);
@@ -396,8 +430,6 @@ XULContentSinkImpl::NormalizeAttributeString(const PRUnichar *aExpatName,
     ni = mNodeInfoManager->GetNodeInfo(localName, prefix,
                                        nameSpaceID,
                                        nsIDOMNode::ATTRIBUTE_NODE);
-    NS_ENSURE_TRUE(ni, NS_ERROR_OUT_OF_MEMORY);
-
     aName.SetTo(ni);
 
     return NS_OK;
@@ -423,9 +455,9 @@ XULContentSinkImpl::CreateElement(nsINodeInfo *aNodeInfo,
 NS_IMETHODIMP 
 XULContentSinkImpl::HandleStartElement(const PRUnichar *aName, 
                                        const PRUnichar **aAtts,
-                                       PRUint32 aAttsCount, 
-                                       PRInt32 aIndex, 
-                                       PRUint32 aLineNumber)
+                                       uint32_t aAttsCount, 
+                                       int32_t aIndex, 
+                                       uint32_t aLineNumber)
 { 
   // XXX Hopefully the parser will flag this before we get here. If
   // we're in the epilog, there should be no new elements
@@ -442,7 +474,7 @@ XULContentSinkImpl::HandleStartElement(const PRUnichar *aName,
       FlushText();
   }
 
-  PRInt32 nameSpaceID;
+  int32_t nameSpaceID;
   nsCOMPtr<nsIAtom> prefix, localName;
   nsContentUtils::SplitExpatName(aName, getter_AddRefs(prefix),
                                  getter_AddRefs(localName), &nameSpaceID);
@@ -450,7 +482,6 @@ XULContentSinkImpl::HandleStartElement(const PRUnichar *aName,
   nsCOMPtr<nsINodeInfo> nodeInfo;
   nodeInfo = mNodeInfoManager->GetNodeInfo(localName, prefix, nameSpaceID,
                                            nsIDOMNode::ELEMENT_NODE);
-  NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
   
   nsresult rv = NS_OK;
   switch (mState) {
@@ -506,19 +537,18 @@ XULContentSinkImpl::HandleEndElement(const PRUnichar *aName)
         FlushText();
 
         // Pop the context stack and do prototype hookup.
-        nsPrototypeArray* children = nsnull;
+        nsPrototypeArray* children = nullptr;
         rv = mContextStack.GetTopChildren(&children);
         if (NS_FAILED(rv)) return rv;
 
         nsXULPrototypeElement* element =
           static_cast<nsXULPrototypeElement*>(node.get());
 
-        PRInt32 count = children->Length();
+        int32_t count = children->Length();
         if (count) {
-            if (!element->mChildren.SetCapacity(count))
-                return NS_ERROR_OUT_OF_MEMORY;
+            element->mChildren.SetCapacity(count);
 
-            for (PRInt32 i = 0; i < count; ++i)
+            for (int32_t i = 0; i < count; ++i)
                 element->mChildren.AppendElement(children->ElementAt(i));
 
         }
@@ -530,7 +560,7 @@ XULContentSinkImpl::HandleEndElement(const PRUnichar *aName)
             static_cast<nsXULPrototypeScript*>(node.get());
 
         // If given a src= attribute, we must ignore script tag content.
-        if (! script->mSrcURI && ! script->mScriptObject.mObject) {
+        if (!script->mSrcURI && !script->GetScriptObject()) {
             nsCOMPtr<nsIDocument> doc = do_QueryReferent(mDocument);
 
             script->mOutOfLine = false;
@@ -580,7 +610,7 @@ XULContentSinkImpl::HandleComment(const PRUnichar *aName)
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleCDataSection(const PRUnichar *aData, PRUint32 aLength)
+XULContentSinkImpl::HandleCDataSection(const PRUnichar *aData, uint32_t aLength)
 {
     FlushText();
     return AddText(aData, aLength);
@@ -598,7 +628,7 @@ XULContentSinkImpl::HandleDoctypeDecl(const nsAString & aSubset,
 
 NS_IMETHODIMP 
 XULContentSinkImpl::HandleCharacterData(const PRUnichar *aData, 
-                                        PRUint32 aLength)
+                                        uint32_t aLength)
 {
   if (aData && mState != eInProlog && mState != eInEpilog) {
     return AddText(aData, aLength);
@@ -629,7 +659,7 @@ XULContentSinkImpl::HandleProcessingInstruction(const PRUnichar *aTarget,
     }
 
     nsresult rv;
-    nsPrototypeArray* children = nsnull;
+    nsPrototypeArray* children = nullptr;
     rv = mContextStack.GetTopChildren(&children);
     if (NS_FAILED(rv)) {
         return rv;
@@ -646,7 +676,7 @@ XULContentSinkImpl::HandleProcessingInstruction(const PRUnichar *aTarget,
 NS_IMETHODIMP
 XULContentSinkImpl::HandleXMLDeclaration(const PRUnichar *aVersion,
                                          const PRUnichar *aEncoding,
-                                         PRInt32 aStandalone)
+                                         int32_t aStandalone)
 {
   return NS_OK;
 }
@@ -718,7 +748,7 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
 
 nsresult
 XULContentSinkImpl::OpenRoot(const PRUnichar** aAttributes, 
-                             const PRUint32 aAttrLen, 
+                             const uint32_t aAttrLen, 
                              nsINodeInfo *aNodeInfo)
 {
     NS_ASSERTION(mState == eInProlog, "how'd we get here?");
@@ -772,8 +802,8 @@ XULContentSinkImpl::OpenRoot(const PRUnichar** aAttributes,
 
 nsresult
 XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes, 
-                            const PRUint32 aAttrLen,
-                            const PRUint32 aLineNumber,
+                            const uint32_t aAttrLen,
+                            const uint32_t aLineNumber,
                             nsINodeInfo *aNodeInfo)
 {
     nsresult rv;
@@ -798,7 +828,7 @@ XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes,
     }
 
     // Link this element to its parent.
-    nsPrototypeArray* children = nsnull;
+    nsPrototypeArray* children = nullptr;
     rv = mContextStack.GetTopChildren(&children);
     if (NS_FAILED(rv)) {
         delete element;
@@ -837,10 +867,10 @@ XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes,
 
 nsresult
 XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
-                               const PRUint32 aLineNumber)
+                               const uint32_t aLineNumber)
 {
-  PRUint32 langID = nsIProgrammingLanguage::JAVASCRIPT;
-  PRUint32 version = 0;
+  uint32_t langID = nsIProgrammingLanguage::JAVASCRIPT;
+  uint32_t version = JSVERSION_LATEST;
   nsresult rv;
 
   // Look for SRC attribute and look for a LANGUAGE attribute
@@ -865,29 +895,7 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
               NS_ENSURE_SUCCESS(rv, rv);
           }
 
-          // Javascript keeps the fast path, optimized for most-likely type
-          // Table ordered from most to least likely JS MIME types. For .xul
-          // files that we host, the likeliest type is application/x-javascript.
-          // See bug 62485, feel free to add <script type="..."> survey data to it,
-          // or to a new bug once 62485 is closed.
-          static const char *jsTypes[] = {
-              "application/x-javascript",
-              "text/javascript",
-              "text/ecmascript",
-              "application/javascript",
-              "application/ecmascript",
-              nsnull
-          };
-
-          bool isJavaScript = false;
-          for (PRInt32 i = 0; jsTypes[i]; i++) {
-              if (mimeType.LowerCaseEqualsASCII(jsTypes[i])) {
-                  isJavaScript = true;
-                  break;
-              }
-          }
-
-          if (isJavaScript) {
+          if (nsContentUtils::IsJavascriptMIMEType(mimeType)) {
               langID = nsIProgrammingLanguage::JAVASCRIPT;
               version = JSVERSION_LATEST;
           } else {
@@ -895,39 +903,14 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
           }
 
           if (langID != nsIProgrammingLanguage::UNKNOWN) {
-            // Get the version string, and ensure the language supports it.
-            nsAutoString versionName;
-            rv = parser.GetParameter("version", versionName);
-            if (NS_FAILED(rv)) {
-              if (rv != NS_ERROR_INVALID_ARG)
-                return rv;
-              // no version specified - version remains the default.
-            } else {
-              nsCOMPtr<nsIScriptRuntime> runtime;
-              rv = NS_GetJSRuntime(getter_AddRefs(runtime));
-              if (NS_FAILED(rv))
-                return rv;
-              rv = runtime->ParseVersion(versionName, &version);
-              if (NS_FAILED(rv)) {
-                NS_WARNING("This script language version is not supported - ignored");
-                langID = nsIProgrammingLanguage::UNKNOWN;
-              }
-            }
-          }
-          // Some js specifics yet to be abstracted.
-          if (langID == nsIProgrammingLanguage::JAVASCRIPT) {
-              // By default scripts in XUL documents have E4X turned on. This
-              // is still OK if version is JSVERSION_UNKNOWN (-1),
-              version = js::VersionSetMoarXML(JSVersion(version), true);
+              // Get the version string, and ensure the language supports it.
+              nsAutoString versionName;
+              rv = parser.GetParameter("version", versionName);
 
-              nsAutoString value;
-              rv = parser.GetParameter("e4x", value);
-              if (NS_FAILED(rv)) {
-                  if (rv != NS_ERROR_INVALID_ARG)
-                      return rv;
-              } else {
-                  if (value.Length() == 1 && value[0] == '0')
-                    version = js::VersionSetMoarXML(JSVersion(version), false);
+              if (NS_SUCCEEDED(rv)) {
+                  version = nsContentUtils::ParseJavascriptVersion(versionName);
+              } else if (rv != NS_ERROR_INVALID_ARG) {
+                  return rv;
               }
           }
       }
@@ -936,12 +919,9 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
           // various version strings anyway.  So we make no attempt to support
           // languages other than JS for language=
           nsAutoString lang(aAttributes[1]);
-          if (nsContentUtils::IsJavaScriptLanguage(lang, &version)) {
+          if (nsContentUtils::IsJavaScriptLanguage(lang)) {
+              version = JSVERSION_DEFAULT;
               langID = nsIProgrammingLanguage::JAVASCRIPT;
-
-              // Even when JS version < 1.6 is specified, E4X is
-              // turned on in XUL.
-              version = js::VersionSetMoarXML(JSVersion(version), true);
           }
       }
       aAttributes += 2;
@@ -965,9 +945,9 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
 
   // Don't process scripts that aren't known
   if (langID != nsIProgrammingLanguage::UNKNOWN) {
-      nsIScriptGlobalObject* globalObject = nsnull; // borrowed reference
+      nsCOMPtr<nsIScriptGlobalObject> globalObject;
       if (doc)
-          globalObject = doc->GetScriptGlobalObject();
+          globalObject = do_QueryInterface(doc->GetWindow());
       nsRefPtr<nsXULPrototypeScript> script =
           new nsXULPrototypeScript(aLineNumber, version);
       if (! script)
@@ -976,7 +956,7 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
       // If there is a SRC attribute...
       if (! src.IsEmpty()) {
           // Use the SRC attribute value to load the URL
-          rv = NS_NewURI(getter_AddRefs(script->mSrcURI), src, nsnull, mDocumentURL);
+          rv = NS_NewURI(getter_AddRefs(script->mSrcURI), src, nullptr, mDocumentURL);
 
           // Check if this document is allowed to load a script from this source
           // NOTE: if we ever allow scripts added via the DOM to run, we need to
@@ -1004,11 +984,10 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
           // file right away.  Otherwise we'll end up reloading the script and
           // corrupting the FastLoad file trying to serialize it, in the case
           // where it's already there.
-          if (globalObject)
-                script->DeserializeOutOfLine(nsnull, globalObject);
+          script->DeserializeOutOfLine(nullptr, mPrototype);
       }
 
-      nsPrototypeArray* children = nsnull;
+      nsPrototypeArray* children = nullptr;
       rv = mContextStack.GetTopChildren(&children);
       if (NS_FAILED(rv)) {
           return rv;
@@ -1027,14 +1006,14 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
 
 nsresult
 XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes, 
-                                  const PRUint32 aAttrLen, 
+                                  const uint32_t aAttrLen, 
                                   nsXULPrototypeElement* aElement)
 {
   // Add tag attributes to the element
   nsresult rv;
 
   // Create storage for the attributes
-  nsXULPrototypeAttribute* attrs = nsnull;
+  nsXULPrototypeAttribute* attrs = nullptr;
   if (aAttrLen > 0) {
     attrs = new nsXULPrototypeAttribute[aAttrLen];
     if (! attrs)
@@ -1045,7 +1024,7 @@ XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes,
   aElement->mNumAttributes = aAttrLen;
 
   // Copy the attributes into the prototype
-  PRUint32 i;
+  uint32_t i;
   for (i = 0; i < aAttrLen; ++i) {
       rv = NormalizeAttributeString(aAttributes[i * 2], attrs[i].mName);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -1057,7 +1036,7 @@ XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes,
 #ifdef PR_LOGGING
       if (PR_LOG_TEST(gLog, PR_LOG_DEBUG)) {
           nsAutoString extraWhiteSpace;
-          PRInt32 cnt = mContextStack.Depth();
+          int32_t cnt = mContextStack.Depth();
           while (--cnt >= 0)
               extraWhiteSpace.AppendLiteral("  ");
           nsAutoString qnameC,valueC;
@@ -1078,21 +1057,21 @@ XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes,
 
 nsresult
 XULContentSinkImpl::AddText(const PRUnichar* aText, 
-                            PRInt32 aLength)
+                            int32_t aLength)
 {
   // Create buffer when we first need it
   if (0 == mTextSize) {
-      mText = (PRUnichar *) PR_MALLOC(sizeof(PRUnichar) * 4096);
-      if (nsnull == mText) {
+      mText = (PRUnichar *) moz_malloc(sizeof(PRUnichar) * 4096);
+      if (nullptr == mText) {
           return NS_ERROR_OUT_OF_MEMORY;
       }
       mTextSize = 4096;
   }
 
   // Copy data from string into our buffer; flush buffer when it fills up
-  PRInt32 offset = 0;
+  int32_t offset = 0;
   while (0 != aLength) {
-    PRInt32 amount = mTextSize - mTextLength;
+    int32_t amount = mTextSize - mTextLength;
     if (amount > aLength) {
         amount = aLength;
     }
@@ -1105,8 +1084,8 @@ XULContentSinkImpl::AddText(const PRUnichar* aText,
       }
       else {
         mTextSize += aLength;
-        mText = (PRUnichar *) PR_REALLOC(mText, sizeof(PRUnichar) * mTextSize);
-        if (nsnull == mText) {
+        mText = (PRUnichar *) moz_realloc(mText, sizeof(PRUnichar) * mTextSize);
+        if (nullptr == mText) {
             return NS_ERROR_OUT_OF_MEMORY;
         }
       }
