@@ -7,16 +7,22 @@
 #include "jsfriendapi.h"
 #include "nsJSUtils.h"
 #include "nsIDOMTCPSocket.h"
+#include "nsCxPusher.h"
 #include "mozilla/unused.h"
-#include "mozilla/AppProcessPermissions.h"
+#include "mozilla/AppProcessChecker.h"
+#include "mozilla/net/NeckoCommon.h"
+#include "mozilla/net/PNeckoParent.h"
+#include "mozilla/dom/ContentParent.h"
+#include "mozilla/dom/TabParent.h"
+#include "nsIScriptSecurityManager.h"
 
 namespace IPC {
 
 //Defined in TCPSocketChild.cpp
 extern bool
-DeserializeUint8Array(JSRawObject aObj,
-                      const nsTArray<uint8_t>& aBuffer,
-                      jsval* aVal);
+DeserializeArrayBuffer(JS::Handle<JSObject*> aObj,
+                       const InfallibleTArray<uint8_t>& aBuffer,
+                       JS::MutableHandle<JS::Value> aVal);
 
 }
 
@@ -28,31 +34,75 @@ FireInteralError(mozilla::net::PTCPSocketParent* aActor, uint32_t aLineNo)
 {
   mozilla::unused <<
       aActor->SendCallback(NS_LITERAL_STRING("onerror"),
-                          JSError(NS_LITERAL_STRING("Internal error"),
-                                  NS_LITERAL_STRING(__FILE__), aLineNo, 0),
-                          NS_LITERAL_STRING("connecting"), 0);
+                           TCPError(NS_LITERAL_STRING("InvalidStateError")),
+                           NS_LITERAL_STRING("connecting"), 0);
 }
 
-NS_IMPL_CYCLE_COLLECTION_2(TCPSocketParent, mSocket, mIntermediary)
-NS_IMPL_CYCLE_COLLECTING_ADDREF(TCPSocketParent)
-NS_IMPL_CYCLE_COLLECTING_RELEASE(TCPSocketParent)
+NS_IMPL_CYCLE_COLLECTION_2(TCPSocketParentBase, mSocket, mIntermediary)
+NS_IMPL_CYCLE_COLLECTING_ADDREF(TCPSocketParentBase)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(TCPSocketParentBase)
 
-NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(TCPSocketParent)
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(TCPSocketParentBase)
   NS_INTERFACE_MAP_ENTRY(nsITCPSocketParent)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
+TCPSocketParentBase::TCPSocketParentBase()
+: mIPCOpen(false)
+{
+}
+
+TCPSocketParentBase::~TCPSocketParentBase()
+{
+}
+
+void
+TCPSocketParentBase::ReleaseIPDLReference()
+{
+  MOZ_ASSERT(mIPCOpen);
+  mIPCOpen = false;
+  this->Release();
+}
+
+void
+TCPSocketParentBase::AddIPDLReference()
+{
+  MOZ_ASSERT(!mIPCOpen);
+  mIPCOpen = true;
+  this->AddRef();
+}
+
+NS_IMETHODIMP_(nsrefcnt) TCPSocketParent::Release(void)
+{
+  nsrefcnt refcnt = TCPSocketParentBase::Release();
+  if (refcnt == 1 && mIPCOpen) {
+    mozilla::unused << PTCPSocketParent::SendRequestDelete();
+    return 1;
+  }
+  return refcnt;
+}
+
 bool
-TCPSocketParent::Init(const nsString& aHost, const uint16_t& aPort, const bool& aUseSSL,
-                      const nsString& aBinaryType, PBrowserParent* aBrowser)
+TCPSocketParent::RecvOpen(const nsString& aHost, const uint16_t& aPort, const bool& aUseSSL,
+                          const nsString& aBinaryType)
 {
   // We don't have browser actors in xpcshell, and hence can't run automated
   // tests without this loophole.
-  if (aBrowser && !AssertAppProcessPermission(aBrowser, "tcp-socket")) {
+  if (net::UsingNeckoIPCSecurity() &&
+      !AssertAppProcessPermission(Manager()->Manager(), "tcp-socket")) {
     FireInteralError(this, __LINE__);
     return true;
   }
 
+  // Obtain App ID
+  uint32_t appId = nsIScriptSecurityManager::NO_APP_ID;
+  const PContentParent *content = Manager()->Manager();
+  const InfallibleTArray<PBrowserParent*>& browsers = content->ManagedPBrowserParent();
+  if (browsers.Length() > 0) {
+    TabParent *tab = static_cast<TabParent*>(browsers[0]);
+    appId = tab->OwnAppId();
+  }
+
   nsresult rv;
   mIntermediary = do_CreateInstance("@mozilla.org/tcp-socket-intermediary;1", &rv);
   if (NS_FAILED(rv)) {
@@ -60,7 +110,8 @@ TCPSocketParent::Init(const nsString& aHost, const uint16_t& aPort, const bool&
     return true;
   }
 
-  rv = mIntermediary->Open(this, aHost, aPort, aUseSSL, aBinaryType, getter_AddRefs(mSocket));
+  rv = mIntermediary->Open(this, aHost, aPort, aUseSSL, aBinaryType, appId,
+                           getter_AddRefs(mSocket));
   if (NS_FAILED(rv) || !mSocket) {
     FireInteralError(this, __LINE__);
     return true;
@@ -77,6 +128,15 @@ TCPSocketParent::InitJS(const JS::Value& aIntermediary, JSContext* aCx)
   return NS_OK;
 }
 
+bool
+TCPSocketParent::RecvStartTLS()
+{
+  NS_ENSURE_TRUE(mSocket, true);
+  nsresult rv = mSocket->UpgradeToSecure();
+  NS_ENSURE_SUCCESS(rv, true);
+  return true;
+}
+
 bool
 TCPSocketParent::RecvSuspend()
 {
@@ -103,8 +163,11 @@ TCPSocketParent::RecvData(const SendableData& aData)
   nsresult rv;
   switch (aData.type()) {
     case SendableData::TArrayOfuint8_t: {
-      jsval val;
-      IPC::DeserializeUint8Array(mIntermediaryObj, aData.get_ArrayOfuint8_t(), &val);
+      AutoSafeJSContext cx;
+      JSAutoRequest ar(cx);
+      JS::Rooted<JS::Value> val(cx);
+      JS::Rooted<JSObject*> obj(cx, mIntermediaryObj);
+      IPC::DeserializeArrayBuffer(obj, aData.get_ArrayOfuint8_t(), &val);
       rv = mIntermediary->SendArrayBuffer(val);
       NS_ENSURE_SUCCESS(rv, true);
       break;
@@ -116,8 +179,7 @@ TCPSocketParent::RecvData(const SendableData& aData)
       break;
 
     default:
-      MOZ_NOT_REACHED();
-      return false;
+      MOZ_CRASH("unexpected SendableData type");
   }
   return true;
 }
@@ -149,17 +211,16 @@ TCPSocketParent::SendCallback(const nsAString& aType, const JS::Value& aDataVal,
       FireInteralError(this, __LINE__);
       return NS_ERROR_OUT_OF_MEMORY;
     }
-    data = str;
+    data = SendableData(str);
 
   } else if (aDataVal.isUndefined() || aDataVal.isNull()) {
     data = mozilla::void_t();
 
   } else if (aDataVal.isObject()) {
     JSObject* obj = &aDataVal.toObject();
-    if (JS_IsTypedArrayObject(obj, aCx)) {
-      NS_ENSURE_TRUE(JS_IsUint8Array(obj, aCx), NS_ERROR_FAILURE);
-      uint32_t nbytes = JS_GetTypedArrayByteLength(obj, aCx);
-      uint8_t* buffer = JS_GetUint8ArrayData(obj, aCx);
+    if (JS_IsArrayBufferObject(obj)) {
+      uint32_t nbytes = JS_GetArrayBufferByteLength(obj);
+      uint8_t* buffer = JS_GetArrayBufferData(obj);
       if (!buffer) {
         FireInteralError(this, __LINE__);
         return NS_ERROR_OUT_OF_MEMORY;
@@ -174,40 +235,18 @@ TCPSocketParent::SendCallback(const nsAString& aType, const JS::Value& aDataVal,
       data = SendableData(arr);
 
     } else {
-      nsDependentJSString message, filename;
-      uint32_t lineNumber = 0;
-      uint32_t columnNumber = 0;
+      nsDependentJSString name;
 
-      jsval val;
-      if (!JS_GetProperty(aCx, obj, "message", &val)) {
-        NS_ERROR("No message property on supposed error object");
+      JS::Rooted<JS::Value> val(aCx);
+      if (!JS_GetProperty(aCx, obj, "name", &val)) {
+        NS_ERROR("No name property on supposed error object");
       } else if (JSVAL_IS_STRING(val)) {
-        if (!message.init(aCx, JSVAL_TO_STRING(val))) {
+        if (!name.init(aCx, JSVAL_TO_STRING(val))) {
           NS_WARNING("couldn't initialize string");
         }
       }
 
-      if (!JS_GetProperty(aCx, obj, "fileName", &val)) {
-        NS_ERROR("No fileName property on supposed error object");
-      } else if (JSVAL_IS_STRING(val)) {
-        if (!filename.init(aCx, JSVAL_TO_STRING(val))) {
-          NS_WARNING("couldn't initialize string");
-        }
-      }
-
-      if (!JS_GetProperty(aCx, obj, "lineNumber", &val)) {
-        NS_ERROR("No lineNumber property on supposed error object");
-      } else if (JSVAL_IS_INT(val)) {
-        lineNumber = JSVAL_TO_INT(val);
-      }
-
-      if (!JS_GetProperty(aCx, obj, "columnNumber", &val)) {
-        NS_ERROR("No columnNumber property on supposed error object");
-      } else if (JSVAL_IS_INT(val)) {
-        columnNumber = JSVAL_TO_INT(val);
-      }
-
-      data = JSError(message, filename, lineNumber, columnNumber);
+      data = TCPError(name);
     }
   } else {
     NS_ERROR("Unexpected JS value encountered");
@@ -220,11 +259,19 @@ TCPSocketParent::SendCallback(const nsAString& aType, const JS::Value& aDataVal,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+TCPSocketParent::SetSocketAndIntermediary(nsIDOMTCPSocket *socket,
+                                          nsITCPSocketIntermediary *intermediary,
+                                          JSContext* cx)
+{
+  mSocket = socket;
+  mIntermediary = intermediary;
+  return NS_OK;
+}
+
 void
 TCPSocketParent::ActorDestroy(ActorDestroyReason why)
 {
-  MOZ_ASSERT(mIPCOpen);
-  mIPCOpen = false;
   if (mSocket) {
     mSocket->Close();
   }
