@@ -14,15 +14,17 @@
 #include "nsIThreadInternal.h"
 #include "nsPIDOMWindow.h"
 
-#include "jsapi.h"
+#include "mozilla/Assertions.h"
 #include "mozilla/CondVar.h"
-#include "mozilla/Mutex.h"
 #include "mozilla/TimeStamp.h"
-#include "nsAutoPtr.h"
-#include "nsCOMPtr.h"
+#include "mozilla/dom/BindingDeclarations.h"
+#include "nsCycleCollectionParticipant.h"
+#include "nsDataHashtable.h"
 #include "nsEventQueue.h"
-#include "nsStringGlue.h"
+#include "nsHashKeys.h"
+#include "nsString.h"
 #include "nsTArray.h"
+#include "nsThreadUtils.h"
 #include "nsTPriorityQueue.h"
 #include "StructuredCloneTags.h"
 
@@ -31,17 +33,22 @@
 #include "WorkerFeature.h"
 
 class JSAutoStructuredCloneBuffer;
+class nsIChannel;
 class nsIDocument;
 class nsIPrincipal;
 class nsIScriptContext;
-class nsIURI;
-class nsPIDOMWindow;
 class nsITimer;
-class nsIXPCScriptNotify;
+class nsIURI;
+
+namespace JS {
+class RuntimeStats;
+}
 
 BEGIN_WORKERS_NAMESPACE
 
-class WorkerMemoryReporter;
+class MessagePort;
+class SharedWorker;
+class WorkerMessagePort;
 class WorkerPrivate;
 
 class WorkerRunnable : public nsIRunnable
@@ -58,7 +65,7 @@ protected:
   ClearingBehavior mClearingBehavior;
 
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
 
   bool
   Dispatch(JSContext* aCx);
@@ -103,9 +110,7 @@ protected:
   virtual void
   PostRun(JSContext* aCx, WorkerPrivate* aWorkerPrivate, bool aRunResult);
 
-  void NotifyScriptExecutedIfNeeded() const;
-
-private:
+public:
   NS_DECL_NSIRUNNABLE
 };
 
@@ -130,7 +135,35 @@ protected:
   { }
 
   virtual bool
-  DispatchInternal();
+  DispatchInternal() MOZ_OVERRIDE;
+};
+
+class MainThreadSyncRunnable : public WorkerSyncRunnable
+{
+public:
+  MainThreadSyncRunnable(WorkerPrivate* aWorkerPrivate,
+                         ClearingBehavior aClearingBehavior,
+                         uint32_t aSyncQueueKey,
+                         bool aBypassSyncEventQueue)
+  : WorkerSyncRunnable(aWorkerPrivate, aSyncQueueKey, aBypassSyncEventQueue,
+                       aClearingBehavior)
+  {
+    AssertIsOnMainThread();
+  }
+
+  bool
+  PreDispatch(JSContext* aCx, WorkerPrivate* aWorkerPrivate) MOZ_OVERRIDE
+  {
+    AssertIsOnMainThread();
+    return true;
+  }
+
+  void
+  PostDispatch(JSContext* aCx, WorkerPrivate* aWorkerPrivate,
+               bool aDispatchResult) MOZ_OVERRIDE
+  {
+    AssertIsOnMainThread();
+  }
 };
 
 class WorkerControlRunnable : public WorkerRunnable
@@ -145,12 +178,66 @@ protected:
   { }
 
   virtual bool
-  DispatchInternal();
+  DispatchInternal() MOZ_OVERRIDE;
+};
+
+// SharedMutex is a small wrapper around an (internal) reference-counted Mutex
+// object. It exists to avoid changing a lot of code to use Mutex* instead of
+// Mutex&.
+class SharedMutex
+{
+  typedef mozilla::Mutex Mutex;
+
+  class RefCountedMutex : public Mutex
+  {
+  public:
+    RefCountedMutex(const char* aName)
+    : Mutex(aName)
+    { }
+
+    NS_INLINE_DECL_THREADSAFE_REFCOUNTING(RefCountedMutex)
+
+  private:
+    ~RefCountedMutex()
+    { }
+  };
+
+  nsRefPtr<RefCountedMutex> mMutex;
+
+public:
+  SharedMutex(const char* aName)
+  : mMutex(new RefCountedMutex(aName))
+  { }
+
+  SharedMutex(SharedMutex& aOther)
+  : mMutex(aOther.mMutex)
+  { }
+
+  operator Mutex&()
+  {
+    MOZ_ASSERT(mMutex);
+    return *mMutex;
+  }
+
+  operator const Mutex&() const
+  {
+    MOZ_ASSERT(mMutex);
+    return *mMutex;
+  }
+
+  void
+  AssertCurrentThreadOwns() const
+  {
+    MOZ_ASSERT(mMutex);
+    mMutex->AssertCurrentThreadOwns();
+  }
 };
 
 template <class Derived>
 class WorkerPrivateParent : public EventTarget
 {
+  class SynchronizeAndResumeRunnable;
+
 public:
   struct LocationInfo
   {
@@ -164,52 +251,90 @@ public:
     nsCString mHash;
   };
 
+  struct LoadInfo
+  {
+    // All of these should be released in ForgetMainThreadObjects.
+    nsCOMPtr<nsIURI> mBaseURI;
+    nsCOMPtr<nsIURI> mResolvedScriptURI;
+    nsCOMPtr<nsIPrincipal> mPrincipal;
+    nsCOMPtr<nsIScriptContext> mScriptContext;
+    nsCOMPtr<nsPIDOMWindow> mWindow;
+    nsCOMPtr<nsIContentSecurityPolicy> mCSP;
+    nsCOMPtr<nsIChannel> mChannel;
+
+    nsCString mDomain;
+
+    bool mEvalAllowed;
+    bool mReportCSPViolations;
+    bool mXHRParamsAllowed;
+    bool mPrincipalIsSystem;
+
+    LoadInfo()
+    : mEvalAllowed(false), mReportCSPViolations(false),
+      mXHRParamsAllowed(false), mPrincipalIsSystem(false)
+    { }
+
+    void
+    StealFrom(LoadInfo& aOther)
+    {
+      mBaseURI = aOther.mBaseURI.forget();
+      mResolvedScriptURI = aOther.mResolvedScriptURI.forget();
+      mPrincipal = aOther.mPrincipal.forget();
+      mScriptContext = aOther.mScriptContext.forget();
+      mWindow = aOther.mWindow.forget();
+      mCSP = aOther.mCSP.forget();
+      mChannel = aOther.mChannel.forget();
+      mDomain = aOther.mDomain;
+      mEvalAllowed = aOther.mEvalAllowed;
+      mReportCSPViolations = aOther.mReportCSPViolations;
+      mXHRParamsAllowed = aOther.mXHRParamsAllowed;
+      mPrincipalIsSystem = aOther.mPrincipalIsSystem;
+    }
+  };
+
 protected:
-  mozilla::Mutex mMutex;
+  typedef mozilla::ErrorResult ErrorResult;
+
+  SharedMutex mMutex;
   mozilla::CondVar mCondVar;
+  mozilla::CondVar mMemoryReportCondVar;
 
 private:
   JSObject* mJSObject;
   WorkerPrivate* mParent;
-  JSContext* mParentJSContext;
   nsString mScriptURL;
-  nsCString mDomain;
+  nsString mSharedWorkerName;
   LocationInfo mLocationInfo;
-
-  // Main-thread things.
-  nsCOMPtr<nsPIDOMWindow> mWindow;
-  nsCOMPtr<nsIScriptContext> mScriptContext;
-  nsCOMPtr<nsIXPCScriptNotify> mScriptNotify;
-  nsCOMPtr<nsIURI> mBaseURI;
-  nsCOMPtr<nsIURI> mScriptURI;
-  nsCOMPtr<nsIPrincipal> mPrincipal;
-  nsCOMPtr<nsIContentSecurityPolicy> mCSP;
+  LoadInfo mLoadInfo;
 
   // Only used for top level workers.
   nsTArray<nsRefPtr<WorkerRunnable> > mQueuedRunnables;
+  nsRevocableEventPtr<SynchronizeAndResumeRunnable> mSynchronizeRunnable;
+
+  // Only for ChromeWorkers without window and only touched on the main thread.
+  nsTArray<nsCString> mHostObjectURIs;
+
+  // Protected by mMutex.
+  JSSettings mJSSettings;
+
+  // Only touched on the parent thread (currently this is always the main
+  // thread as SharedWorkers are always top-level).
+  nsDataHashtable<nsUint64HashKey, SharedWorker*> mSharedWorkers;
 
   uint64_t mBusyCount;
+  uint64_t mMessagePortSerial;
   Status mParentStatus;
-  uint32_t mJSContextOptions;
-  uint32_t mJSRuntimeHeapSize;
-  uint8_t mGCZeal;
   bool mJSObjectRooted;
   bool mParentSuspended;
   bool mIsChromeWorker;
-  bool mPrincipalIsSystem;
   bool mMainThreadObjectsForgotten;
-  bool mEvalAllowed;
+  bool mIsSharedWorker;
 
 protected:
-  WorkerPrivateParent(JSContext* aCx, JSObject* aObject, WorkerPrivate* aParent,
-                      JSContext* aParentJSContext, const nsAString& aScriptURL,
-                      bool aIsChromeWorker, const nsACString& aDomain,
-                      nsCOMPtr<nsPIDOMWindow>& aWindow,
-                      nsCOMPtr<nsIScriptContext>& aScriptContext,
-                      nsCOMPtr<nsIURI>& aBaseURI,
-                      nsCOMPtr<nsIPrincipal>& aPrincipal,
-                      nsCOMPtr<nsIContentSecurityPolicy>& aCSP,
-                      bool aEvalAllowed);
+  WorkerPrivateParent(JSContext* aCx, JS::HandleObject aObject,
+                      WorkerPrivate* aParent, const nsAString& aScriptURL,
+                      bool aIsChromeWorker, bool aIsSharedWorker,
+                      const nsAString& aSharedWorkerName, LoadInfo& aLoadInfo);
 
   ~WorkerPrivateParent();
 
@@ -231,6 +356,11 @@ private:
     return NotifyPrivate(aCx, Terminating);
   }
 
+  bool
+  PostMessageInternal(JSContext* aCx, JS::Handle<JS::Value> aMessage,
+                      JS::Handle<JS::Value> aTransferable,
+                      bool aToMessagePort, uint64_t aMessagePortSerial);
+
 public:
   // May be called on any thread...
   bool
@@ -256,10 +386,14 @@ public:
   }
 
   bool
-  Suspend(JSContext* aCx);
+  Suspend(JSContext* aCx, nsPIDOMWindow* aWindow);
 
   bool
-  Resume(JSContext* aCx);
+  Resume(JSContext* aCx, nsPIDOMWindow* aWindow);
+
+  bool
+  SynchronizeAndResume(JSContext* aCx, nsPIDOMWindow* aWindow,
+                       nsIScriptContext* aScriptContext);
 
   virtual void
   _trace(JSTracer* aTrc) MOZ_OVERRIDE;
@@ -294,25 +428,66 @@ public:
   ForgetMainThreadObjects(nsTArray<nsCOMPtr<nsISupports> >& aDoomed);
 
   bool
-  PostMessage(JSContext* aCx, jsval aMessage, jsval aTransferable);
+  PostMessage(JSContext* aCx, JS::Handle<JS::Value> aMessage,
+              JS::Handle<JS::Value> aTransferable)
+  {
+    return PostMessageInternal(aCx, aMessage, aTransferable, false, 0);
+  }
+
+  void
+  PostMessageToMessagePort(JSContext* aCx,
+                           uint64_t aMessagePortSerial,
+                           JS::Handle<JS::Value> aMessage,
+                           const Optional<Sequence<JS::Value > >& aTransferable,
+                           ErrorResult& aRv);
+
+  bool
+  DispatchMessageEventToMessagePort(
+                               JSContext* aCx,
+                               uint64_t aMessagePortSerial,
+                               JSAutoStructuredCloneBuffer& aBuffer,
+                               nsTArray<nsCOMPtr<nsISupports>>& aClonedObjects);
 
   uint64_t
   GetInnerWindowId();
 
   void
-  UpdateJSContextOptions(JSContext* aCx, uint32_t aOptions);
+  UpdateJSContextOptions(JSContext* aCx, const JS::ContextOptions& aChromeOptions,
+                         const JS::ContextOptions& aContentOptions);
 
   void
-  UpdateJSRuntimeHeapSize(JSContext* aCx, uint32_t aJSRuntimeHeapSize);
+  UpdateJSWorkerMemoryParameter(JSContext* aCx, JSGCParamKey key,
+                                uint32_t value);
 
 #ifdef JS_GC_ZEAL
   void
-  UpdateGCZeal(JSContext* aCx, uint8_t aGCZeal);
+  UpdateGCZeal(JSContext* aCx, uint8_t aGCZeal, uint32_t aFrequency);
 #endif
 
+  void
+  UpdateJITHardening(JSContext* aCx, bool aJITHardening);
+
   void
   GarbageCollect(JSContext* aCx, bool aShrinking);
 
+  bool
+  RegisterSharedWorker(JSContext* aCx, SharedWorker* aSharedWorker);
+
+  void
+  UnregisterSharedWorker(JSContext* aCx, SharedWorker* aSharedWorker);
+
+  void
+  BroadcastErrorToSharedWorkers(JSContext* aCx,
+                                const nsAString& aMessage,
+                                const nsAString& aFilename,
+                                const nsAString& aLine,
+                                uint32_t aLineNumber,
+                                uint32_t aColumnNumber,
+                                uint32_t aFlags);
+
+  void
+  WorkerScriptLoaded();
+
   void
   QueueRunnable(WorkerRunnable* aRunnable)
   {
@@ -338,9 +513,10 @@ public:
   {
     AssertIsOnParentThread();
     bool acceptingEvents;
-    mMutex.Lock();
-    acceptingEvents = mParentStatus < Terminating;
-    mMutex.Unlock();
+    {
+      mozilla::MutexAutoLock lock(mMutex);
+      acceptingEvents = mParentStatus < Terminating;
+    }
     return acceptingEvents;
   }
 
@@ -358,14 +534,7 @@ public:
   GetScriptContext() const
   {
     AssertIsOnMainThread();
-    return mScriptContext;
-  }
-
-  nsIXPCScriptNotify*
-  GetScriptNotify() const
-  {
-    AssertIsOnMainThread();
-    return mScriptNotify;
+    return mLoadInfo.mScriptContext;
   }
 
   JSObject*
@@ -383,38 +552,31 @@ public:
   const nsCString&
   Domain() const
   {
-    return mDomain;
+    return mLoadInfo.mDomain;
   }
 
   nsIURI*
   GetBaseURI() const
   {
     AssertIsOnMainThread();
-    return mBaseURI;
+    return mLoadInfo.mBaseURI;
   }
 
   void
   SetBaseURI(nsIURI* aBaseURI);
 
   nsIURI*
-  GetScriptURI() const
+  GetResolvedScriptURI() const
   {
     AssertIsOnMainThread();
-    return mScriptURI;
-  }
-
-  void
-  SetScriptURI(nsIURI* aScriptURI)
-  {
-    AssertIsOnMainThread();
-    mScriptURI = aScriptURI;
+    return mLoadInfo.mResolvedScriptURI;
   }
 
   nsIPrincipal*
   GetPrincipal() const
   {
     AssertIsOnMainThread();
-    return mPrincipal;
+    return mLoadInfo.mPrincipal;
   }
 
   void
@@ -423,47 +585,72 @@ public:
   bool
   UsesSystemPrincipal() const
   {
-    return mPrincipalIsSystem;
+    return mLoadInfo.mPrincipalIsSystem;
+  }
+
+  already_AddRefed<nsIChannel>
+  ForgetWorkerChannel()
+  {
+    AssertIsOnMainThread();
+    return mLoadInfo.mChannel.forget();
   }
 
   nsIDocument*
   GetDocument() const
   {
     AssertIsOnMainThread();
-    return mWindow ? mWindow->GetExtantDoc() : nullptr;
+    return mLoadInfo.mWindow ? mLoadInfo.mWindow->GetExtantDoc() : nullptr;
   }
 
   nsPIDOMWindow*
   GetWindow()
   {
     AssertIsOnMainThread();
-    return mWindow;
+    return mLoadInfo.mWindow;
   }
 
   nsIContentSecurityPolicy*
   GetCSP() const
   {
     AssertIsOnMainThread();
-    return mCSP;
+    return mLoadInfo.mCSP;
   }
 
   void
   SetCSP(nsIContentSecurityPolicy* aCSP)
   {
     AssertIsOnMainThread();
-    mCSP = aCSP;
+    mLoadInfo.mCSP = aCSP;
   }
 
   bool
   IsEvalAllowed() const
   {
-    return mEvalAllowed;
+    return mLoadInfo.mEvalAllowed;
   }
 
   void
   SetEvalAllowed(bool aEvalAllowed)
   {
-    mEvalAllowed = aEvalAllowed;
+    mLoadInfo.mEvalAllowed = aEvalAllowed;
+  }
+
+  bool
+  GetReportCSPViolations() const
+  {
+    return mLoadInfo.mReportCSPViolations;
+  }
+
+  bool
+  XHRParamsAllowed() const
+  {
+    return mLoadInfo.mXHRParamsAllowed;
+  }
+
+  void
+  SetXHRParamsAllowed(bool aAllowed)
+  {
+    mLoadInfo.mXHRParamsAllowed = aAllowed;
   }
 
   LocationInfo&
@@ -472,32 +659,56 @@ public:
     return mLocationInfo;
   }
 
-  uint32_t
-  GetJSContextOptions() const
+  void
+  CopyJSSettings(JSSettings& aSettings)
   {
-    return mJSContextOptions;
+    mozilla::MutexAutoLock lock(mMutex);
+    aSettings = mJSSettings;
   }
 
-  uint32_t
-  GetJSRuntimeHeapSize() const
-  {
-    return mJSRuntimeHeapSize;
-  }
-
-#ifdef JS_GC_ZEAL
-  uint8_t
-  GetGCZeal() const
-  {
-    return mGCZeal;
-  }
-#endif
-
   bool
   IsChromeWorker() const
   {
     return mIsChromeWorker;
   }
 
+  bool
+  IsSharedWorker() const
+  {
+    return mIsSharedWorker;
+  }
+
+  const nsString&
+  SharedWorkerName() const
+  {
+    return mSharedWorkerName;
+  }
+
+  uint64_t
+  NextMessagePortSerial()
+  {
+    AssertIsOnMainThread();
+    return mMessagePortSerial++;
+  }
+
+  void
+  GetAllSharedWorkers(nsTArray<nsRefPtr<SharedWorker>>& aSharedWorkers);
+
+  void
+  CloseSharedWorkersForWindow(nsPIDOMWindow* aWindow);
+
+  void
+  RegisterHostObjectURI(const nsACString& aURI);
+
+  void
+  UnregisterHostObjectURI(const nsACString& aURI);
+
+  void
+  StealHostObjectURIs(nsTArray<nsCString>& aArray);
+
+  virtual JSObject*
+  WrapObject(JSContext* aCx, JS::HandleObject aScope) MOZ_OVERRIDE;
+
 #ifdef DEBUG
   void
   AssertIsOnParentThread() const;
@@ -518,7 +729,6 @@ public:
 class WorkerPrivate : public WorkerPrivateParent<WorkerPrivate>
 {
   friend class WorkerPrivateParent<WorkerPrivate>;
-  friend class WorkerMemoryReporter;
   typedef WorkerPrivateParent<WorkerPrivate> ParentType;
 
   struct TimeoutInfo;
@@ -546,6 +756,9 @@ class WorkerPrivate : public WorkerPrivateParent<WorkerPrivate>
     }
   };
 
+  class MemoryReporter;
+  friend class MemoryReporter;
+
   nsTArray<nsAutoPtr<SyncQueue> > mSyncQueues;
 
   // Touched on multiple threads, protected with mMutex.
@@ -558,7 +771,9 @@ class WorkerPrivate : public WorkerPrivateParent<WorkerPrivate>
   nsTArray<nsAutoPtr<TimeoutInfo> > mTimeouts;
 
   nsCOMPtr<nsITimer> mTimer;
-  nsRefPtr<WorkerMemoryReporter> mMemoryReporter;
+  nsRefPtr<MemoryReporter> mMemoryReporter;
+
+  nsDataHashtable<nsUint64HashKey, WorkerMessagePort*> mWorkerPorts;
 
   mozilla::TimeStamp mKillTime;
   uint32_t mErrorHandlerRecursionCount;
@@ -570,7 +785,7 @@ class WorkerPrivate : public WorkerPrivateParent<WorkerPrivate>
   bool mCloseHandlerStarted;
   bool mCloseHandlerFinished;
   bool mMemoryReporterRunning;
-  bool mXHRParamsAllowed;
+  bool mBlockedForMemoryReporter;
 
 #ifdef DEBUG
   nsCOMPtr<nsIThread> mThread;
@@ -580,8 +795,15 @@ public:
   ~WorkerPrivate();
 
   static already_AddRefed<WorkerPrivate>
-  Create(JSContext* aCx, JSObject* aObj, WorkerPrivate* aParent,
-         JSString* aScriptURL, bool aIsChromeWorker);
+  Create(JSContext* aCx, JS::HandleObject aObject, WorkerPrivate* aParent,
+         const nsAString& aScriptURL, bool aIsChromeWorker,
+         bool aIsSharedWorker, const nsAString& aSharedWorkerName,
+         LoadInfo* aLoadInfo = nullptr);
+
+  static nsresult
+  GetLoadInfo(JSContext* aCx, nsPIDOMWindow* aWindow, WorkerPrivate* aParent,
+              const nsAString& aScriptURL, bool aIsChromeWorker,
+              LoadInfo* aLoadInfo);
 
   void
   DoRunLoop(JSContext* aCx);
@@ -665,8 +887,20 @@ public:
   DestroySyncLoop(uint32_t aSyncLoopKey);
 
   bool
-  PostMessageToParent(JSContext* aCx, jsval aMessage,
-                      jsval transferable);
+  PostMessageToParent(JSContext* aCx,
+                      JS::Handle<JS::Value> aMessage,
+                      JS::Handle<JS::Value> aTransferable)
+  {
+    return PostMessageToParentInternal(aCx, aMessage, aTransferable, false, 0);
+  }
+
+  void
+  PostMessageToParentMessagePort(
+                             JSContext* aCx,
+                             uint64_t aMessagePortSerial,
+                             JS::HandleValue aMessage,
+                             const Optional<Sequence<JS::Value>>& aTransferable,
+                             ErrorResult& aRv);
 
   bool
   NotifyInternal(JSContext* aCx, Status aStatus);
@@ -701,37 +935,26 @@ public:
   }
 
   void
-  UpdateJSContextOptionsInternal(JSContext* aCx, uint32_t aOptions);
+  UpdateJSContextOptionsInternal(JSContext* aCx, const JS::ContextOptions& aContentOptions,
+                                 const JS::ContextOptions& aChromeOptions);
 
   void
-  UpdateJSRuntimeHeapSizeInternal(JSContext* aCx, uint32_t aJSRuntimeHeapSize);
+  UpdateJSWorkerMemoryParameterInternal(JSContext* aCx, JSGCParamKey key, uint32_t aValue);
 
   void
   ScheduleDeletion(bool aWasPending);
 
   bool
-  BlockAndCollectRuntimeStats(bool isQuick, void* aData);
-
-  bool
-  DisableMemoryReporter();
-
-  bool
-  XHRParamsAllowed() const
-  {
-    return mXHRParamsAllowed;
-  }
-
-  void
-  SetXHRParamsAllowed(bool aAllowed)
-  {
-    mXHRParamsAllowed = aAllowed;
-  }
+  BlockAndCollectRuntimeStats(JS::RuntimeStats* aRtStats);
 
 #ifdef JS_GC_ZEAL
   void
-  UpdateGCZealInternal(JSContext* aCx, uint8_t aGCZeal);
+  UpdateGCZealInternal(JSContext* aCx, uint8_t aGCZeal, uint32_t aFrequency);
 #endif
 
+  void
+  UpdateJITHardeningInternal(JSContext* aCx, bool aJITHardening);
+
   void
   GarbageCollectInternal(JSContext* aCx, bool aShrinking,
                          bool aCollectChildren);
@@ -761,19 +984,44 @@ public:
   WorkerCrossThreadDispatcher*
   GetCrossThreadDispatcher();
 
-private:
-  WorkerPrivate(JSContext* aCx, JSObject* aObject, WorkerPrivate* aParent,
-                JSContext* aParentJSContext, const nsAString& aScriptURL,
-                bool aIsChromeWorker, const nsACString& aDomain,
-                nsCOMPtr<nsPIDOMWindow>& aWindow,
-                nsCOMPtr<nsIScriptContext>& aScriptContext,
-                nsCOMPtr<nsIURI>& aBaseURI, nsCOMPtr<nsIPrincipal>& aPrincipal,
-                nsCOMPtr<nsIContentSecurityPolicy>& aCSP, bool aEvalAllowed,
-                bool aXHRParamsAllowed);
+  // This may block!
+  void
+  BeginCTypesCall();
 
-  static bool
-  GetContentSecurityPolicy(JSContext *aCx,
-                           nsIContentSecurityPolicy** aCsp);
+  // This may block!
+  void
+  EndCTypesCall();
+
+  void
+  BeginCTypesCallback()
+  {
+    // If a callback is beginning then we need to do the exact same thing as
+    // when a ctypes call ends.
+    EndCTypesCall();
+  }
+
+  void
+  EndCTypesCallback()
+  {
+    // If a callback is ending then we need to do the exact same thing as
+    // when a ctypes call begins.
+    BeginCTypesCall();
+  }
+
+  bool
+  ConnectMessagePort(JSContext* aCx, uint64_t aMessagePortSerial);
+
+  void
+  DisconnectMessagePort(uint64_t aMessagePortSerial);
+
+  WorkerMessagePort*
+  GetMessagePort(uint64_t aMessagePortSerial);
+
+private:
+  WorkerPrivate(JSContext* aCx, JS::HandleObject aObject,
+                WorkerPrivate* aParent, const nsAString& aScriptURL,
+                bool aIsChromeWorker, bool aIsSharedWorker,
+                const nsAString& aSharedWorkerName, LoadInfo& aLoadInfo);
 
   bool
   Dispatch(WorkerRunnable* aEvent, EventQueue* aQueue);
@@ -830,13 +1078,37 @@ private:
   bool
   ProcessAllControlRunnables();
 
-  static bool
-  CheckXHRParamsAllowed(nsPIDOMWindow* aWindow);
+  void
+  EnableMemoryReporter();
+
+  void
+  DisableMemoryReporter();
+
+  void
+  WaitForWorkerEvents(PRIntervalTime interval = PR_INTERVAL_NO_TIMEOUT);
+
+  static PLDHashOperator
+  TraceMessagePorts(const uint64_t& aKey,
+                    WorkerMessagePort* aData,
+                    void* aUserArg);
+
+  bool
+  PostMessageToParentInternal(JSContext* aCx,
+                              JS::Handle<JS::Value> aMessage,
+                              JS::Handle<JS::Value> aTransferable,
+                              bool aToMessagePort,
+                              uint64_t aMessagePortSerial);
 };
 
 WorkerPrivate*
 GetWorkerPrivateFromContext(JSContext* aCx);
 
+bool
+IsCurrentThreadRunningChromeWorker();
+
+JSContext*
+GetCurrentThreadJSContext();
+
 enum WorkerStructuredDataType
 {
   DOMWORKER_SCTAG_FILE = SCTAG_DOM_MAX,
