@@ -6,6 +6,9 @@
 
 #include <string.h>
 
+#include "mozilla/dom/DocumentFragment.h"
+#include "mozilla/Base64.h"
+#include "mozilla/BasicEvents.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Selection.h"
 #include "mozilla/Util.h"
@@ -24,7 +27,6 @@
 #include "nsEditor.h"
 #include "nsEditorUtils.h"
 #include "nsError.h"
-#include "nsGUIEvent.h"
 #include "nsGkAtoms.h"
 #include "nsHTMLEditUtils.h"
 #include "nsHTMLEditor.h"
@@ -38,7 +40,6 @@
 #include "nsIDOMDocumentFragment.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMHTMLAnchorElement.h"
-#include "nsIDOMHTMLBodyElement.h"
 #include "nsIDOMHTMLEmbedElement.h"
 #include "nsIDOMHTMLFrameElement.h"
 #include "nsIDOMHTMLIFrameElement.h"
@@ -47,9 +48,6 @@
 #include "nsIDOMHTMLLinkElement.h"
 #include "nsIDOMHTMLObjectElement.h"
 #include "nsIDOMHTMLScriptElement.h"
-#include "nsIDOMHTMLTableCellElement.h"
-#include "nsIDOMHTMLTableElement.h"
-#include "nsIDOMHTMLTableRowElement.h"
 #include "nsIDOMNode.h"
 #include "nsIDOMRange.h"
 #include "nsIDocument.h"
@@ -89,16 +87,13 @@
 #include "nsWSRunObject.h"
 #include "nsXPCOM.h"
 #include "nscore.h"
-#include "plbase64.h"
-#include "prmem.h"
 
 class nsIAtom;
 class nsILoadContext;
 class nsISupports;
 
 using namespace mozilla;
-
-const PRUnichar nbsp = 160;
+using namespace mozilla::dom;
 
 #define kInsertCookie  "_moz_Insert Here_moz_"
 
@@ -157,6 +152,8 @@ NS_IMETHODIMP nsHTMLEditor::LoadHTML(const nsAString & aInputString)
 
   nsTextRulesInfo ruleInfo(EditAction::loadHTML);
   bool cancel, handled;
+  // Protect the edit rules object from dying
+  nsCOMPtr<nsIEditRules> kungFuDeathGrip(mRules);
   nsresult rv = mRules->WillDoAction(selection, &ruleInfo, &cancel, &handled);
   NS_ENSURE_SUCCESS(rv, rv);
   if (cancel) {
@@ -227,7 +224,7 @@ nsHTMLEditor::InsertHTMLWithContext(const nsAString & aInputString,
 {
   return DoInsertHTMLWithContext(aInputString, aContextStr, aInfoStr,
       aFlavor, aSourceDoc, aDestNode, aDestOffset, aDeleteSelection,
-      true);
+      /* trusted input */ true, /* clear style */ false);
 }
 
 nsresult
@@ -239,7 +236,8 @@ nsHTMLEditor::DoInsertHTMLWithContext(const nsAString & aInputString,
                                       nsIDOMNode *aDestNode,
                                       int32_t aDestOffset,
                                       bool aDeleteSelection,
-                                      bool aTrustedInput)
+                                      bool aTrustedInput,
+                                      bool aClearStyle)
 {
   NS_ENSURE_TRUE(mRules, NS_ERROR_NOT_INITIALIZED);
 
@@ -367,12 +365,14 @@ nsHTMLEditor::DoInsertHTMLWithContext(const nsAString & aInputString,
     rv = DeleteSelectionAndPrepareToCreateNode();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    // pasting does not inherit local inline styles
-    nsCOMPtr<nsIDOMNode> tmpNode =
-      do_QueryInterface(selection->GetAnchorNode());
-    int32_t tmpOffset = selection->GetAnchorOffset();
-    rv = ClearStyle(address_of(tmpNode), &tmpOffset, nullptr, nullptr);
-    NS_ENSURE_SUCCESS(rv, rv);
+    if (aClearStyle) {
+      // pasting does not inherit local inline styles
+      nsCOMPtr<nsIDOMNode> tmpNode =
+        do_QueryInterface(selection->GetAnchorNode());
+      int32_t tmpOffset = selection->GetAnchorOffset();
+      rv = ClearStyle(address_of(tmpNode), &tmpOffset, nullptr, nullptr);
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
   }
   else
   {
@@ -716,141 +716,6 @@ nsHTMLEditor::DoInsertHTMLWithContext(const nsAString & aInputString,
   return mRules->DidDoAction(selection, &ruleInfo, rv);
 }
 
-// returns empty string if nothing to modify on node
-nsresult
-nsHTMLEditor::GetAttributeToModifyOnNode(nsIDOMNode *aNode, nsAString &aAttr)
-{
-  aAttr.Truncate();
-
-  NS_NAMED_LITERAL_STRING(srcStr, "src");
-  nsCOMPtr<nsIDOMHTMLImageElement> nodeAsImage = do_QueryInterface(aNode);
-  if (nodeAsImage)
-  {
-    aAttr = srcStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLAnchorElement> nodeAsAnchor = do_QueryInterface(aNode);
-  if (nodeAsAnchor)
-  {
-    aAttr.AssignLiteral("href");
-    return NS_OK;
-  }
-
-  NS_NAMED_LITERAL_STRING(bgStr, "background");
-  nsCOMPtr<nsIDOMHTMLBodyElement> nodeAsBody = do_QueryInterface(aNode);
-  if (nodeAsBody)
-  {
-    aAttr = bgStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLTableElement> nodeAsTable = do_QueryInterface(aNode);
-  if (nodeAsTable)
-  {
-    aAttr = bgStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLTableRowElement> nodeAsTableRow = do_QueryInterface(aNode);
-  if (nodeAsTableRow)
-  {
-    aAttr = bgStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLTableCellElement> nodeAsTableCell = do_QueryInterface(aNode);
-  if (nodeAsTableCell)
-  {
-    aAttr = bgStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLScriptElement> nodeAsScript = do_QueryInterface(aNode);
-  if (nodeAsScript)
-  {
-    aAttr = srcStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLEmbedElement> nodeAsEmbed = do_QueryInterface(aNode);
-  if (nodeAsEmbed)
-  {
-    aAttr = srcStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLObjectElement> nodeAsObject = do_QueryInterface(aNode);
-  if (nodeAsObject)
-  {
-    aAttr.AssignLiteral("data");
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLLinkElement> nodeAsLink = do_QueryInterface(aNode);
-  if (nodeAsLink)
-  {
-    // Test if the link has a rel value indicating it to be a stylesheet
-    nsAutoString linkRel;
-    if (NS_SUCCEEDED(nodeAsLink->GetRel(linkRel)) && !linkRel.IsEmpty())
-    {
-      nsReadingIterator<PRUnichar> start;
-      nsReadingIterator<PRUnichar> end;
-      nsReadingIterator<PRUnichar> current;
-
-      linkRel.BeginReading(start);
-      linkRel.EndReading(end);
-
-      // Walk through space delimited string looking for "stylesheet"
-      for (current = start; current != end; ++current)
-      {
-        // Ignore whitespace
-        if (nsCRT::IsAsciiSpace(*current))
-          continue;
-
-        // Grab the next space delimited word
-        nsReadingIterator<PRUnichar> startWord = current;
-        do {
-          ++current;
-        } while (current != end && !nsCRT::IsAsciiSpace(*current));
-
-        // Store the link for fix up if it says "stylesheet"
-        if (Substring(startWord, current).LowerCaseEqualsLiteral("stylesheet"))
-        {
-          aAttr.AssignLiteral("href");
-          return NS_OK;
-        }
-        if (current == end)
-          break;
-      }
-    }
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLFrameElement> nodeAsFrame = do_QueryInterface(aNode);
-  if (nodeAsFrame)
-  {
-    aAttr = srcStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLIFrameElement> nodeAsIFrame = do_QueryInterface(aNode);
-  if (nodeAsIFrame)
-  {
-    aAttr = srcStr;
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIDOMHTMLInputElement> nodeAsInput = do_QueryInterface(aNode);
-  if (nodeAsInput)
-  {
-    aAttr = srcStr;
-    return NS_OK;
-  }
-
-  return NS_OK;
-}
-
 nsresult
 nsHTMLEditor::AddInsertionListener(nsIContentFilter *aListener)
 {
@@ -1228,14 +1093,15 @@ nsresult nsHTMLEditor::InsertObject(const char* aType, nsISupports* aObject, boo
     rv = imageStream->Close();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    char * base64 = PL_Base64Encode(imageData.get(), imageData.Length(), nullptr);
-    NS_ENSURE_TRUE(base64, NS_ERROR_OUT_OF_MEMORY);
+    nsAutoCString data64;
+    rv = Base64Encode(imageData, data64);
+    NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoString stuffToPaste;
     stuffToPaste.AssignLiteral("<IMG src=\"data:");
     AppendUTF8toUTF16(type, stuffToPaste);
     stuffToPaste.AppendLiteral(";base64,");
-    AppendUTF8toUTF16(base64, stuffToPaste);
+    AppendUTF8toUTF16(data64, stuffToPaste);
     stuffToPaste.AppendLiteral("\" alt=\"\" >");
     nsAutoEditBatch beginBatching(this);
     rv = DoInsertHTMLWithContext(stuffToPaste, EmptyString(), EmptyString(), 
@@ -1244,7 +1110,6 @@ nsresult nsHTMLEditor::InsertObject(const char* aType, nsISupports* aObject, boo
                                  aDestinationNode, aDestOffset,
                                  aDoDeleteSelection,
                                  aIsSafe);
-    PR_Free(base64);
   }
 
   return NS_OK;
@@ -1474,7 +1339,7 @@ bool nsHTMLEditor::HavePrivateHTMLFlavor(nsIClipboard *aClipboard)
 
 NS_IMETHODIMP nsHTMLEditor::Paste(int32_t aSelectionType)
 {
-  if (!FireClipboardEvent(NS_PASTE))
+  if (!FireClipboardEvent(NS_PASTE, aSelectionType))
     return NS_OK;
 
   // Get Clipboard Service
@@ -1555,7 +1420,9 @@ NS_IMETHODIMP nsHTMLEditor::Paste(int32_t aSelectionType)
 
 NS_IMETHODIMP nsHTMLEditor::PasteTransferable(nsITransferable *aTransferable)
 {
-  if (!FireClipboardEvent(NS_PASTE))
+  // Use an invalid value for the clipboard type as data comes from aTransferable
+  // and we don't currently implement a way to put that in the data transfer yet.
+  if (!FireClipboardEvent(NS_PASTE, nsIClipboard::kGlobalClipboard))
     return NS_OK;
 
   // handle transferable hooks
@@ -1573,7 +1440,7 @@ NS_IMETHODIMP nsHTMLEditor::PasteTransferable(nsITransferable *aTransferable)
 //
 NS_IMETHODIMP nsHTMLEditor::PasteNoFormatting(int32_t aSelectionType)
 {
-  if (!FireClipboardEvent(NS_PASTE))
+  if (!FireClipboardEvent(NS_PASTE, aSelectionType))
     return NS_OK;
 
   ForceCompositionEnd();
@@ -1713,6 +1580,8 @@ NS_IMETHODIMP nsHTMLEditor::PasteAsCitedQuotation(const nsAString & aCitation,
   // give rules a chance to handle or cancel
   nsTextRulesInfo ruleInfo(EditAction::insertElement);
   bool cancel, handled;
+  // Protect the edit rules object from dying
+  nsCOMPtr<nsIEditRules> kungFuDeathGrip(mRules);
   nsresult rv = mRules->WillDoAction(selection, &ruleInfo, &cancel, &handled);
   NS_ENSURE_SUCCESS(rv, rv);
   if (cancel || handled) {
@@ -1917,6 +1786,8 @@ nsHTMLEditor::InsertAsPlaintextQuotation(const nsAString & aQuotedText,
   // give rules a chance to handle or cancel
   nsTextRulesInfo ruleInfo(EditAction::insertElement);
   bool cancel, handled;
+  // Protect the edit rules object from dying
+  nsCOMPtr<nsIEditRules> kungFuDeathGrip(mRules);
   nsresult rv = mRules->WillDoAction(selection, &ruleInfo, &cancel, &handled);
   NS_ENSURE_SUCCESS(rv, rv);
   if (cancel || handled) {
@@ -2010,6 +1881,8 @@ nsHTMLEditor::InsertAsCitedQuotation(const nsAString & aQuotedText,
   // give rules a chance to handle or cancel
   nsTextRulesInfo ruleInfo(EditAction::insertElement);
   bool cancel, handled;
+  // Protect the edit rules object from dying
+  nsCOMPtr<nsIEditRules> kungFuDeathGrip(mRules);
   nsresult rv = mRules->WillDoAction(selection, &ruleInfo, &cancel, &handled);
   NS_ENSURE_SUCCESS(rv, rv);
   if (cancel || handled) {
@@ -2099,7 +1972,7 @@ void RemoveBodyAndHead(nsIDOMNode *aNode)
 /**
  * This function finds the target node that we will be pasting into. aStart is
  * the context that we're given and aResult will be the target. Initially,
- * *aResult must be NULL.
+ * *aResult must be nullptr.
  *
  * The target for a paste is found by either finding the node that contains
  * the magical comment node containing kInsertCookie or, failing that, the
@@ -2116,7 +1989,7 @@ nsresult FindTargetNode(nsIDOMNode *aStart, nsCOMPtr<nsIDOMNode> &aResult)
 
   if (!child)
   {
-    // If the current result is NULL, then aStart is a leaf, and is the
+    // If the current result is nullptr, then aStart is a leaf, and is the
     // fallback result.
     if (!aResult)
       aResult = aStart;
@@ -2284,10 +2157,8 @@ nsresult nsHTMLEditor::ParseFragment(const nsAString & aFragStr,
 {
   nsAutoScriptBlockerSuppressNodeRemoved autoBlocker;
 
-  nsCOMPtr<nsIDOMDocumentFragment> frag;
-  NS_NewDocumentFragment(getter_AddRefs(frag),
-                         aTargetDocument->NodeInfoManager());
-  nsCOMPtr<nsIContent> fragment = do_QueryInterface(frag);
+  nsRefPtr<DocumentFragment> fragment =
+    new DocumentFragment(aTargetDocument->NodeInfoManager());
   nsresult rv = nsContentUtils::ParseFragmentHTML(aFragStr,
                                                   fragment,
                                                   aContextLocalName ?
@@ -2301,7 +2172,7 @@ nsresult nsHTMLEditor::ParseFragment(const nsAString & aFragStr,
                               nsIParserUtils::SanitizerAllowComments);
     sanitizer.Sanitize(fragment);
   }
-  *outNode = do_QueryInterface(frag);
+  *outNode = fragment.forget();
   return rv;
 }
 
