@@ -5,16 +5,21 @@
 
 #include "nsIMemoryReporter.h"
 #include "nsMemory.h"
+#include "mozilla/Base64.h"
 #include "mozilla/CheckedInt.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/MemoryReporting.h"
+#include "nsTraceRefcnt.h"
+#include "mozilla/gfx/2D.h"
 
 #include "gfxASurface.h"
 #include "gfxContext.h"
 #include "gfxImageSurface.h"
-
-#include "nsRect.h"
+#include "gfxPlatform.h"
+#include "gfxRect.h"
 
 #include "cairo.h"
+#include <algorithm>
 
 #ifdef CAIRO_HAS_WIN32_SURFACE
 #include "gfxWindowsSurface.h"
@@ -41,19 +46,31 @@
 
 #include "imgIEncoder.h"
 #include "nsComponentManagerUtils.h"
-#include "prmem.h"
 #include "nsISupportsUtils.h"
-#include "plbase64.h"
 #include "nsCOMPtr.h"
-#include "nsIConsoleService.h"
 #include "nsServiceManagerUtils.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsIClipboardHelper.h"
 
-using mozilla::CheckedInt;
+using namespace mozilla;
+using namespace mozilla::gfx;
 
 static cairo_user_data_key_t gfxasurface_pointer_key;
 
+gfxASurface::gfxASurface()
+ : mSurface(nullptr), mFloatingRefs(0), mBytesRecorded(0),
+   mSurfaceValid(false), mAllowUseAsSource(true)
+{
+    MOZ_COUNT_CTOR(gfxASurface);
+}
+
+gfxASurface::~gfxASurface()
+{
+    RecordMemoryFreed();
+
+    MOZ_COUNT_DTOR(gfxASurface);
+}
+
 // Surfaces use refcounting that's tied to the cairo surface refcnt, to avoid
 // refcount mismatch issues.
 nsrefcnt
@@ -100,6 +117,18 @@ gfxASurface::Release(void)
     }
 }
 
+nsrefcnt
+gfxASurface::AddRefExternal(void)
+{
+  return AddRef();
+}
+
+nsrefcnt
+gfxASurface::ReleaseExternal(void)
+{
+  return Release();
+}
+
 void
 gfxASurface::SurfaceDestroyFunc(void *data) {
     gfxASurface *surf = (gfxASurface*) data;
@@ -111,7 +140,7 @@ gfxASurface*
 gfxASurface::GetSurfaceWrapper(cairo_surface_t *csurf)
 {
     if (!csurf)
-        return NULL;
+        return nullptr;
     return (gfxASurface*) cairo_surface_get_user_data(csurf, &gfxasurface_pointer_key);
 }
 
@@ -126,14 +155,13 @@ gfxASurface::SetSurfaceWrapper(cairo_surface_t *csurf, gfxASurface *asurf)
 already_AddRefed<gfxASurface>
 gfxASurface::Wrap (cairo_surface_t *csurf)
 {
-    gfxASurface *result;
+    nsRefPtr<gfxASurface> result;
 
     /* Do we already have a wrapper for this surface? */
     result = GetSurfaceWrapper(csurf);
     if (result) {
         // fprintf(stderr, "Existing wrapper for %p -> %p\n", csurf, result);
-        NS_ADDREF(result);
-        return result;
+        return result.forget();
     }
 
     /* No wrapper; figure out the surface type and create it */
@@ -177,8 +205,7 @@ gfxASurface::Wrap (cairo_surface_t *csurf)
 
     // fprintf(stderr, "New wrapper for %p -> %p\n", csurf, result);
 
-    NS_ADDREF(result);
-    return result;
+    return result.forget();
 }
 
 void
@@ -201,7 +228,7 @@ gfxASurface::Init(cairo_surface_t* surface, bool existingSurface)
     }
 }
 
-gfxASurface::gfxSurfaceType
+gfxSurfaceType
 gfxASurface::GetType() const
 {
     if (!mSurfaceValid)
@@ -210,7 +237,7 @@ gfxASurface::GetType() const
     return (gfxSurfaceType)cairo_surface_get_type(mSurface);
 }
 
-gfxASurface::gfxContentType
+gfxContentType
 gfxASurface::GetContentType() const
 {
     if (!mSurfaceValid)
@@ -244,6 +271,7 @@ gfxASurface::Flush() const
     if (!mSurfaceValid)
         return;
     cairo_surface_flush(mSurface);
+    gfxPlatform::ClearSourceSurfaceForSurface(const_cast<gfxASurface*>(this));
 }
 
 void
@@ -252,6 +280,7 @@ gfxASurface::MarkDirty()
     if (!mSurfaceValid)
         return;
     cairo_surface_mark_dirty(mSurface);
+    gfxPlatform::ClearSourceSurfaceForSurface(this);
 }
 
 void
@@ -262,6 +291,7 @@ gfxASurface::MarkDirty(const gfxRect& r)
     cairo_surface_mark_dirty_rectangle(mSurface,
                                        (int) r.X(), (int) r.Y(),
                                        (int) r.Width(), (int) r.Height());
+    gfxPlatform::ClearSourceSurfaceForSurface(this);
 }
 
 void
@@ -278,7 +308,7 @@ void *
 gfxASurface::GetData(const cairo_user_data_key_t *key)
 {
     if (!mSurfaceValid)
-        return NULL;
+        return nullptr;
     return cairo_surface_get_user_data(mSurface, key);
 }
 
@@ -291,10 +321,10 @@ gfxASurface::Finish()
 
 already_AddRefed<gfxASurface>
 gfxASurface::CreateSimilarSurface(gfxContentType aContent,
-                                  const gfxIntSize& aSize)
+                                  const nsIntSize& aSize)
 {
     if (!mSurface || !mSurfaceValid) {
-      return nsnull;
+      return nullptr;
     }
     
     cairo_surface_t *surface =
@@ -302,7 +332,7 @@ gfxASurface::CreateSimilarSurface(gfxContentType aContent,
                                      aSize.width, aSize.height);
     if (cairo_surface_status(surface)) {
         cairo_surface_destroy(surface);
-        return nsnull;
+        return nullptr;
     }
 
     nsRefPtr<gfxASurface> result = Wrap(surface);
@@ -310,6 +340,42 @@ gfxASurface::CreateSimilarSurface(gfxContentType aContent,
     return result.forget();
 }
 
+already_AddRefed<gfxImageSurface>
+gfxASurface::GetAsReadableARGB32ImageSurface()
+{
+    nsRefPtr<gfxImageSurface> imgSurface = GetAsImageSurface();
+    if (!imgSurface || imgSurface->Format() != gfxImageFormatARGB32) {
+      imgSurface = CopyToARGB32ImageSurface();
+    }
+    return imgSurface.forget();
+}
+
+already_AddRefed<gfxImageSurface>
+gfxASurface::CopyToARGB32ImageSurface()
+{
+    if (!mSurface || !mSurfaceValid) {
+      return nullptr;
+    }
+
+    const nsIntSize size = GetSize();
+    nsRefPtr<gfxImageSurface> imgSurface =
+        new gfxImageSurface(size, gfxImageFormatARGB32);
+
+    if (gfxPlatform::GetPlatform()->SupportsAzureContent()) {
+        RefPtr<DrawTarget> dt = gfxPlatform::GetPlatform()->CreateDrawTargetForSurface(imgSurface, IntSize(size.width, size.height));
+        RefPtr<SourceSurface> source = gfxPlatform::GetPlatform()->GetSourceSurfaceForSurface(dt, this);
+
+        dt->CopySurface(source, IntRect(0, 0, size.width, size.height), IntPoint());
+    } else {
+        gfxContext ctx(imgSurface);
+        ctx.SetOperator(gfxContext::OPERATOR_SOURCE);
+        ctx.SetSource(this);
+        ctx.Paint();
+    }
+
+    return imgSurface.forget();
+}
+
 int
 gfxASurface::CairoStatus()
 {
@@ -321,7 +387,7 @@ gfxASurface::CairoStatus()
 
 /* static */
 bool
-gfxASurface::CheckSurfaceSize(const gfxIntSize& sz, PRInt32 limit)
+gfxASurface::CheckSurfaceSize(const nsIntSize& sz, int32_t limit)
 {
     if (sz.width < 0 || sz.height < 0) {
         NS_WARNING("Surface width or height < 0!");
@@ -343,8 +409,8 @@ gfxASurface::CheckSurfaceSize(const gfxIntSize& sz, PRInt32 limit)
     }
 #endif
 
-    // make sure the surface area doesn't overflow a PRInt32
-    CheckedInt<PRInt32> tmp = sz.width;
+    // make sure the surface area doesn't overflow a int32_t
+    CheckedInt<int32_t> tmp = sz.width;
     tmp *= sz.height;
     if (!tmp.isValid()) {
         NS_WARNING("Surface size too large (would overflow)!");
@@ -352,7 +418,7 @@ gfxASurface::CheckSurfaceSize(const gfxIntSize& sz, PRInt32 limit)
     }
 
     // assuming 4-byte stride, make sure the allocation size
-    // doesn't overflow a PRInt32 either
+    // doesn't overflow a int32_t either
     tmp *= 4;
     if (!tmp.isValid()) {
         NS_WARNING("Allocation too large (would overflow)!");
@@ -363,8 +429,8 @@ gfxASurface::CheckSurfaceSize(const gfxIntSize& sz, PRInt32 limit)
 }
 
 /* static */
-PRInt32
-gfxASurface::FormatStrideForWidth(gfxImageFormat format, PRInt32 width)
+int32_t
+gfxASurface::FormatStrideForWidth(gfxImageFormat format, int32_t width)
 {
     return cairo_format_stride_for_width((cairo_format_t)format, (int)width);
 }
@@ -399,22 +465,22 @@ gfxASurface::EndPage()
     return NS_OK;
 }
 
-gfxASurface::gfxContentType
+gfxContentType
 gfxASurface::ContentFromFormat(gfxImageFormat format)
 {
     switch (format) {
-        case ImageFormatARGB32:
-            return CONTENT_COLOR_ALPHA;
-        case ImageFormatRGB24:
-        case ImageFormatRGB16_565:
-            return CONTENT_COLOR;
-        case ImageFormatA8:
-        case ImageFormatA1:
-            return CONTENT_ALPHA;
+        case gfxImageFormatARGB32:
+            return GFX_CONTENT_COLOR_ALPHA;
+        case gfxImageFormatRGB24:
+        case gfxImageFormatRGB16_565:
+            return GFX_CONTENT_COLOR;
+        case gfxImageFormatA8:
+        case gfxImageFormatA1:
+            return GFX_CONTENT_ALPHA;
 
-        case ImageFormatUnknown:
+        case gfxImageFormatUnknown:
         default:
-            return CONTENT_COLOR;
+            return GFX_CONTENT_COLOR;
     }
 }
 
@@ -441,22 +507,22 @@ gfxASurface::GetSubpixelAntialiasingEnabled()
 #endif
 }
 
-gfxASurface::MemoryLocation
+gfxMemoryLocation
 gfxASurface::GetMemoryLocation() const
 {
-    return MEMORY_IN_PROCESS_HEAP;
+    return GFX_MEMORY_IN_PROCESS_HEAP;
 }
 
-PRInt32
+int32_t
 gfxASurface::BytePerPixelFromFormat(gfxImageFormat format)
 {
     switch (format) {
-        case ImageFormatARGB32:
-        case ImageFormatRGB24:
+        case gfxImageFormatARGB32:
+        case gfxImageFormatRGB24:
             return 4;
-        case ImageFormatRGB16_565:
+        case gfxImageFormatRGB16_565:
             return 2;
-        case ImageFormatA8:
+        case gfxImageFormatA8:
             return 1;
         default:
             NS_WARNING("Unknown byte per pixel value for Image format");
@@ -487,7 +553,14 @@ gfxASurface::MovePixels(const nsIntRect& aSourceRect,
     // a temporary surface instead.
     nsRefPtr<gfxASurface> tmp = 
       CreateSimilarSurface(GetContentType(), 
-                           gfxIntSize(aSourceRect.width, aSourceRect.height));
+                           nsIntSize(aSourceRect.width, aSourceRect.height));
+    // CreateSimilarSurface can return nullptr if the current surface is
+    // in an error state. This isn't good, but its better to carry
+    // on with the error surface instead of crashing.
+    NS_WARN_IF_FALSE(tmp, "Must have temporary surface to move pixels!");
+    if (!tmp) {
+        return;
+    }
     nsRefPtr<gfxContext> ctx = new gfxContext(tmp);
     ctx->SetOperator(gfxContext::OPERATOR_SOURCE);
     ctx->SetSource(this, gfxPoint(-aSourceRect.x, -aSourceRect.y));
@@ -514,52 +587,52 @@ struct SurfaceMemoryReporterAttrs {
 };
 
 static const SurfaceMemoryReporterAttrs sSurfaceMemoryReporterAttrs[] = {
-    {"gfx-surface-image", nsnull},
-    {"gfx-surface-pdf", nsnull},
-    {"gfx-surface-ps", nsnull},
+    {"gfx-surface-image", nullptr},
+    {"gfx-surface-pdf", nullptr},
+    {"gfx-surface-ps", nullptr},
     {"gfx-surface-xlib",
      "Memory used by xlib surfaces to store pixmaps. This memory lives in "
      "the X server's process rather than in this application, so the bytes "
      "accounted for here aren't counted in vsize, resident, explicit, or any of "
      "the other measurements on this page."},
-    {"gfx-surface-xcb", nsnull},
-    {"gfx-surface-glitz???", nsnull},       // should never be used
-    {"gfx-surface-quartz", nsnull},
-    {"gfx-surface-win32", nsnull},
-    {"gfx-surface-beos", nsnull},
-    {"gfx-surface-directfb???", nsnull},    // should never be used
-    {"gfx-surface-svg", nsnull},
-    {"gfx-surface-os2", nsnull},
-    {"gfx-surface-win32printing", nsnull},
-    {"gfx-surface-quartzimage", nsnull},
-    {"gfx-surface-script", nsnull},
-    {"gfx-surface-qpainter", nsnull},
-    {"gfx-surface-recording", nsnull},
-    {"gfx-surface-vg", nsnull},
-    {"gfx-surface-gl", nsnull},
-    {"gfx-surface-drm", nsnull},
-    {"gfx-surface-tee", nsnull},
-    {"gfx-surface-xml", nsnull},
-    {"gfx-surface-skia", nsnull},
-    {"gfx-surface-subsurface", nsnull},
-    {"gfx-surface-d2d", nsnull},
+    {"gfx-surface-xcb", nullptr},
+    {"gfx-surface-glitz???", nullptr},       // should never be used
+    {"gfx-surface-quartz", nullptr},
+    {"gfx-surface-win32", nullptr},
+    {"gfx-surface-beos", nullptr},
+    {"gfx-surface-directfb???", nullptr},    // should never be used
+    {"gfx-surface-svg", nullptr},
+    {"gfx-surface-os2", nullptr},
+    {"gfx-surface-win32printing", nullptr},
+    {"gfx-surface-quartzimage", nullptr},
+    {"gfx-surface-script", nullptr},
+    {"gfx-surface-qpainter", nullptr},
+    {"gfx-surface-recording", nullptr},
+    {"gfx-surface-vg", nullptr},
+    {"gfx-surface-gl", nullptr},
+    {"gfx-surface-drm", nullptr},
+    {"gfx-surface-tee", nullptr},
+    {"gfx-surface-xml", nullptr},
+    {"gfx-surface-skia", nullptr},
+    {"gfx-surface-subsurface", nullptr},
+    {"gfx-surface-d2d", nullptr},
 };
 
 PR_STATIC_ASSERT(NS_ARRAY_LENGTH(sSurfaceMemoryReporterAttrs) ==
-                 gfxASurface::SurfaceTypeMax);
+                 gfxSurfaceTypeMax);
 #ifdef CAIRO_HAS_D2D_SURFACE
-PR_STATIC_ASSERT(PRUint32(CAIRO_SURFACE_TYPE_D2D) ==
-                 PRUint32(gfxASurface::SurfaceTypeD2D));
+PR_STATIC_ASSERT(uint32_t(CAIRO_SURFACE_TYPE_D2D) ==
+                 uint32_t(gfxSurfaceTypeD2D));
 #endif
-PR_STATIC_ASSERT(PRUint32(CAIRO_SURFACE_TYPE_SKIA) ==
-                 PRUint32(gfxASurface::SurfaceTypeSkia));
+PR_STATIC_ASSERT(uint32_t(CAIRO_SURFACE_TYPE_SKIA) ==
+                 uint32_t(gfxSurfaceTypeSkia));
 
 /* Surface size memory reporting */
 
-static PRInt64 gSurfaceMemoryUsed[gfxASurface::SurfaceTypeMax] = { 0 };
+static int64_t gSurfaceMemoryUsed[gfxSurfaceTypeMax] = { 0 };
 
 class SurfaceMemoryReporter MOZ_FINAL :
-    public nsIMemoryMultiReporter
+    public nsIMemoryReporter
 {
 public:
     SurfaceMemoryReporter()
@@ -573,12 +646,12 @@ public:
         return NS_OK;
     }
 
-    NS_IMETHOD CollectReports(nsIMemoryMultiReporterCallback *aCb,
+    NS_IMETHOD CollectReports(nsIMemoryReporterCallback *aCb,
                               nsISupports *aClosure)
     {
         size_t len = NS_ARRAY_LENGTH(sSurfaceMemoryReporterAttrs);
         for (size_t i = 0; i < len; i++) {
-            PRInt64 amount = gSurfaceMemoryUsed[i];
+            int64_t amount = gSurfaceMemoryUsed[i];
 
             if (amount != 0) {
                 const char *path = sSurfaceMemoryReporterAttrs[i].path;
@@ -598,28 +671,22 @@ public:
 
         return NS_OK;
     }
-
-    NS_IMETHOD GetExplicitNonHeap(PRInt64 *n)
-    {
-        *n = 0; // this reporter makes neither "explicit" non NONHEAP reports
-        return NS_OK;
-    }
 };
 
-NS_IMPL_ISUPPORTS1(SurfaceMemoryReporter, nsIMemoryMultiReporter)
+NS_IMPL_ISUPPORTS1(SurfaceMemoryReporter, nsIMemoryReporter)
 
 void
-gfxASurface::RecordMemoryUsedForSurfaceType(gfxASurface::gfxSurfaceType aType,
-                                            PRInt32 aBytes)
+gfxASurface::RecordMemoryUsedForSurfaceType(gfxSurfaceType aType,
+                                            int32_t aBytes)
 {
-    if (aType < 0 || aType >= SurfaceTypeMax) {
+    if (aType < 0 || aType >= gfxSurfaceTypeMax) {
         NS_WARNING("Invalid type to RecordMemoryUsedForSurfaceType!");
         return;
     }
 
     static bool registered = false;
     if (!registered) {
-        NS_RegisterMemoryMultiReporter(new SurfaceMemoryReporter());
+        NS_RegisterMemoryReporter(new SurfaceMemoryReporter());
         registered = true;
     }
 
@@ -627,7 +694,7 @@ gfxASurface::RecordMemoryUsedForSurfaceType(gfxASurface::gfxSurfaceType aType,
 }
 
 void
-gfxASurface::RecordMemoryUsed(PRInt32 aBytes)
+gfxASurface::RecordMemoryUsed(int32_t aBytes)
 {
     RecordMemoryUsedForSurfaceType(GetType(), aBytes);
     mBytesRecorded += aBytes;
@@ -642,7 +709,40 @@ gfxASurface::RecordMemoryFreed()
     }
 }
 
-#ifdef MOZ_DUMP_IMAGES
+size_t
+gfxASurface::SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const
+{
+    // We don't measure mSurface because cairo doesn't allow it.
+    return 0;
+}
+
+size_t
+gfxASurface::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
+{
+    return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
+}
+
+/* static */ uint8_t
+gfxASurface::BytesPerPixel(gfxImageFormat aImageFormat)
+{
+  switch (aImageFormat) {
+    case gfxImageFormatARGB32:
+      return 4;
+    case gfxImageFormatRGB24:
+      return 4;
+    case gfxImageFormatRGB16_565:
+      return 2;
+    case gfxImageFormatA8:
+      return 1;
+    case gfxImageFormatA1:
+      return 1; // Close enough
+    case gfxImageFormatUnknown:
+    default:
+      NS_NOTREACHED("Not really sure what you want me to say here");
+      return 0;
+  }
+}
+
 void
 gfxASurface::WriteAsPNG(const char* aFile)
 {
@@ -671,7 +771,7 @@ gfxASurface::PrintAsDataURL()
 void
 gfxASurface::CopyAsDataURL()
 {
-  WriteAsPNG_internal(nsnull, false);
+  WriteAsPNG_internal(nullptr, false);
 }
 
 /**
@@ -683,9 +783,10 @@ void
 gfxASurface::WriteAsPNG_internal(FILE* aFile, bool aBinary)
 {
   nsRefPtr<gfxImageSurface> imgsurf = GetAsImageSurface();
-  gfxIntSize size;
+  nsIntSize size;
 
-  if (!imgsurf) {
+  // FIXME/bug 831898: hack r5g6b5 for now.
+  if (!imgsurf || imgsurf->Format() == gfxImageFormatRGB16_565) {
     size = GetSize();
     if (size.width == -1 && size.height == -1) {
       printf("Could not determine surface size\n");
@@ -693,8 +794,8 @@ gfxASurface::WriteAsPNG_internal(FILE* aFile, bool aBinary)
     }
 
     imgsurf =
-      new gfxImageSurface(gfxIntSize(size.width, size.height),
-                          gfxASurface::ImageFormatARGB32);
+      new gfxImageSurface(nsIntSize(size.width, size.height),
+                          gfxImageFormatARGB32);
 
     if (!imgsurf || imgsurf->CairoStatus()) {
       printf("Could not allocate image surface\n");
@@ -716,12 +817,12 @@ gfxASurface::WriteAsPNG_internal(FILE* aFile, bool aBinary)
   nsCOMPtr<imgIEncoder> encoder =
     do_CreateInstance("@mozilla.org/image/encoder;2?type=image/png");
   if (!encoder) {
-    PRInt32 w = NS_MIN(size.width, 8);
-    PRInt32 h = NS_MIN(size.height, 8);
+    int32_t w = std::min(size.width, 8);
+    int32_t h = std::min(size.height, 8);
     printf("Could not create encoder. Printing %dx%d pixels.\n", w, h);
-    for (PRInt32 y = 0; y < h; ++y) {
-      for (PRInt32 x = 0; x < w; ++x) {
-        printf("%x ", reinterpret_cast<PRUint32*>(imgsurf->Data())[y*imgsurf->Stride()+ x]);
+    for (int32_t y = 0; y < h; ++y) {
+      for (int32_t x = 0; x < w; ++x) {
+        printf("%x ", reinterpret_cast<uint32_t*>(imgsurf->Data())[y*imgsurf->Stride()+ x]);
       }
     }
     return;
@@ -742,19 +843,24 @@ gfxASurface::WriteAsPNG_internal(FILE* aFile, bool aBinary)
   if (!imgStream)
     return;
 
-  PRUint32 bufSize;
-  rv = imgStream->Available(&bufSize);
+  uint64_t bufSize64;
+  rv = imgStream->Available(&bufSize64);
   if (NS_FAILED(rv))
     return;
 
+  if (bufSize64 > UINT32_MAX - 16)
+    return;
+
+  uint32_t bufSize = (uint32_t)bufSize64;
+
   // ...leave a little extra room so we can call read again and make sure we
   // got everything. 16 bytes for better padding (maybe)
   bufSize += 16;
-  PRUint32 imgSize = 0;
-  char* imgData = (char*)PR_Malloc(bufSize);
+  uint32_t imgSize = 0;
+  char* imgData = (char*)moz_malloc(bufSize);
   if (!imgData)
     return;
-  PRUint32 numReadThisTime = 0;
+  uint32_t numReadThisTime = 0;
   while ((rv = imgStream->Read(&imgData[imgSize],
                                bufSize - imgSize,
                                &numReadThisTime)) == NS_OK && numReadThisTime > 0)
@@ -763,9 +869,9 @@ gfxASurface::WriteAsPNG_internal(FILE* aFile, bool aBinary)
     if (imgSize == bufSize) {
       // need a bigger buffer, just double
       bufSize *= 2;
-      char* newImgData = (char*)PR_Realloc(imgData, bufSize);
+      char* newImgData = (char*)moz_realloc(imgData, bufSize);
       if (!newImgData) {
-        PR_Free(imgData);
+        moz_free(imgData);
         return;
       }
       imgData = newImgData;
@@ -782,9 +888,10 @@ gfxASurface::WriteAsPNG_internal(FILE* aFile, bool aBinary)
   }
 
   // base 64, result will be NULL terminated
-  char* encodedImg = PL_Base64Encode(imgData, imgSize, nsnull);
-  PR_Free(imgData);
-  if (!encodedImg) // not sure why this would fail
+  nsCString encodedImg;
+  rv = Base64Encode(Substring(imgData, imgSize), encodedImg);
+  moz_free(imgData);
+  if (NS_FAILED(rv)) // not sure why this would fail
     return;
 
   nsCString string("data:image/png;base64,");
@@ -809,13 +916,40 @@ gfxASurface::WriteAsPNG_internal(FILE* aFile, bool aBinary)
   } else {
     nsCOMPtr<nsIClipboardHelper> clipboard(do_GetService("@mozilla.org/widget/clipboardhelper;1", &rv));
     if (clipboard) {
-      clipboard->CopyString(NS_ConvertASCIItoUTF16(string), nsnull);
+      clipboard->CopyString(NS_ConvertASCIItoUTF16(string), nullptr);
     }
   }
 
-  PR_Free(encodedImg);
-
   return;
 }
-#endif
 
+void
+gfxASurface::SetOpaqueRect(const gfxRect& aRect)
+{
+    if (aRect.IsEmpty()) {
+        mOpaqueRect = nullptr;
+    } else if (!!mOpaqueRect) {
+        *mOpaqueRect = aRect;
+    } else {
+        mOpaqueRect = new gfxRect(aRect);
+    }
+}
+
+/* static */const gfxRect&
+gfxASurface::GetEmptyOpaqueRect()
+{
+  static const gfxRect empty(0, 0, 0, 0);
+  return empty;
+}
+
+const nsIntSize
+gfxASurface::GetSize() const
+{
+  return nsIntSize(-1, -1);
+}
+
+already_AddRefed<gfxImageSurface>
+gfxASurface::GetAsImageSurface()
+{
+  return nullptr;
+}
