@@ -10,7 +10,8 @@
 #include "nsRegion.h"
 #include "yuv_convert.h"
 #include "ycbcr_to_rgb565.h"
-#include "sampler.h"
+#include "GeckoProfiler.h"
+#include "ImageContainer.h"
 
 #ifdef XP_WIN
 #include "gfxWindowsPlatform.h"
@@ -20,49 +21,14 @@ using namespace mozilla;
 using namespace mozilla::layers;
 using namespace mozilla::gfx;
 
-static uint8_t sUnpremultiplyTable[256*256];
-static uint8_t sPremultiplyTable[256*256];
-static bool sTablesInitialized = false;
+#include "PremultiplyTables.h"
 
 static const uint8_t PremultiplyValue(uint8_t a, uint8_t v) {
-    return sPremultiplyTable[a*256+v];
+    return gfxUtils::sPremultiplyTable[a*256+v];
 }
 
 static const uint8_t UnpremultiplyValue(uint8_t a, uint8_t v) {
-    return sUnpremultiplyTable[a*256+v];
-}
-
-static void
-CalculateTables()
-{
-    // It's important that the array be indexed first by alpha and then by rgb
-    // value.  When we unpremultiply a pixel, we're guaranteed to do three
-    // lookups with the same alpha; indexing by alpha first makes it likely that
-    // those three lookups will be close to one another in memory, thus
-    // increasing the chance of a cache hit.
-
-    // Unpremultiply table
-
-    // a == 0 case
-    for (uint32_t c = 0; c <= 255; c++) {
-        sUnpremultiplyTable[c] = c;
-    }
-
-    for (int a = 1; a <= 255; a++) {
-        for (int c = 0; c <= 255; c++) {
-            sUnpremultiplyTable[a*256+c] = (uint8_t)((c * 255) / a);
-        }
-    }
-
-    // Premultiply table
-
-    for (int a = 0; a <= 255; a++) {
-        for (int c = 0; c <= 255; c++) {
-            sPremultiplyTable[a*256+c] = (a * c + 254) / 255;
-        }
-    }
-
-    sTablesInitialized = true;
+    return gfxUtils::sUnpremultiplyTable[a*256+v];
 }
 
 void
@@ -72,17 +38,17 @@ gfxUtils::PremultiplyImageSurface(gfxImageSurface *aSourceSurface,
     if (!aDestSurface)
         aDestSurface = aSourceSurface;
 
-    NS_ASSERTION(aSourceSurface->Format() == aDestSurface->Format() &&
-                 aSourceSurface->Width() == aDestSurface->Width() &&
-                 aSourceSurface->Height() == aDestSurface->Height() &&
-                 aSourceSurface->Stride() == aDestSurface->Stride(),
-                 "Source and destination surfaces don't have identical characteristics");
+    MOZ_ASSERT(aSourceSurface->Format() == aDestSurface->Format() &&
+               aSourceSurface->Width()  == aDestSurface->Width() &&
+               aSourceSurface->Height() == aDestSurface->Height() &&
+               aSourceSurface->Stride() == aDestSurface->Stride(),
+               "Source and destination surfaces don't have identical characteristics");
 
-    NS_ASSERTION(aSourceSurface->Stride() == aSourceSurface->Width() * 4,
-                 "Source surface stride isn't tightly packed");
+    MOZ_ASSERT(aSourceSurface->Stride() == aSourceSurface->Width() * 4,
+               "Source surface stride isn't tightly packed");
 
     // Only premultiply ARGB32
-    if (aSourceSurface->Format() != gfxASurface::ImageFormatARGB32) {
+    if (aSourceSurface->Format() != gfxImageFormatARGB32) {
         if (aDestSurface != aSourceSurface) {
             memcpy(aDestSurface->Data(), aSourceSurface->Data(),
                    aSourceSurface->Stride() * aSourceSurface->Height());
@@ -90,9 +56,6 @@ gfxUtils::PremultiplyImageSurface(gfxImageSurface *aSourceSurface,
         return;
     }
 
-    if (!sTablesInitialized)
-        CalculateTables();
-
     uint8_t *src = aSourceSurface->Data();
     uint8_t *dst = aDestSurface->Data();
 
@@ -129,17 +92,17 @@ gfxUtils::UnpremultiplyImageSurface(gfxImageSurface *aSourceSurface,
     if (!aDestSurface)
         aDestSurface = aSourceSurface;
 
-    NS_ASSERTION(aSourceSurface->Format() == aDestSurface->Format() &&
-                 aSourceSurface->Width() == aDestSurface->Width() &&
-                 aSourceSurface->Height() == aDestSurface->Height() &&
-                 aSourceSurface->Stride() == aDestSurface->Stride(),
-                 "Source and destination surfaces don't have identical characteristics");
+    MOZ_ASSERT(aSourceSurface->Format() == aDestSurface->Format() &&
+               aSourceSurface->Width()  == aDestSurface->Width() &&
+               aSourceSurface->Height() == aDestSurface->Height() &&
+               aSourceSurface->Stride() == aDestSurface->Stride(),
+               "Source and destination surfaces don't have identical characteristics");
 
-    NS_ASSERTION(aSourceSurface->Stride() == aSourceSurface->Width() * 4,
-                 "Source surface stride isn't tightly packed");
+    MOZ_ASSERT(aSourceSurface->Stride() == aSourceSurface->Width() * 4,
+               "Source surface stride isn't tightly packed");
 
     // Only premultiply ARGB32
-    if (aSourceSurface->Format() != gfxASurface::ImageFormatARGB32) {
+    if (aSourceSurface->Format() != gfxImageFormatARGB32) {
         if (aDestSurface != aSourceSurface) {
             memcpy(aDestSurface->Data(), aSourceSurface->Data(),
                    aSourceSurface->Stride() * aSourceSurface->Height());
@@ -147,9 +110,6 @@ gfxUtils::UnpremultiplyImageSurface(gfxImageSurface *aSourceSurface,
         return;
     }
 
-    if (!sTablesInitialized)
-        CalculateTables();
-
     uint8_t *src = aSourceSurface->Data();
     uint8_t *dst = aDestSurface->Data();
 
@@ -185,17 +145,17 @@ gfxUtils::ConvertBGRAtoRGBA(gfxImageSurface *aSourceSurface,
     if (!aDestSurface)
         aDestSurface = aSourceSurface;
 
-    NS_ABORT_IF_FALSE(aSourceSurface->Format() == aDestSurface->Format() &&
-                      aSourceSurface->Width() == aDestSurface->Width() &&
-                      aSourceSurface->Height() == aDestSurface->Height() &&
-                      aSourceSurface->Stride() == aDestSurface->Stride(),
-                      "Source and destination surfaces don't have identical characteristics");
+    MOZ_ASSERT(aSourceSurface->Format() == aDestSurface->Format() &&
+               aSourceSurface->Width()  == aDestSurface->Width() &&
+               aSourceSurface->Height() == aDestSurface->Height() &&
+               aSourceSurface->Stride() == aDestSurface->Stride(),
+               "Source and destination surfaces don't have identical characteristics");
 
-    NS_ABORT_IF_FALSE(aSourceSurface->Stride() == aSourceSurface->Width() * 4,
-                      "Source surface stride isn't tightly packed");
+    MOZ_ASSERT(aSourceSurface->Stride() == aSourceSurface->Width() * 4,
+               "Source surface stride isn't tightly packed");
 
-    NS_ABORT_IF_FALSE(aSourceSurface->Format() == gfxASurface::ImageFormatARGB32,
-                      "Surfaces must be ARGB32");
+    MOZ_ASSERT(aSourceSurface->Format() == gfxImageFormatARGB32 || aSourceSurface->Format() == gfxImageFormatRGB24,
+               "Surfaces must be ARGB32 or RGB24");
 
     uint8_t *src = aSourceSurface->Data();
     uint8_t *dst = aDestSurface->Data();
@@ -230,6 +190,7 @@ IsSafeImageTransformComponent(gfxFloat aValue)
   return aValue >= -32768 && aValue <= 32767;
 }
 
+#ifndef MOZ_GFX_OPTIMIZE_MOBILE
 /**
  * This returns the fastest operator to use for solid surfaces which have no
  * alpha channel or their alpha channel is uniformly opaque.
@@ -259,9 +220,9 @@ CreateSamplingRestrictedDrawable(gfxDrawable* aDrawable,
                                  const gfxMatrix& aUserSpaceToImageSpace,
                                  const gfxRect& aSourceRect,
                                  const gfxRect& aSubimage,
-                                 const gfxImageSurface::gfxImageFormat aFormat)
+                                 const gfxImageFormat aFormat)
 {
-    SAMPLE_LABEL("gfxUtils", "CreateSamplingRestricedDrawable");
+    PROFILER_LABEL("gfxUtils", "CreateSamplingRestricedDrawable");
     gfxRect userSpaceClipExtents = aContext->GetClipExtents();
     // This isn't optimal --- if aContext has a rotation then GetClipExtents
     // will have to do a bounding-box computation, and TransformBounds might
@@ -285,29 +246,33 @@ CreateSamplingRestrictedDrawable(gfxDrawable* aDrawable,
     if (needed.IsEmpty())
         return nullptr;
 
+    nsRefPtr<gfxASurface> temp;
     gfxIntSize size(int32_t(needed.Width()), int32_t(needed.Height()));
-    nsRefPtr<gfxASurface> temp =
-        gfxPlatform::GetPlatform()->CreateOffscreenSurface(size, gfxASurface::ContentFromFormat(aFormat));
-    if (!temp || temp->CairoStatus())
-        return nullptr;
 
-    nsRefPtr<gfxContext> tmpCtx = new gfxContext(temp);
-    tmpCtx->SetOperator(OptimalFillOperator());
-    aDrawable->Draw(tmpCtx, needed - needed.TopLeft(), true,
-                    gfxPattern::FILTER_FAST, gfxMatrix().Translate(needed.TopLeft()));
+    nsRefPtr<gfxImageSurface> image = aDrawable->GetAsImageSurface();
+    if (image && gfxRect(0, 0, image->GetSize().width, image->GetSize().height).Contains(needed)) {
+      temp = image->GetSubimage(needed);
+    } else {
+      temp =
+          gfxPlatform::GetPlatform()->CreateOffscreenSurface(size, gfxASurface::ContentFromFormat(aFormat));
+      if (!temp || temp->CairoStatus())
+          return nullptr;
 
-    nsRefPtr<gfxPattern> resultPattern = new gfxPattern(temp);
-    if (!resultPattern)
-        return nullptr;
+      nsRefPtr<gfxContext> tmpCtx = new gfxContext(temp);
+      tmpCtx->SetOperator(OptimalFillOperator());
+      aDrawable->Draw(tmpCtx, needed - needed.TopLeft(), true,
+                      GraphicsFilter::FILTER_FAST, gfxMatrix().Translate(needed.TopLeft()));
+    }
 
     nsRefPtr<gfxDrawable> drawable = 
         new gfxSurfaceDrawable(temp, size, gfxMatrix().Translate(-needed.TopLeft()));
     return drawable.forget();
 }
+#endif // !MOZ_GFX_OPTIMIZE_MOBILE
 
 // working around cairo/pixman bug (bug 364968)
 // Our device-space-to-image-space transform may not be acceptable to pixman.
-struct NS_STACK_CLASS AutoCairoPixmanBugWorkaround
+struct MOZ_STACK_CLASS AutoCairoPixmanBugWorkaround
 {
     AutoCairoPixmanBugWorkaround(gfxContext*      aContext,
                                  const gfxMatrix& aDeviceSpaceToImageSpace,
@@ -316,7 +281,7 @@ struct NS_STACK_CLASS AutoCairoPixmanBugWorkaround
      : mContext(aContext), mSucceeded(true), mPushedGroup(false)
     {
         // Quartz's limits for matrix are much larger than pixman
-        if (!aSurface || aSurface->GetType() == gfxASurface::SurfaceTypeQuartz)
+        if (!aSurface || aSurface->GetType() == gfxSurfaceTypeQuartz)
             return;
 
         if (!IsSafeImageTransformComponent(aDeviceSpaceToImageSpace.xx) ||
@@ -345,7 +310,7 @@ struct NS_STACK_CLASS AutoCairoPixmanBugWorkaround
         bounds.RoundOut();
         mContext->Clip(bounds);
         mContext->SetMatrix(currentMatrix);
-        mContext->PushGroup(gfxASurface::CONTENT_COLOR_ALPHA);
+        mContext->PushGroup(GFX_CONTENT_COLOR_ALPHA);
         mContext->SetOperator(gfxContext::OPERATOR_OVER);
 
         mPushedGroup = true;
@@ -384,9 +349,9 @@ DeviceToImageTransform(gfxContext* aContext,
 
 /* These heuristics are based on Source/WebCore/platform/graphics/skia/ImageSkia.cpp:computeResamplingMode() */
 #ifdef MOZ_GFX_OPTIMIZE_MOBILE
-static gfxPattern::GraphicsFilter ReduceResamplingFilter(gfxPattern::GraphicsFilter aFilter,
-                                                         int aImgWidth, int aImgHeight,
-                                                         float aSourceWidth, float aSourceHeight)
+static GraphicsFilter ReduceResamplingFilter(GraphicsFilter aFilter,
+                                             int aImgWidth, int aImgHeight,
+                                             float aSourceWidth, float aSourceHeight)
 {
     // Images smaller than this in either direction are considered "small" and
     // are not resampled ever (see below).
@@ -401,7 +366,7 @@ static gfxPattern::GraphicsFilter ReduceResamplingFilter(gfxPattern::GraphicsFil
         || aImgHeight <= kSmallImageSizeThreshold) {
         // Never resample small images. These are often used for borders and
         // rules (think 1x1 images used to make lines).
-        return gfxPattern::FILTER_NEAREST;
+        return GraphicsFilter::FILTER_NEAREST;
     }
 
     if (aImgHeight * kLargeStretch <= aSourceHeight || aImgWidth * kLargeStretch <= aSourceWidth) {
@@ -412,7 +377,7 @@ static gfxPattern::GraphicsFilter ReduceResamplingFilter(gfxPattern::GraphicsFil
         // (which might be large) and then is stretching it to fill some part
         // of the page.
         if (fabs(aSourceWidth - aImgWidth)/aImgWidth < 0.5 || fabs(aSourceHeight - aImgHeight)/aImgHeight < 0.5)
-            return gfxPattern::FILTER_NEAREST;
+            return GraphicsFilter::FILTER_NEAREST;
 
         // The image is growing a lot and in more than one direction. Resampling
         // is slow and doesn't give us very much when growing a lot.
@@ -440,9 +405,9 @@ static gfxPattern::GraphicsFilter ReduceResamplingFilter(gfxPattern::GraphicsFil
     return aFilter;
 }
 #else
-static gfxPattern::GraphicsFilter ReduceResamplingFilter(gfxPattern::GraphicsFilter aFilter,
-                                                          int aImgWidth, int aImgHeight,
-                                                          int aSourceWidth, int aSourceHeight)
+static GraphicsFilter ReduceResamplingFilter(GraphicsFilter aFilter,
+                                             int aImgWidth, int aImgHeight,
+                                             int aSourceWidth, int aSourceHeight)
 {
     // Just pass the filter through unchanged
     return aFilter;
@@ -457,11 +422,11 @@ gfxUtils::DrawPixelSnapped(gfxContext*      aContext,
                            const gfxRect&   aSourceRect,
                            const gfxRect&   aImageRect,
                            const gfxRect&   aFill,
-                           const gfxImageSurface::gfxImageFormat aFormat,
-                           gfxPattern::GraphicsFilter aFilter,
+                           const gfxImageFormat aFormat,
+                           GraphicsFilter aFilter,
                            uint32_t         aImageFlags)
 {
-    SAMPLE_LABEL("gfxUtils", "DrawPixelSnapped");
+    PROFILER_LABEL("gfxUtils", "DrawPixelSnapped");
     bool doTile = !aImageRect.Contains(aSourceRect) &&
                   !(aImageFlags & imgIContainer::FLAG_CLAMP);
 
@@ -517,26 +482,18 @@ gfxUtils::DrawPixelSnapped(gfxContext*      aContext,
     }
 #endif
 
-    gfxContext::GraphicsOperator op = aContext->CurrentOperator();
-    if ((op == gfxContext::OPERATOR_OVER || workaround.PushedGroup()) &&
-        aFormat == gfxASurface::ImageFormatRGB24) {
-        aContext->SetOperator(OptimalFillOperator());
-    }
-
     drawable->Draw(aContext, aFill, doTile, aFilter, userSpaceToImageSpace);
-
-    aContext->SetOperator(op);
 }
 
 /* static */ int
-gfxUtils::ImageFormatToDepth(gfxASurface::gfxImageFormat aFormat)
+gfxUtils::ImageFormatToDepth(gfxImageFormat aFormat)
 {
     switch (aFormat) {
-        case gfxASurface::ImageFormatARGB32:
+        case gfxImageFormatARGB32:
             return 32;
-        case gfxASurface::ImageFormatRGB24:
+        case gfxImageFormatRGB24:
             return 24;
-        case gfxASurface::ImageFormatRGB16_565:
+        case gfxImageFormatRGB16_565:
             return 16;
         default:
             break;
@@ -564,18 +521,80 @@ ClipToRegionInternal(gfxContext* aContext, const nsIntRegion& aRegion,
   aContext->Clip();
 }
 
+static TemporaryRef<Path>
+PathFromRegionInternal(gfx::DrawTarget* aTarget, const nsIntRegion& aRegion,
+                       bool aSnap)
+{
+  Matrix mat = aTarget->GetTransform();
+  const gfxFloat epsilon = 0.000001;
+#define WITHIN_E(a,b) (fabs((a)-(b)) < epsilon)
+  // We're essentially duplicating the logic in UserToDevicePixelSnapped here.
+  bool shouldNotSnap = !aSnap || (WITHIN_E(mat._11,1.0) &&
+                                  WITHIN_E(mat._22,1.0) &&
+                                  WITHIN_E(mat._12,0.0) &&
+                                  WITHIN_E(mat._21,0.0));
+#undef WITHIN_E
+
+  RefPtr<PathBuilder> pb = aTarget->CreatePathBuilder();
+  nsIntRegionRectIterator iter(aRegion);
+
+  const nsIntRect* r;
+  if (shouldNotSnap) {
+    while ((r = iter.Next()) != nullptr) {
+      pb->MoveTo(Point(r->x, r->y));
+      pb->LineTo(Point(r->XMost(), r->y));
+      pb->LineTo(Point(r->XMost(), r->YMost()));
+      pb->LineTo(Point(r->x, r->YMost()));
+      pb->Close();
+    }
+  } else {
+    while ((r = iter.Next()) != nullptr) {
+      Rect rect(r->x, r->y, r->width, r->height);
+
+      rect.Round();
+      pb->MoveTo(rect.TopLeft());
+      pb->LineTo(rect.TopRight());
+      pb->LineTo(rect.BottomRight());
+      pb->LineTo(rect.BottomLeft());
+      pb->Close();
+    }
+  }
+  RefPtr<Path> path = pb->Finish();
+  return path;
+}
+
+static void
+ClipToRegionInternal(gfx::DrawTarget* aTarget, const nsIntRegion& aRegion,
+                     bool aSnap)
+{
+  RefPtr<Path> path = PathFromRegionInternal(aTarget, aRegion, aSnap);
+  aTarget->PushClip(path);
+}
+
 /*static*/ void
 gfxUtils::ClipToRegion(gfxContext* aContext, const nsIntRegion& aRegion)
 {
   ClipToRegionInternal(aContext, aRegion, false);
 }
 
+/*static*/ void
+gfxUtils::ClipToRegion(DrawTarget* aTarget, const nsIntRegion& aRegion)
+{
+  ClipToRegionInternal(aTarget, aRegion, false);
+}
+
 /*static*/ void
 gfxUtils::ClipToRegionSnapped(gfxContext* aContext, const nsIntRegion& aRegion)
 {
   ClipToRegionInternal(aContext, aRegion, true);
 }
 
+/*static*/ void
+gfxUtils::ClipToRegionSnapped(DrawTarget* aTarget, const nsIntRegion& aRegion)
+{
+  ClipToRegionInternal(aTarget, aRegion, true);
+}
+
 /*static*/ gfxFloat
 gfxUtils::ClampToScaleFactor(gfxFloat aVal)
 {
@@ -631,6 +650,29 @@ gfxUtils::PathFromRegionSnapped(gfxContext* aContext, const nsIntRegion& aRegion
   PathFromRegionInternal(aContext, aRegion, true);
 }
 
+gfxMatrix
+gfxUtils::TransformRectToRect(const gfxRect& aFrom, const gfxPoint& aToTopLeft,
+                              const gfxPoint& aToTopRight, const gfxPoint& aToBottomRight)
+{
+  gfxMatrix m;
+  if (aToTopRight.y == aToTopLeft.y && aToTopRight.x == aToBottomRight.x) {
+    // Not a rotation, so xy and yx are zero
+    m.xy = m.yx = 0.0;
+    m.xx = (aToBottomRight.x - aToTopLeft.x)/aFrom.width;
+    m.yy = (aToBottomRight.y - aToTopLeft.y)/aFrom.height;
+    m.x0 = aToTopLeft.x - m.xx*aFrom.x;
+    m.y0 = aToTopLeft.y - m.yy*aFrom.y;
+  } else {
+    NS_ASSERTION(aToTopRight.y == aToBottomRight.y && aToTopRight.x == aToTopLeft.x,
+                 "Destination rectangle not axis-aligned");
+    m.xx = m.yy = 0.0;
+    m.xy = (aToBottomRight.x - aToTopLeft.x)/aFrom.height;
+    m.yx = (aToBottomRight.y - aToTopLeft.y)/aFrom.width;
+    m.x0 = aToTopLeft.x - m.xy*aFrom.y;
+    m.y0 = aToTopLeft.y - m.yx*aFrom.x;
+  }
+  return m;
+}
 
 bool
 gfxUtils::GfxRectToIntRect(const gfxRect& aIn, nsIntRect* aOut)
@@ -641,8 +683,8 @@ gfxUtils::GfxRectToIntRect(const gfxRect& aIn, nsIntRect* aOut)
 }
 
 void
-gfxUtils::GetYCbCrToRGBDestFormatAndSize(const PlanarYCbCrImage::Data& aData,
-                                         gfxASurface::gfxImageFormat& aSuggestedFormat,
+gfxUtils::GetYCbCrToRGBDestFormatAndSize(const PlanarYCbCrData& aData,
+                                         gfxImageFormat& aSuggestedFormat,
                                          gfxIntSize& aSuggestedSize)
 {
   gfx::YUVType yuvtype =
@@ -656,7 +698,7 @@ gfxUtils::GetYCbCrToRGBDestFormatAndSize(const PlanarYCbCrImage::Data& aData,
   bool prescale = aSuggestedSize.width > 0 && aSuggestedSize.height > 0 &&
                     aSuggestedSize != aData.mPicSize;
 
-  if (aSuggestedFormat == gfxASurface::ImageFormatRGB16_565) {
+  if (aSuggestedFormat == gfxImageFormatRGB16_565) {
 #if defined(HAVE_YCBCR_TO_RGB565)
     if (prescale &&
         !gfx::IsScaleYCbCrToRGB565Fast(aData.mPicX,
@@ -676,14 +718,14 @@ gfxUtils::GetYCbCrToRGBDestFormatAndSize(const PlanarYCbCrImage::Data& aData,
     }
 #else
     // yuv2rgb16 function not available
-    aSuggestedFormat = gfxASurface::ImageFormatRGB24;
+    aSuggestedFormat = gfxImageFormatRGB24;
 #endif
   }
-  else if (aSuggestedFormat != gfxASurface::ImageFormatRGB24) {
+  else if (aSuggestedFormat != gfxImageFormatRGB24) {
     // No other formats are currently supported.
-    aSuggestedFormat = gfxASurface::ImageFormatRGB24;
+    aSuggestedFormat = gfxImageFormatRGB24;
   }
-  if (aSuggestedFormat == gfxASurface::ImageFormatRGB24) {
+  if (aSuggestedFormat == gfxImageFormatRGB24) {
     /* ScaleYCbCrToRGB32 does not support a picture offset, nor 4:4:4 data.
        See bugs 639415 and 640073. */
     if (aData.mPicX != 0 || aData.mPicY != 0 || yuvtype == gfx::YV24)
@@ -695,8 +737,8 @@ gfxUtils::GetYCbCrToRGBDestFormatAndSize(const PlanarYCbCrImage::Data& aData,
 }
 
 void
-gfxUtils::ConvertYCbCrToRGB(const PlanarYCbCrImage::Data& aData,
-                            const gfxASurface::gfxImageFormat& aDestFormat,
+gfxUtils::ConvertYCbCrToRGB(const PlanarYCbCrData& aData,
+                            const gfxImageFormat& aDestFormat,
                             const gfxIntSize& aDestSize,
                             unsigned char* aDestBuffer,
                             int32_t aStride)
@@ -716,7 +758,7 @@ gfxUtils::ConvertYCbCrToRGB(const PlanarYCbCrImage::Data& aData,
   // Convert from YCbCr to RGB now, scaling the image if needed.
   if (aDestSize != aData.mPicSize) {
 #if defined(HAVE_YCBCR_TO_RGB565)
-    if (aDestFormat == gfxASurface::ImageFormatRGB16_565) {
+    if (aDestFormat == gfxImageFormatRGB16_565) {
       gfx::ScaleYCbCrToRGB565(aData.mYChannel,
                               aData.mCbChannel,
                               aData.mCrChannel,
@@ -750,7 +792,7 @@ gfxUtils::ConvertYCbCrToRGB(const PlanarYCbCrImage::Data& aData,
                              gfx::FILTER_BILINEAR);
   } else { // no prescale
 #if defined(HAVE_YCBCR_TO_RGB565)
-    if (aDestFormat == gfxASurface::ImageFormatRGB16_565) {
+    if (aDestFormat == gfxImageFormatRGB16_565) {
       gfx::ConvertYCbCrToRGB565(aData.mYChannel,
                                 aData.mCbChannel,
                                 aData.mCrChannel,
@@ -763,7 +805,7 @@ gfxUtils::ConvertYCbCrToRGB(const PlanarYCbCrImage::Data& aData,
                                 aData.mCbCrStride,
                                 aStride,
                                 yuvtype);
-    } else // aDestFormat != gfxASurface::ImageFormatRGB16_565
+    } else // aDestFormat != gfxImageFormatRGB16_565
 #endif
       gfx::ConvertYCbCrToRGB32(aData.mYChannel,
                                aData.mCbChannel,
@@ -820,5 +862,5 @@ gfxUtils::CopyAsDataURL(DrawTarget* aDT)
 bool gfxUtils::sDumpPaintList = getenv("MOZ_DUMP_PAINT_LIST") != 0;
 bool gfxUtils::sDumpPainting = getenv("MOZ_DUMP_PAINT") != 0;
 bool gfxUtils::sDumpPaintingToFile = getenv("MOZ_DUMP_PAINT_TO_FILE") != 0;
-FILE *gfxUtils::sDumpPaintFile = NULL;
+FILE *gfxUtils::sDumpPaintFile = nullptr;
 #endif
