@@ -1,5 +1,6 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* This Source Code Form is subject to the terms of the Mozilla Public
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
+ * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
@@ -8,14 +9,17 @@
 **
 ** Author: Kipp E.B. Hickman
 */
+
+#include "jsprf.h"
+
 #include <stdarg.h>
 #include <stdio.h>
-#include <string.h>
 #include <stdlib.h>
-#include "jsprf.h"
-#include "jsutil.h"
+#include <string.h>
+
 #include "jspubtd.h"
 #include "jsstr.h"
+#include "jsutil.h"
 
 using namespace js;
 
@@ -360,23 +364,26 @@ static int cvt_s(SprintfState *ss, const char *s, int width, int prec,
     return fill2(ss, s ? s : "(null)", slen, width, flags);
 }
 
-static int cvt_ws(SprintfState *ss, const jschar *ws, int width, int prec,
-                  int flags)
+static int
+cvt_ws(SprintfState *ss, const jschar *ws, int width, int prec, int flags)
 {
     int result;
     /*
-     * Supply NULL as the JSContext; errors are not reported,
+     * Supply nullptr as the JSContext; errors are not reported,
      * and malloc() is used to allocate the buffer buffer.
      */
     if (ws) {
-        int slen = js_strlen(ws);
-        char *s = DeflateString(NULL, ws, slen);
-        if (!s)
+        size_t wslen = js_strlen(ws);
+        char *latin1 = js_pod_malloc<char>(wslen + 1);
+        if (!latin1)
             return -1; /* JSStuffFunc error indicator. */
-        result = cvt_s(ss, s, width, prec, flags);
-        js_free(s);
+        for (size_t i = 0; i < wslen; ++i)
+            latin1[i] = (char)ws[i];
+        latin1[wslen] = '\0';
+        result = cvt_s(ss, latin1, width, prec, flags);
+        js_free(latin1);
     } else {
-        result = cvt_s(ss, NULL, width, prec, flags);
+        result = cvt_s(ss, nullptr, width, prec, flags);
     }
     return result;
 }
@@ -415,13 +422,13 @@ static struct NumArgState* BuildArgArray( const char *fmt, va_list ap, int* rv,
                 if( c == '$' ){         /* numbered argument csae */
                     if( i > 0 ){
                         *rv = -1;
-                        return NULL;
+                        return nullptr;
                     }
                     number++;
                 } else {                /* non-numbered argument case */
                     if( number > 0 ){
                         *rv = -1;
-                        return NULL;
+                        return nullptr;
                     }
                     i = 1;
                 }
@@ -433,7 +440,7 @@ static struct NumArgState* BuildArgArray( const char *fmt, va_list ap, int* rv,
     }
 
     if( number == 0 ){
-        return NULL;
+        return nullptr;
     }
 
 
@@ -441,7 +448,7 @@ static struct NumArgState* BuildArgArray( const char *fmt, va_list ap, int* rv,
         nas = (struct NumArgState*)malloc( number * sizeof( struct NumArgState ) );
         if( !nas ){
             *rv = -1;
-            return NULL;
+            return nullptr;
         }
     } else {
         nas = nasArray;
@@ -593,7 +600,7 @@ static struct NumArgState* BuildArgArray( const char *fmt, va_list ap, int* rv,
     if( *rv < 0 ){
         if( nas != nasArray )
             js_free( nas );
-        return NULL;
+        return nullptr;
     }
 
     cn = 0;
@@ -631,7 +638,7 @@ static struct NumArgState* BuildArgArray( const char *fmt, va_list ap, int* rv,
             if( nas != nasArray )
                 js_free( nas );
             *rv = -1;
-            return NULL;
+            return nullptr;
         }
 
         cn++;
@@ -664,12 +671,10 @@ static int dosprintf(SprintfState *ss, const char *fmt, va_list ap)
     static const char HEX[] = "0123456789ABCDEF";
     const char *hexp;
     int rv, i;
-    struct NumArgState *nas = NULL;
+    struct NumArgState *nas = nullptr;
     struct NumArgState nasArray[ NAS_DEFAULT_NUM ];
     char pattern[20];
-    const char *dolPt = NULL;  /* in "%4$.2f", dolPt will poiont to . */
-    uint8_t utf8buf[6];
-    int utf8len;
+    const char *dolPt = nullptr;  /* in "%4$.2f", dolPt will poiont to . */
 
     /*
     ** build an argument array, IF the fmt is numbered argument
@@ -708,7 +713,7 @@ static int dosprintf(SprintfState *ss, const char *fmt, va_list ap)
             continue;
         }
 
-        if( nas != NULL ){
+        if( nas != nullptr ){
             /* the fmt contains the Numbered Arguments feature */
             i = 0;
             while( c && c != '$' ){         /* should imporve error check later */
@@ -880,7 +885,7 @@ static int dosprintf(SprintfState *ss, const char *fmt, va_list ap)
           case 'f':
           case 'g':
             u.d = va_arg(ap, double);
-            if( nas != NULL ){
+            if( nas != nullptr ){
                 i = fmt - dolPt;
                 if( i < (int)sizeof( pattern ) ){
                     pattern[0] = '%';
@@ -906,13 +911,6 @@ static int dosprintf(SprintfState *ss, const char *fmt, va_list ap)
             }
             switch (type) {
               case TYPE_INT16:
-                /* Treat %hc as %c unless js_CStringsAreUTF8. */
-                if (js_CStringsAreUTF8) {
-                    u.wch = va_arg(ap, int);
-                    utf8len = js_OneUcs4ToUtf8Char (utf8buf, u.wch);
-                    rv = (*ss->stuff)(ss, (char *)utf8buf, utf8len);
-                    break;
-                }
               case TYPE_INTN:
                 u.ch = va_arg(ap, int);
                 rv = (*ss->stuff)(ss, &u.ch, 1);
@@ -957,10 +955,6 @@ static int dosprintf(SprintfState *ss, const char *fmt, va_list ap)
 
           case 's':
             if(type == TYPE_INT16) {
-                /*
-                 * This would do a simple string/byte conversion
-                 * unless js_CStringsAreUTF8.
-                 */
                 u.ws = va_arg(ap, const jschar*);
                 rv = cvt_ws(ss, u.ws, width, prec, flags);
             } else {
@@ -1059,11 +1053,7 @@ static int GrowStuff(SprintfState *ss, const char *sp, uint32_t len)
     if (off + len >= ss->maxlen) {
         /* Grow the buffer */
         newlen = ss->maxlen + ((len > 32) ? len : 32);
-        if (ss->base) {
-            newbase = (char*) js_realloc(ss->base, newlen);
-        } else {
-            newbase = (char*) js_malloc(newlen);
-        }
+        newbase = (char*) js_realloc(ss->base, newlen);
         if (!newbase) {
             /* Ran out of memory */
             return -1;
@@ -1115,9 +1105,7 @@ JS_PUBLIC_API(char *) JS_vsmprintf(const char *fmt, va_list ap)
     ss.maxlen = 0;
     rv = dosprintf(&ss, fmt, ap);
     if (rv < 0) {
-        if (ss.base) {
-            js_free(ss.base);
-        }
+        js_free(ss.base);
         return 0;
     }
     return ss.base;
@@ -1214,9 +1202,7 @@ JS_PUBLIC_API(char *) JS_vsprintf_append(char *last, const char *fmt, va_list ap
     }
     rv = dosprintf(&ss, fmt, ap);
     if (rv < 0) {
-        if (ss.base) {
-            js_free(ss.base);
-        }
+        js_free(ss.base);
         return 0;
     }
     return ss.base;
