@@ -80,22 +80,16 @@
 #
 # - There are many features of IDL that XPConnect supports but qsgen does not,
 #   including dependent types, arrays, and out parameters.
-#
-# - Since quick stubs are JSPropertyOps, we have to do additional work to make
-#   __lookup[GS]etter__ work on them.
 
 
 import xpidl
 import header
+import makeutils
 import os, re
 import sys
 
 # === Preliminaries
 
-# --makedepend-output support.
-make_dependencies = []
-make_targets = []
-
 def warn(msg):
     sys.stderr.write(msg + '\n')
 
@@ -153,7 +147,7 @@ def findIDL(includePath, irregularFilenames, interfaceName):
                     % (interfaceName, includePath))
 
 def loadIDL(parser, includePath, filename):
-    make_dependencies.append(filename)
+    makeutils.dependencies.append(filename)
     text = open(filename, 'r').read()
     idl = parser.parse(text, filename=filename)
     idl.resolve(includePath, parser)
@@ -232,8 +226,8 @@ class Configuration:
         # optional settings
         self.irregularFilenames = config.get('irregularFilenames', {})
         self.customIncludes = config.get('customIncludes', [])
-        self.customReturnInterfaces = config.get('customReturnInterfaces', [])
         self.customMethodCalls = config.get('customMethodCalls', {})
+        self.newBindingProperties = config.get('newBindingProperties', {})
 
 def readConfigFile(filename, includePath, cachedir):
     # Read the config file.
@@ -257,6 +251,7 @@ def readConfigFile(filename, includePath, cachedir):
                                 % (interfaceName, idlFile))
             iface = idl.getName(interfaceName, errorLoc)
             iface.stubMembers = []
+            iface.newBindingProperties = 'nullptr'
             interfaces.append(iface)
             interfacesByName[interfaceName] = iface
         return iface
@@ -275,8 +270,7 @@ def readConfigFile(filename, includePath, cachedir):
         iface = getInterface(interfaceName, errorLoc='looking for %r' % memberId)
 
         if not iface.attributes.scriptable:
-            raise UserError("Interface %s is not scriptable. "
-                            "IDL file: %r." % (interfaceName, idlFile))
+            raise UserError("Interface %s is not scriptable." % interfaceName)
 
         if memberName == '*':
             if not add:
@@ -285,8 +279,6 @@ def readConfigFile(filename, includePath, cachedir):
             # Stub all scriptable members of this interface.
             for member in iface.members:
                 if member.kind in ('method', 'attribute') and not member.noscript:
-                    cmc = conf.customMethodCalls.get(interfaceName + "_" + header.methodNativeName(member), None)
-
                     addStubMember(iface.name + '.' + member.name, member)
 
                     if member.iface not in stubbedInterfaces:
@@ -304,23 +296,23 @@ def readConfigFile(filename, includePath, cachedir):
                     raise UserError("Member %s is specified more than once."
                                     % memberId)
 
-                cmc = conf.customMethodCalls.get(interfaceName + "_" + header.methodNativeName(member), None)
-
                 addStubMember(memberId, member)
                 if member.iface not in stubbedInterfaces:
                     stubbedInterfaces.append(member.iface)
             else:
                 removeStubMember(memberId, member)
 
+    for (interfaceName, v) in conf.newBindingProperties.iteritems():
+        iface = getInterface(interfaceName, errorLoc='looking for %r' % interfaceName)
+        iface.newBindingProperties = v
+        if iface not in stubbedInterfaces:
+            stubbedInterfaces.append(iface)
+
     # Now go through and check all the interfaces' members
     for iface in stubbedInterfaces:
         for member in iface.stubMembers:
             checkStubMember(member)
 
-    for iface in conf.customReturnInterfaces:
-        # just ensure that it exists so that we can grab it later
-        iface = getInterface(iface, errorLoc='looking for %s' % (iface,))
-
     return conf, interfaces
 
 
@@ -335,7 +327,7 @@ def writeHeaderFile(filename, name):
         f.write("/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */\n"
                 "#ifndef " + headerMacro + "\n"
                 "#define " + headerMacro + "\n\n"
-                "JSBool " + name + "_DefineQuickStubs("
+                "bool " + name + "_DefineQuickStubs("
                 "JSContext *cx, JSObject *proto, unsigned flags, "
                 "uint32_t count, const nsID **iids);\n\n"
                 "void " + name + "_MarkInterfaces();\n\n"
@@ -381,6 +373,7 @@ class StringTable:
                     % (offset, explodeToCharArray(string)))
         f.write("  /* %5d */ %s, '\\0' };\n\n"
                 % (entries[-1][1], explodeToCharArray(entries[-1][0])))
+        f.write("const char* xpc_qsStringTable = %s;\n\n" % name);
 
 def substitute(template, vals):
     """ Simple replacement for string.Template, which isn't in Python 2.3. """
@@ -393,91 +386,92 @@ argumentUnboxingTemplates = {
     'octet':
         "    uint32_t ${name}_u32;\n"
         "    if (!JS_ValueToECMAUint32(cx, ${argVal}, &${name}_u32))\n"
-        "        return JS_FALSE;\n"
+        "        return false;\n"
         "    uint8_t ${name} = (uint8_t) ${name}_u32;\n",
 
     'short':
         "    int32_t ${name}_i32;\n"
-        "    if (!JS_ValueToECMAInt32(cx, ${argVal}, &${name}_i32))\n"
-        "        return JS_FALSE;\n"
+        "    if (!JS::ToInt32(cx, ${argVal}, &${name}_i32))\n"
+        "        return false;\n"
         "    int16_t ${name} = (int16_t) ${name}_i32;\n",
 
     'unsigned short':
         "    uint32_t ${name}_u32;\n"
         "    if (!JS_ValueToECMAUint32(cx, ${argVal}, &${name}_u32))\n"
-        "        return JS_FALSE;\n"
+        "        return false;\n"
         "    uint16_t ${name} = (uint16_t) ${name}_u32;\n",
 
     'long':
         "    int32_t ${name};\n"
-        "    if (!JS_ValueToECMAInt32(cx, ${argVal}, &${name}))\n"
-        "        return JS_FALSE;\n",
+        "    if (!JS::ToInt32(cx, ${argVal}, &${name}))\n"
+        "        return false;\n",
 
     'unsigned long':
         "    uint32_t ${name};\n"
         "    if (!JS_ValueToECMAUint32(cx, ${argVal}, &${name}))\n"
-        "        return JS_FALSE;\n",
+        "        return false;\n",
 
     'long long':
         "    int64_t ${name};\n"
         "    if (!JS::ToInt64(cx, ${argVal}, &${name}))\n"
-        "        return JS_FALSE;\n",
+        "        return false;\n",
 
     'unsigned long long':
         "    uint64_t ${name};\n"
         "    if (!JS::ToUint64(cx, ${argVal}, &${name}))\n"
-        "        return JS_FALSE;\n",
+        "        return false;\n",
 
     'float':
         "    double ${name}_dbl;\n"
-        "    if (!JS_ValueToNumber(cx, ${argVal}, &${name}_dbl))\n"
-        "        return JS_FALSE;\n"
+        "    if (!JS::ToNumber(cx, ${argVal}, &${name}_dbl))\n"
+        "        return false;\n"
         "    float ${name} = (float) ${name}_dbl;\n",
 
     'double':
         "    double ${name};\n"
-        "    if (!JS_ValueToNumber(cx, ${argVal}, &${name}))\n"
-        "        return JS_FALSE;\n",
+        "    if (!JS::ToNumber(cx, ${argVal}, &${name}))\n"
+        "        return false;\n",
 
     'boolean':
-        "    JSBool ${name};\n"
+        "    bool ${name};\n"
         "    JS_ValueToBoolean(cx, ${argVal}, &${name});\n",
 
     '[astring]':
-        "    xpc_qsAString ${name}(cx, ${argVal}, ${argPtr});\n"
+        "    xpc_qsAString ${name}(cx, ${argVal}, ${argPtr}, ${notPassed});\n"
         "    if (!${name}.IsValid())\n"
-        "        return JS_FALSE;\n",
+        "        return false;\n",
 
     '[domstring]':
-        "    xpc_qsDOMString ${name}(cx, ${argVal}, ${argPtr},\n"
+        "    xpc_qsDOMString ${name}(cx, ${argVal},\n"
+        "                            ${argPtr}, ${notPassed},\n"
         "                            xpc_qsDOMString::e${nullBehavior},\n"
         "                            xpc_qsDOMString::e${undefinedBehavior});\n"
         "    if (!${name}.IsValid())\n"
-        "        return JS_FALSE;\n",
+        "        return false;\n",
 
     'string':
         "    JSAutoByteString ${name}_bytes;\n"
         "    if (!xpc_qsJsvalToCharStr(cx, ${argVal}, &${name}_bytes))\n"
-        "        return JS_FALSE;\n"
+        "        return false;\n"
         "    char *${name} = ${name}_bytes.ptr();\n",
 
     'wstring':
         "    const PRUnichar *${name};\n"
         "    if (!xpc_qsJsvalToWcharStr(cx, ${argVal}, ${argPtr}, &${name}))\n"
-        "        return JS_FALSE;\n",
+        "        return false;\n",
 
     '[cstring]':
-        "    xpc_qsACString ${name}(cx, ${argVal}, ${argPtr});\n"
+        "    xpc_qsACString ${name}(cx, ${argVal}, ${argPtr}, ${notPassed});\n"
         "    if (!${name}.IsValid())\n"
-        "        return JS_FALSE;\n",
+        "        return false;\n",
 
     '[utf8string]':
-        "    xpc_qsAUTF8String ${name}(cx, ${argVal}, ${argPtr});\n"
+        "    xpc_qsAUTF8String ${name}(cx, ${argVal}, ${argPtr}, ${notPassed});\n"
         "    if (!${name}.IsValid())\n"
-        "        return JS_FALSE;\n",
+        "        return false;\n",
 
     '[jsval]':
-        "    jsval ${name} = ${argVal};\n"
+        "    JS::RootedValue ${name}(cx, ${argVal});\n"
     }
 
 # From JSData2Native.
@@ -487,10 +481,11 @@ argumentUnboxingTemplates = {
 # however, defaults to 'undefined'.
 #
 def writeArgumentUnboxing(f, i, name, type, optional, rvdeclared,
-                          nullBehavior, undefinedBehavior):
+                          nullBehavior, undefinedBehavior,
+                          propIndex=None):
     # f - file to write to
     # i - int or None - Indicates the source jsval.  If i is an int, the source
-    #     jsval is argv[i]; otherwise it is *vp.  But if Python i >= C++ argc,
+    #     jsval is args[i]; otherwise it is args[0].  But if Python i >= C++ argc,
     #     which can only happen if optional is True, the argument is missing;
     #     use JSVAL_NULL as the source jsval instead.
     # name - str - name of the native C++ variable to create.
@@ -502,24 +497,42 @@ def writeArgumentUnboxing(f, i, name, type, optional, rvdeclared,
 
     isSetter = (i is None)
 
+    notPassed = "false"
     if isSetter:
-        argPtr = "vp"
-        argVal = "*vp"
+        argPtr = "args[0]"
+        argVal = "args[0]"
     elif optional:
         if typeName == "[jsval]":
-            val = "JSVAL_VOID"
+            val = "JS::UndefinedHandleValue"
         else:
-            val = "JSVAL_NULL"
-        argVal = "(%d < argc ? argv[%d] : %s)" % (i, i, val)
-        argPtr = "(%d < argc ? &argv[%d] : NULL)" % (i, i)
+            val = "JS::NullHandleValue"
+        argVal = "(%d < argc ? args[%d] : %s)" % (i, i, val)
+        if typeName == "[jsval]":
+            # This should use the rooted argument,
+            # however we probably won't ever need to support that.
+            argPtr = None
+            notPassed = None
+        else:
+            # Need a MutableHandleValue to pass into eg the xpc_qsAString
+            # constructor, but the corresponding argument may not have been
+            # passed in at all. In that case, point the MutableHandleValue at a
+            # dummy variable, and pass in a boolean saying that the argument
+            # wasn't passed (previously, this used a NULL ptr sentinel value.)
+            f.write("    JS::RootedValue {name}_dummy(cx);\n".format(name=name))
+            f.write("    JS::MutableHandleValue {name}_mhv({i} < argc ? args[{i}] : &{name}_dummy);\n".format(name=name, i=i))
+            f.write("    (void) {name}_mhv;\n".format(name=name, i=i))
+
+            argPtr = "{name}_mhv".format(name=name)
+            notPassed = "argc < {i}".format(i=i)
     else:
-        argVal = "argv[%d]" % i
-        argPtr = "&" + argVal
+        argVal = "args[%d]" % i
+        argPtr = "args[%d]" % i
 
     params = {
         'name': name,
         'argVal': argVal,
         'argPtr': argPtr,
+        'notPassed': notPassed,
         'nullBehavior': nullBehavior or 'DefaultNullBehavior',
         'undefinedBehavior': undefinedBehavior or 'DefaultUndefinedBehavior'
         }
@@ -538,7 +551,7 @@ def writeArgumentUnboxing(f, i, name, type, optional, rvdeclared,
                 "XPCVariant::newVariant(cx, ${argVal})));\n"
                 "    if (!${name}) {\n"
                 "        xpc_qsThrowBadArg(cx, NS_ERROR_INVALID_ARG, vp, %d);\n"
-                "        return JS_FALSE;\n"
+                "        return false;\n"
                 "    }") % i
             f.write(substitute(template, params))
             return rvdeclared
@@ -555,19 +568,20 @@ def writeArgumentUnboxing(f, i, name, type, optional, rvdeclared,
                     % (type.name, argVal, name, name, argPtr))
             f.write("    if (NS_FAILED(rv)) {\n")
             if isSetter:
-                f.write("        xpc_qsThrowBadSetterValue("
-                        "cx, rv, JSVAL_TO_OBJECT(*tvr.jsval_addr()), id);\n")
+                assert(propIndex is not None)
+                f.write("        xpc_qsThrowBadSetterValue(cx, rv, JSVAL_TO_OBJECT(vp[1]), (uint16_t)%s);\n" %
+                        propIndex)
             else:
                 f.write("        xpc_qsThrowBadArg(cx, rv, vp, %d);\n" % i)
-            f.write("        return JS_FALSE;\n"
+            f.write("        return false;\n"
                     "    }\n")
             return True
 
     warn("Unable to unbox argument of type %s (native type %s)" % (type.name, typeName))
     if i is None:
-        src = '*vp'
+        src = 'args[0]'
     else:
-        src = 'argv[%d]' % i
+        src = 'args[%d]' % i
     f.write("    !; // TODO - Unbox argument %s = %s\n" % (name, src))
     return rvdeclared
 
@@ -590,7 +604,7 @@ def writeResultDecl(f, type, varname):
             f.write("    nsString %s;\n" % varname)
             return
         elif name == '[jsval]':
-            f.write("    jsval %s;\n" % varname)
+            f.write("    JS::RootedValue %s(cx);\n" % varname)
             return
     elif t.kind in ('interface', 'forward'):
         f.write("    nsCOMPtr<%s> %s;\n" % (type.name, varname))
@@ -605,7 +619,7 @@ def outParamForm(name, type):
         return '&' + name
     elif type.kind == 'native':
         if getBuiltinOrNativeTypeName(type) == '[jsval]':
-            return '&' + name
+            return name + '.address()'
         elif type.modifier == 'ref':
             return name
         else:
@@ -617,45 +631,45 @@ def outParamForm(name, type):
 resultConvTemplates = {
     'void':
             "    ${jsvalRef} = JSVAL_VOID;\n"
-            "    return JS_TRUE;\n",
+            "    return true;\n",
 
     'octet':
         "    ${jsvalRef} = INT_TO_JSVAL((int32_t) result);\n"
-        "    return JS_TRUE;\n",
+        "    return true;\n",
 
     'short':
         "    ${jsvalRef} = INT_TO_JSVAL((int32_t) result);\n"
-        "    return JS_TRUE;\n",
+        "    return true;\n",
 
     'long':
         "    ${jsvalRef} = INT_TO_JSVAL(result);\n"
-        "    return JS_TRUE;\n",
+        "    return true;\n",
 
     'long long':
         "    return xpc_qsInt64ToJsval(cx, result, ${jsvalPtr});\n",
 
     'unsigned short':
         "    ${jsvalRef} = INT_TO_JSVAL((int32_t) result);\n"
-        "    return JS_TRUE;\n",
+        "    return true;\n",
 
     'unsigned long':
         "    ${jsvalRef} = UINT_TO_JSVAL(result);\n"
-        "    return JS_TRUE;\n",
+        "    return true;\n",
 
     'unsigned long long':
         "    return xpc_qsUint64ToJsval(cx, result, ${jsvalPtr});\n",
 
     'float':
         "    ${jsvalRef} = JS_NumberValue(result);\n"
-        "    return JS_TRUE;\n",
+        "    return true;\n",
 
     'double':
         "    ${jsvalRef} =  JS_NumberValue(result);\n"
-        "    return JS_TRUE;\n",
+        "    return true;\n",
 
     'boolean':
         "    ${jsvalRef} = (result ? JSVAL_TRUE : JSVAL_FALSE);\n"
-        "    return JS_TRUE;\n",
+        "    return true;\n",
 
     '[astring]':
         "    return xpc::StringToJsval(cx, result, ${jsvalPtr});\n",
@@ -664,7 +678,7 @@ resultConvTemplates = {
         "    return xpc::StringToJsval(cx, result, ${jsvalPtr});\n",
 
     '[jsval]':
-        "    ${jsvalRef} = result;\n"
+        "    ${jsvalPtr}.set(result);\n"
         "    return JS_WrapValue(cx, ${jsvalPtr});\n"
     }
 
@@ -674,8 +688,8 @@ def isVariantType(t):
 def writeResultConv(f, type, jsvalPtr, jsvalRef):
     """ Emit code to convert the C++ variable `result` to a jsval.
 
-    The emitted code contains a return statement; it returns JS_TRUE on
-    success, JS_FALSE on error.
+    The emitted code contains a return statement; it returns true on
+    success, false on error.
     """
     # From NativeData2JS.
     typeName = getBuiltinOrNativeTypeName(type)
@@ -689,21 +703,21 @@ def writeResultConv(f, type, jsvalPtr, jsvalRef):
         # else fall through; this type isn't supported yet
     elif isInterfaceType(type):
         if isVariantType(type):
-            f.write("    return xpc_qsVariantToJsval(lccx, result, %s);\n"
+            f.write("    return xpc_qsVariantToJsval(cx, result, %s);\n"
                     % jsvalPtr)
             return
         else:
             f.write("    if (!result) {\n"
-                    "      *%s = JSVAL_NULL;\n"
-                    "      return JS_TRUE;\n"
+                    "      %s.setNull();\n"
+                    "      return true;\n"
                     "    }\n"
                     "    nsWrapperCache* cache = xpc_qsGetWrapperCache(result);\n"
                     "    if (xpc_FastGetCachedWrapper(cache, obj, %s)) {\n"
-                    "      return JS_TRUE;\n"
+                    "      return true;\n"
                     "    }\n"
                     "    // After this point do not use 'result'!\n"
                     "    qsObjectHelper helper(result, cache);\n"
-                    "    return xpc_qsXPCOMObjectToJsval(lccx, "
+                    "    return xpc_qsXPCOMObjectToJsval(cx, "
                     "helper, &NS_GET_IID(%s), &interfaces[k_%s], %s);\n"
                     % (jsvalPtr, jsvalPtr, type.name, type.name, jsvalPtr))
             return
@@ -745,7 +759,8 @@ def setOptimizationForMSVC(f, b):
     f.write('# pragma optimize("", %s)\n'%pragmaParam)
     f.write("#endif\n")
 
-def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
+def writeQuickStub(f, customMethodCalls, stringtable, member, stubName,
+                   isSetter=False):
     """ Write a single quick stub (a custom SpiderMonkey getter/setter/method)
     for the specified XPCOM interface-member. 
     """
@@ -758,16 +773,8 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
     assert isAttr or isMethod
     isGetter = isAttr and not isSetter
 
-    signature = "static JSBool\n"
-    if isAttr:
-        # JSPropertyOp signature.
-        if isSetter:
-            signature += "%s(JSContext *cx, JSHandleObject obj, JSHandleId id, JSBool strict,%s JSMutableHandleValue vp_)\n"
-        else:
-            signature += "%s(JSContext *cx, JSHandleObject obj, JSHandleId id,%s JSMutableHandleValue vp_)\n"
-    else:
-        # JSFastNative.
-        signature += "%s(JSContext *cx, unsigned argc,%s jsval *vp)\n"
+    signature = ("static bool\n" +
+                 "%s(JSContext *cx, unsigned argc,%s jsval *vp)\n")
 
     customMethodCall = customMethodCalls.get(stubName, None)
 
@@ -800,12 +807,8 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
                           or header.firstCap(member.name))
             argumentValues = (customMethodCall['additionalArgumentValues']
                               % nativeName)
-            if isAttr:
-                callTemplate += ("    return %s(cx, obj, id%s, %s, vp_);\n"
-                                 % (templateName, ", strict" if isSetter else "", argumentValues))
-            else:
-                callTemplate += ("    return %s(cx, argc, %s, vp);\n"
-                                 % (templateName, argumentValues))
+            callTemplate += ("    return %s(cx, argc, %s, vp);\n"
+                             % (templateName, argumentValues))
             callTemplate += "}\n\n"
 
             # Fall through and create the template function stub called from the
@@ -843,15 +846,14 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
     f.write("{\n")
     f.write("    XPC_QS_ASSERT_CONTEXT_OK(cx);\n")
 
-    # Convert JSMutableHandleValue to jsval*
-    if isAttr:
-        f.write("    jsval *vp = vp_.address();\n")
+    # Compute "args".
+    f.write("    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);\n")
+    f.write("    (void) args;\n")
 
-    # For methods, compute "this".
-    if isMethod:
-        f.write("    JSObject *obj = JS_THIS_OBJECT(cx, vp);\n"
-                "    if (!obj)\n"
-                "        return JS_FALSE;\n")
+    # Compute "this".
+    f.write("    JS::RootedObject obj(cx, JS_THIS_OBJECT(cx, vp));\n"
+            "    if (!obj)\n"
+            "        return false;\n")
 
     # Get the 'self' pointer.
     if customMethodCall is None or not 'thisType' in customMethodCall:
@@ -859,33 +861,22 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
     else:
         f.write("    %s *self;\n" % customMethodCall['thisType'])
     f.write("    xpc_qsSelfRef selfref;\n")
-    if isGetter:
-        pthisval = 'vp'
-    elif isSetter:
-        f.write("    JS::AutoValueRooter tvr(cx);\n")
-        pthisval = 'tvr.jsval_addr()'
-    else:
-        pthisval = '&vp[1]' # as above, ok to overwrite vp[1]
+    pthisval = 'JS::MutableHandleValue::fromMarkedLocation(&vp[1])' # as above, ok to overwrite vp[1]
 
     if unwrapThisFailureFatal:
         unwrapFatalArg = "true"
     else:
         unwrapFatalArg = "false"
 
-    if not isSetter and isInterfaceType(member.realtype):
-        f.write("    XPCLazyCallContext lccx(JS_CALLER, cx, obj);\n")
-        f.write("    if (!xpc_qsUnwrapThis(cx, obj, &self, "
-                "&selfref.ptr, %s, &lccx, %s))\n" % (pthisval, unwrapFatalArg))
-    else:
-        f.write("    if (!xpc_qsUnwrapThis(cx, obj, &self, "
-                "&selfref.ptr, %s, nullptr, %s))\n" % (pthisval, unwrapFatalArg))
-    f.write("        return JS_FALSE;\n")
+    f.write("    if (!xpc_qsUnwrapThis(cx, obj, &self, "
+            "&selfref.ptr, %s, %s))\n" % (pthisval, unwrapFatalArg))
+    f.write("        return false;\n")
 
     if not unwrapThisFailureFatal:
         f.write("      if (!self) {\n")
         if (isGetter):
-            f.write("        *vp = JSVAL_NULL;\n")
-        f.write("        return JS_TRUE;\n")
+            f.write("        args.rval().setNull();\n")
+        f.write("        return true;\n")
         f.write("    }\n");
 
     if isMethod:
@@ -893,16 +884,18 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
         requiredArgs = len(member.params)
         while requiredArgs and member.params[requiredArgs-1].optional:
             requiredArgs -= 1
-        if requiredArgs:
-            f.write("    if (argc < %d)\n" % requiredArgs)
-            f.write("        return xpc_qsThrow(cx, "
-                    "NS_ERROR_XPC_NOT_ENOUGH_ARGS);\n")
+    elif isSetter:
+        requiredArgs = 1
+    else:
+        requiredArgs = 0
+    if requiredArgs:
+        f.write("    if (argc < %d)\n" % requiredArgs)
+        f.write("        return xpc_qsThrow(cx, "
+                "NS_ERROR_XPC_NOT_ENOUGH_ARGS);\n")
 
     # Convert in-parameters.
     rvdeclared = False
     if isMethod:
-        if len(member.params) > 0:
-            f.write("    jsval *argv = JS_ARGV(cx, vp);\n")
         for i, param in enumerate(member.params):
             argName = 'arg%d' % i
             argTypeKey = argName + 'Type'
@@ -924,7 +917,8 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
                                            optional=False,
                                            rvdeclared=rvdeclared,
                                            nullBehavior=member.null,
-                                           undefinedBehavior=member.undefined)
+                                           undefinedBehavior=member.undefined,
+                                           propIndex=stringtable.stringIndex(member.name))
 
     canFail = customMethodCall is None or customMethodCall.get('canFail', True)
     if canFail and not rvdeclared:
@@ -961,7 +955,7 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
             if member.implicit_jscontext:
                 argv.append('cx')
             if member.optional_argc:
-                argv.append('NS_MIN<uint32_t>(argc, %d) - %d' % 
+                argv.append('std::min<uint32_t>(argc, %d) - %d' %
                             (len(member.params), requiredArgs))
             if not isVoidType(member.realtype):
                 argv.append(outParamForm(resultname, member.realtype))
@@ -984,9 +978,9 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
             checkSuccess = "NS_SUCCEEDED(debug_rv)"
             if canFail:
                 checkSuccess += " == NS_SUCCEEDED(rv)"
-            f.write("    NS_ASSERTION(%s && "
+            f.write("    MOZ_ASSERT(%s && "
                     "xpc_qsSameResult(debug_result, result),\n"
-                    "                 \"Got the wrong answer from the custom "
+                    "               \"Got the wrong answer from the custom "
                     "method call!\");\n" % checkSuccess)
             f.write("#endif\n")
 
@@ -997,18 +991,15 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
             f.write("        return xpc_qsThrowMethodFailed("
                     "cx, rv, vp);\n")
         else:
-            if isGetter:
-                thisval = '*vp'
-            else:
-                thisval = '*tvr.jsval_addr()'
             f.write("        return xpc_qsThrowGetterSetterFailed(cx, rv, " +
-                    "JSVAL_TO_OBJECT(%s), id);\n" % thisval)
+                    "JSVAL_TO_OBJECT(vp[1]), (uint16_t)%d);\n" %
+                    stringtable.stringIndex(member.name))
 
     # Convert the return value.
     if isMethod or isGetter:
-        writeResultConv(f, member.realtype, 'vp', '*vp')
+        writeResultConv(f, member.realtype, 'args.rval()', '*vp')
     else:
-        f.write("    return JS_TRUE;\n")
+        f.write("    return true;\n")
 
     # Epilog.
     f.write("}\n")
@@ -1023,13 +1014,14 @@ def writeQuickStub(f, customMethodCalls, member, stubName, isSetter=False):
 def writeAttrStubs(f, customMethodCalls, stringtable, attr):
     getterName = (attr.iface.name + '_'
                   + header.attributeNativeName(attr, True))
-    writeQuickStub(f, customMethodCalls, attr, getterName)
+    writeQuickStub(f, customMethodCalls, stringtable, attr, getterName)
     if attr.readonly:
-        setterName = 'xpc_qsGetterOnlyPropertyStub'
+        setterName = 'xpc_qsGetterOnlyNativeStub'
     else:
         setterName = (attr.iface.name + '_'
                       + header.attributeNativeName(attr, False))
-        writeQuickStub(f, customMethodCalls, attr, setterName, isSetter=True)
+        writeQuickStub(f, customMethodCalls, stringtable, attr, setterName,
+                       isSetter=True)
 
     ps = ('{%d, %s, %s}'
           % (stringtable.stringIndex(attr.name), getterName, setterName))
@@ -1039,7 +1031,7 @@ def writeMethodStub(f, customMethodCalls, stringtable, method):
     """ Write a method stub to `f`. Return an xpc_qsFunctionSpec initializer. """
 
     stubName = method.iface.name + '_' + header.methodNativeName(method)
-    writeQuickStub(f, customMethodCalls, method, stubName)
+    writeQuickStub(f, customMethodCalls, stringtable, method, stubName)
     fs = '{%d, %d, %s}' % (stringtable.stringIndex(method.name),
                            len(method.params), stubName)
     return fs
@@ -1126,7 +1118,7 @@ def writeDefiner(f, conf, stringtable, interfaces):
     for iface in interfaces:
         # This if-statement discards interfaces specified with
         # "nsInterfaceName.*" that don't have any stub-able members.
-        if iface.stubMembers:
+        if iface.stubMembers or iface.newBindingProperties:
             h = hashIID(iface.attributes.uuid)
             buckets[h % size].append(iface)
 
@@ -1143,7 +1135,7 @@ def writeDefiner(f, conf, stringtable, interfaces):
                 arraySize += 1
 
     entries = ["    {{0, 0, 0, {0, 0, 0, 0, 0, 0, 0, 0}}, "
-               "0, 0, 0, 0, XPC_QS_NULL_INDEX, XPC_QS_NULL_INDEX}"
+               "0, 0, 0, 0, nullptr, XPC_QS_NULL_INDEX, XPC_QS_NULL_INDEX}"
                for i in range(arraySize)]
     for i, bucket in enumerate(buckets):
         for j, iface in enumerate(bucket):
@@ -1191,9 +1183,10 @@ def writeDefiner(f, conf, stringtable, interfaces):
                 chain = str(k)
 
             # add entry
-            entry = "    /* %s */ {%s, %d, %d, %d, %d, %s, %s}" % (
+            entry = "    /* %s */ {%s, %d, %d, %d, %d, %s, %s, %s}" % (
                 iface.name, iid, prop_index, prop_n_entries,
-                func_index, func_n_entries, parentInterface, chain)
+                func_index, func_n_entries, iface.newBindingProperties,
+                parentInterface, chain)
             entries[entryIndexes[iface.attributes.uuid]] = entry
 
     f.write("static const xpc_qsHashEntry tableData[] = {\n")
@@ -1229,20 +1222,18 @@ stubTopTemplate = '''\
 /* THIS FILE IS AUTOGENERATED - DO NOT EDIT */
 #include "jsapi.h"
 #include "qsWinUndefs.h"
-#include "prtypes.h"
+#include "qsObjectHelper.h"
 #include "nsID.h"
 #include "%s"
-#include "nscore.h"
 #include "nsCOMPtr.h"
-#include "nsDependentString.h"
 #include "xpcprivate.h"  // for XPCCallContext
 #include "XPCQuickStubs.h"
-#include "nsWrapperCacheInlines.h"
+#include <algorithm>
 '''
 
 def writeStubFile(filename, headerFilename, conf, interfaces):
     print "Creating stub file", filename
-    make_targets.append(filename)
+    makeutils.targets.append(filename)
 
     f = open(filename, 'w')
     filesIncluded = set()
@@ -1279,11 +1270,9 @@ def writeStubFile(filename, headerFilename, conf, interfaces):
 
     try:
         f.write(stubTopTemplate % os.path.basename(headerFilename))
-        N = 256
         resulttypes = []
         for iface in interfaces:
             resulttypes.extend(writeIncludesForInterface(iface))
-        resulttypes.extend(conf.customReturnInterfaces)
         for customInclude in conf.customIncludes:
             f.write('#include "%s"\n' % customInclude)
         f.write("\n\n")
@@ -1295,23 +1284,6 @@ def writeStubFile(filename, headerFilename, conf, interfaces):
     finally:
         f.close()
 
-def makeQuote(filename):
-    return filename.replace(' ', '\\ ')  # enjoy!
-
-def writeMakeDependOutput(filename):
-    print "Creating makedepend file", filename
-    f = open(filename, 'w')
-    try:
-        if len(make_targets) > 0:
-            f.write("%s:" % makeQuote(make_targets[0]))
-            for filename in make_dependencies:
-                f.write(' \\\n\t\t%s' % makeQuote(filename))
-            f.write('\n\n')
-            for filename in make_targets[1:]:
-                f.write('%s: %s\n' % (makeQuote(filename), makeQuote(make_targets[0])))
-    finally:
-        f.close()
-
 def main():
     from optparse import OptionParser
     o = OptionParser(usage="usage: %prog [options] configfile")
@@ -1358,7 +1330,7 @@ def main():
                       conf, interfaces)
         writeHeaderFile(options.header_output, conf.name)
         if options.makedepend_output is not None:
-            writeMakeDependOutput(options.makedepend_output)
+            makeutils.writeMakeDependOutput(options.makedepend_output)
     except Exception, exc:
         if options.verbose_errors:
             raise
