@@ -3,74 +3,48 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/DebugOnly.h"
+
 #include "nsCOMPtr.h"
 #include "nsTextControlFrame.h"
-#include "nsIDocument.h"
-#include "nsIFormControl.h"
-#include "nsIServiceManager.h"
-#include "nsFrameSelection.h"
 #include "nsIPlaintextEditor.h"
-#include "nsEditorCID.h"
-#include "nsLayoutCID.h"
-#include "nsIDocumentEncoder.h"
 #include "nsCaret.h"
-#include "nsISelectionListener.h"
-#include "nsIController.h"
-#include "nsIControllers.h"
-#include "nsIControllerContext.h"
 #include "nsGenericHTMLElement.h"
+#include "nsIEditor.h"
 #include "nsIEditorIMESupport.h"
 #include "nsIPhonetic.h"
 #include "nsTextFragment.h"
-#include "nsIEditorObserver.h"
-#include "nsEditProperty.h"
 #include "nsIDOMHTMLTextAreaElement.h"
 #include "nsINameSpaceManager.h"
 #include "nsINodeInfo.h"
 #include "nsFormControlFrame.h" //for registering accesskeys
 
 #include "nsIContent.h"
-#include "nsIAtom.h"
 #include "nsPresContext.h"
 #include "nsRenderingContext.h"
 #include "nsGkAtoms.h"
 #include "nsLayoutUtils.h"
-#include "nsIComponentManager.h"
-#include "nsIView.h"
-#include "nsIViewManager.h"
-#include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMHTMLElement.h"
 #include "nsIPresShell.h"
 
-#include "nsBoxLayoutState.h"
-//for keylistener for "return" check
-#include "nsIDOMEventTarget.h"
-#include "nsIDocument.h" //observe documents to send onchangenotifications
-#include "nsIStyleSheet.h"//observe documents to send onchangenotifications
-#include "nsIStyleRule.h"//observe documents to send onchangenotifications
-#include "nsIDOMEventListener.h"//observe documents to send onchangenotifications
-#include "nsGUIEvent.h"
-
-#include "nsIDOMCharacterData.h" //for selection setting helper func
+#include <algorithm>
 #include "nsIDOMNodeList.h" //for selection setting helper func
 #include "nsIDOMRange.h" //for selection setting helper func
 #include "nsPIDOMWindow.h" //needed for notify selection changed to update the menus ect.
 #include "nsIDOMNode.h"
 
-#include "nsITransactionManager.h"
 #include "nsIDOMText.h" //for multiline getselection
-#include "nsNodeInfoManager.h"
-#include "nsContentCreatorFunctions.h"
-#include "nsINativeKeyBindings.h"
-#include "nsIJSContextStack.h"
 #include "nsFocusManager.h"
 #include "nsTextEditRules.h"
 #include "nsPresState.h"
 #include "nsContentList.h"
 #include "nsAttrValueInlines.h"
-
 #include "mozilla/Selection.h"
+#include "nsContentUtils.h"
+#include "nsCxPusher.h"
+#include "nsTextNode.h"
+#include "nsStyleSet.h"
 
 #define DEFAULT_COLUMN_WIDTH 20
 
@@ -95,7 +69,7 @@ NS_QUERYFRAME_TAIL_INHERITING(nsContainerFrame)
 a11y::AccType
 nsTextControlFrame::AccessibleType()
 {
-  return a11y::eHTMLTextFieldAccessible;
+  return a11y::eHTMLTextFieldType;
 }
 #endif
 
@@ -126,7 +100,7 @@ private:
 
 nsTextControlFrame::nsTextControlFrame(nsIPresShell* aShell, nsStyleContext* aContext)
   : nsContainerFrame(aContext)
-  , mUseEditor(false)
+  , mEditorHasBeenInitialized(false)
   , mIsProcessing(false)
 #ifdef DEBUG
   , mInEditorInitialization(false)
@@ -184,7 +158,7 @@ nsTextControlFrame::CalcIntrinsicSize(nsRenderingContext* aRenderingContext,
   aRenderingContext->SetFont(fontMet);
 
   lineHeight =
-    nsHTMLReflowState::CalcLineHeight(GetStyleContext(), NS_AUTOHEIGHT,
+    nsHTMLReflowState::CalcLineHeight(StyleContext(), NS_AUTOHEIGHT,
                                       aFontSizeInflation);
   charWidth = fontMet->AveCharWidth();
   charMaxAdvance = fontMet->MaxAdvance();
@@ -198,7 +172,7 @@ nsTextControlFrame::CalcIntrinsicSize(nsRenderingContext* aRenderingContext,
   // this if charMaxAdvance != charWidth; if they are equal, this is almost
   // certainly a fixed-width font.
   if (charWidth != charMaxAdvance) {
-    nscoord internalPadding = NS_MAX(0, charMaxAdvance -
+    nscoord internalPadding = std::max(0, charMaxAdvance -
                                         nsPresContext::CSSPixelsToAppUnits(4));
     nscoord t = nsPresContext::CSSPixelsToAppUnits(1); 
    // Round to a multiple of t
@@ -222,7 +196,7 @@ nsTextControlFrame::CalcIntrinsicSize(nsRenderingContext* aRenderingContext,
     // using percentage padding anyway.
     nsMargin childPadding;
     nsIFrame* firstChild = GetFirstPrincipalChild();
-    if (firstChild && firstChild->GetStylePadding()->GetPadding(childPadding)) {
+    if (firstChild && firstChild->StylePadding()->GetPadding(childPadding)) {
       aIntrinsicSize.width += childPadding.LeftRight();
     } else {
       NS_ERROR("Percentage padding on value div?");
@@ -231,7 +205,7 @@ nsTextControlFrame::CalcIntrinsicSize(nsRenderingContext* aRenderingContext,
 
   // Increment width with cols * letter-spacing.
   {
-    const nsStyleCoord& lsCoord = GetStyleText()->mLetterSpacing;
+    const nsStyleCoord& lsCoord = StyleText()->mLetterSpacing;
     if (eStyleUnit_Coord == lsCoord.GetUnit()) {
       nscoord letterSpacing = lsCoord.GetCoordValue();
       if (letterSpacing != 0) {
@@ -279,9 +253,7 @@ nsTextControlFrame::EnsureEditorInitialized()
   // never get used.  So, now this method is being called lazily only
   // when we actually need an editor.
 
-  // Check if this method has been called already.
-  // If so, just return early.
-  if (mUseEditor)
+  if (mEditorHasBeenInitialized)
     return NS_OK;
 
   nsIDocument* doc = mContent->GetCurrentDoc();
@@ -334,9 +306,9 @@ nsTextControlFrame::EnsureEditorInitialized()
     NS_ENSURE_SUCCESS(rv, rv);
     NS_ENSURE_STATE(weakFrame.IsAlive());
 
-    // Turn on mUseEditor so that subsequent calls will use the
+    // Set mEditorHasBeenInitialized so that subsequent calls will use the
     // editor.
-    mUseEditor = true;
+    mEditorHasBeenInitialized = true;
 
     // Set the selection to the beginning of the text field.
     if (weakFrame.IsAlive()) {
@@ -385,7 +357,7 @@ nsTextControlFrame::CreateAnonymousContent(nsTArray<ContentInfo>& aElements)
 
     nsRefPtr<nsStyleContext> placeholderStyleContext =
       PresContext()->StyleSet()->ResolvePseudoElementStyle(
-          mContent->AsElement(), pseudoType, GetStyleContext());
+          mContent->AsElement(), pseudoType, StyleContext());
 
     if (!aElements.AppendElement(ContentInfo(placeholderNode,
                                  placeholderStyleContext))) {
@@ -486,7 +458,7 @@ nsTextControlFrame::ComputeAutoSize(nsRenderingContext *aRenderingContext,
   }
 #ifdef DEBUG
   // Note: Ancestor ComputeAutoSize only computes a width if we're auto-width
-  else if (GetStylePosition()->mWidth.GetUnit() == eStyleUnit_Auto) {
+  else if (StylePosition()->mWidth.GetUnit() == eStyleUnit_Auto) {
     nsSize ancestorAutoSize =
       nsContainerFrame::ComputeAutoSize(aRenderingContext,
                                         aCBSize, aAvailableWidth,
@@ -525,7 +497,7 @@ nsTextControlFrame::Reflow(nsPresContext*   aPresContext,
   nscoord lineHeight = aReflowState.ComputedHeight();
   float inflation = nsLayoutUtils::FontSizeInflationFor(this);
   if (!IsSingleLineTextControl()) {
-    lineHeight = nsHTMLReflowState::CalcLineHeight(GetStyleContext(), 
+    lineHeight = nsHTMLReflowState::CalcLineHeight(StyleContext(), 
                                                   NS_AUTOHEIGHT, inflation);
   }
   nsRefPtr<nsFontMetrics> fontMet;
@@ -565,8 +537,8 @@ nsTextControlFrame::ReflowTextControlChild(nsIFrame*                aKid,
   // compute available size and frame offsets for child
   nsSize availSize(aReflowState.ComputedWidth(), 
                    aReflowState.ComputedHeight());
-  availSize.width = NS_MAX(availSize.width, 0);
-  availSize.height = NS_MAX(availSize.height, 0);
+  availSize.width = std::max(availSize.width, 0);
+  availSize.height = std::max(availSize.height, 0);
   
   nsHTMLReflowState kidReflowState(aPresContext, aReflowState, 
                                    aKid, availSize);
@@ -575,13 +547,13 @@ nsTextControlFrame::ReflowTextControlChild(nsIFrame*                aKid,
   nscoord width = availSize.width;
   width -= kidReflowState.mComputedMargin.LeftRight() +
               kidReflowState.mComputedBorderPadding.LeftRight();
-  width = NS_MAX(width, 0);
+  width = std::max(width, 0);
   kidReflowState.SetComputedWidth(width);
 
   nscoord height = availSize.height;
   height -= kidReflowState.mComputedMargin.TopBottom() +
               kidReflowState.mComputedBorderPadding.TopBottom();
-  height = NS_MAX(height, 0);       
+  height = std::max(height, 0);       
   kidReflowState.SetComputedHeight(height); 
 
   // compute the offsets
@@ -739,16 +711,6 @@ nsresult nsTextControlFrame::SetFormProperty(nsIAtom* aName, const nsAString& aV
   return NS_OK;
 }
 
-nsresult
-nsTextControlFrame::GetFormProperty(nsIAtom* aName, nsAString& aValue) const
-{
-  NS_ASSERTION(nsGkAtoms::value != aName,
-               "Should get the value from the content node instead");
-  return NS_OK;
-}
-
-
-
 NS_IMETHODIMP
 nsTextControlFrame::GetEditor(nsIEditor **aEditor)
 {
@@ -775,7 +737,7 @@ nsTextControlFrame::SetSelectionInternal(nsIDOMNode *aStartNode,
   // Note that we use a new range to avoid having to do
   // isIncreasing checks to avoid possible errors.
 
-  nsRefPtr<nsRange> range = new nsRange();
+  nsRefPtr<nsRange> range = new nsRange(mContent);
   nsresult rv = range->SetStart(aStartNode, aStartOffset);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1034,7 +996,7 @@ nsTextControlFrame::OffsetToDOMPoint(int32_t aOffset,
     } else {
       // Otherwise, set the selection on the textnode itself.
       NS_IF_ADDREF(*aResult = firstNode);
-      *aPosition = NS_MIN(aOffset, int32_t(textLength));
+      *aPosition = std::min(aOffset, int32_t(textLength));
     }
   } else {
     NS_IF_ADDREF(*aResult = rootNode);
@@ -1071,18 +1033,10 @@ nsTextControlFrame::GetSelectionRange(int32_t* aSelectionStart,
   rv = selCon->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));  
   NS_ENSURE_SUCCESS(rv, rv);
   NS_ENSURE_TRUE(selection, NS_ERROR_FAILURE);
-  nsCOMPtr<nsISelectionPrivate> selPriv = do_QueryInterface(selection);
-  NS_ENSURE_TRUE(selPriv, NS_ERROR_FAILURE);
-  nsRefPtr<nsFrameSelection> frameSel;
-  rv = selPriv->GetFrameSelection(getter_AddRefs(frameSel));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_TRUE(frameSel, NS_ERROR_FAILURE);
-  nsRefPtr<Selection> typedSel =
-    frameSel->GetSelection(nsISelectionController::SELECTION_NORMAL);
-  NS_ENSURE_TRUE(typedSel, NS_ERROR_FAILURE);
 
+  Selection* sel = static_cast<Selection*>(selection.get());
   if (aDirection) {
-    nsDirection direction = typedSel->GetSelectionDirection();
+    nsDirection direction = sel->GetSelectionDirection();
     if (direction == eDirNext) {
       *aDirection = eForward;
     } else if (direction == eDirPrevious) {
@@ -1098,7 +1052,7 @@ nsTextControlFrame::GetSelectionRange(int32_t* aSelectionStart,
 
   mozilla::dom::Element* root = GetRootNodeAndInitializeEditor();
   NS_ENSURE_STATE(root);
-  nsContentUtils::GetSelectionInTextControl(typedSel, root,
+  nsContentUtils::GetSelectionInTextControl(sel, root,
                                             *aSelectionStart, *aSelectionEnd);
 
   return NS_OK;
@@ -1123,80 +1077,73 @@ nsTextControlFrame::AttributeChanged(int32_t         aNameSpaceID,
   if (needEditor) {
     GetEditor(getter_AddRefs(editor));
   }
-  if ((needEditor && !editor) || !selCon)
+  if ((needEditor && !editor) || !selCon) {
     return nsContainerFrame::AttributeChanged(aNameSpaceID, aAttribute, aModType);
+  }
 
-  nsresult rv = NS_OK;
-
-  if (nsGkAtoms::maxlength == aAttribute) 
-  {
+  if (nsGkAtoms::maxlength == aAttribute) {
     int32_t maxLength;
     bool maxDefined = GetMaxLength(&maxLength);
-    
     nsCOMPtr<nsIPlaintextEditor> textEditor = do_QueryInterface(editor);
-    if (textEditor)
-    {
-      if (maxDefined) 
-      {  // set the maxLength attribute
-          textEditor->SetMaxTextLength(maxLength);
+    if (textEditor) {
+      if (maxDefined) { // set the maxLength attribute
+        textEditor->SetMaxTextLength(maxLength);
         // if maxLength>docLength, we need to truncate the doc content
-      }
-      else { // unset the maxLength attribute
-          textEditor->SetMaxTextLength(-1);
+      } else { // unset the maxLength attribute
+        textEditor->SetMaxTextLength(-1);
       }
     }
-    rv = NS_OK; // don't propagate the error
-  } 
-  else if (nsGkAtoms::readonly == aAttribute) 
-  {
-    uint32_t flags;
-    editor->GetFlags(&flags);
-    if (AttributeExists(nsGkAtoms::readonly))
-    { // set readonly
-      flags |= nsIPlaintextEditor::eEditorReadonlyMask;
-      if (nsContentUtils::IsFocusedContent(mContent))
-        selCon->SetCaretEnabled(false);
-    }
-    else 
-    { // unset readonly
-      flags &= ~(nsIPlaintextEditor::eEditorReadonlyMask);
-      if (!(flags & nsIPlaintextEditor::eEditorDisabledMask) &&
-          nsContentUtils::IsFocusedContent(mContent))
-        selCon->SetCaretEnabled(true);
-    }
-    editor->SetFlags(flags);
-  }
-  else if (nsGkAtoms::disabled == aAttribute) 
-  {
-    uint32_t flags;
-    editor->GetFlags(&flags);
-    if (AttributeExists(nsGkAtoms::disabled))
-    { // set disabled
-      flags |= nsIPlaintextEditor::eEditorDisabledMask;
-      selCon->SetDisplaySelection(nsISelectionController::SELECTION_OFF);
-      if (nsContentUtils::IsFocusedContent(mContent))
-        selCon->SetCaretEnabled(false);
-    }
-    else 
-    { // unset disabled
-      flags &= ~(nsIPlaintextEditor::eEditorDisabledMask);
-      selCon->SetDisplaySelection(nsISelectionController::SELECTION_HIDDEN);
-      if (nsContentUtils::IsFocusedContent(mContent)) {
-        selCon->SetCaretEnabled(true);
-      }
-    }
-    editor->SetFlags(flags);
-  }
-  else if (!mUseEditor && nsGkAtoms::value == aAttribute) {
-    UpdateValueDisplay(true);
-  }
-  // Allow the base class to handle common attributes supported
-  // by all form elements... 
-  else {
-    rv = nsContainerFrame::AttributeChanged(aNameSpaceID, aAttribute, aModType);
+    return NS_OK;
   }
 
-  return rv;
+  if (nsGkAtoms::readonly == aAttribute) {
+    uint32_t flags;
+    editor->GetFlags(&flags);
+    if (AttributeExists(nsGkAtoms::readonly)) { // set readonly
+      flags |= nsIPlaintextEditor::eEditorReadonlyMask;
+      if (nsContentUtils::IsFocusedContent(mContent)) {
+        selCon->SetCaretEnabled(false);
+      }
+    } else { // unset readonly
+      flags &= ~(nsIPlaintextEditor::eEditorReadonlyMask);
+      if (!(flags & nsIPlaintextEditor::eEditorDisabledMask) &&
+          nsContentUtils::IsFocusedContent(mContent)) {
+        selCon->SetCaretEnabled(true);
+      }
+    }
+    editor->SetFlags(flags);
+    return NS_OK;
+  }
+
+  if (nsGkAtoms::disabled == aAttribute) {
+    uint32_t flags;
+    editor->GetFlags(&flags);
+    int16_t displaySelection = nsISelectionController::SELECTION_OFF;
+    const bool focused = nsContentUtils::IsFocusedContent(mContent);
+    const bool hasAttr = AttributeExists(nsGkAtoms::disabled);
+    if (hasAttr) { // set disabled
+      flags |= nsIPlaintextEditor::eEditorDisabledMask;
+    } else { // unset disabled
+      flags &= ~(nsIPlaintextEditor::eEditorDisabledMask);
+      displaySelection = focused ? nsISelectionController::SELECTION_ON
+                                 : nsISelectionController::SELECTION_HIDDEN;
+    }
+    selCon->SetDisplaySelection(displaySelection);
+    if (focused) {
+      selCon->SetCaretEnabled(!hasAttr);
+    }
+    editor->SetFlags(flags);
+    return NS_OK;
+  }
+
+  if (!mEditorHasBeenInitialized && nsGkAtoms::value == aAttribute) {
+    UpdateValueDisplay(true);
+    return NS_OK;
+  }
+
+  // Allow the base class to handle common attributes supported by all form
+  // elements...
+  return nsContainerFrame::AttributeChanged(aNameSpaceID, aAttribute, aModType);
 }
 
 
@@ -1294,12 +1241,6 @@ nsTextControlFrame::SetInitialChildList(ChildListID     aListID,
   return rv;
 }
 
-bool
-nsTextControlFrame::IsScrollable() const
-{
-  return !IsSingleLineTextControl();
-}
-
 void
 nsTextControlFrame::SetValueChanged(bool aValueChanged)
 {
@@ -1331,7 +1272,7 @@ nsTextControlFrame::UpdateValueDisplay(bool aNotify,
   nsIContent* rootNode = txtCtrl->GetRootEditorNode();
 
   NS_PRECONDITION(rootNode, "Must have a div content\n");
-  NS_PRECONDITION(!mUseEditor,
+  NS_PRECONDITION(!mEditorHasBeenInitialized,
                   "Do not call this after editor has been initialized");
   NS_ASSERTION(!mUsePlaceholder || txtCtrl->GetPlaceholderNode(),
                "A placeholder div must exist");
@@ -1339,10 +1280,8 @@ nsTextControlFrame::UpdateValueDisplay(bool aNotify,
   nsIContent *textContent = rootNode->GetChildAt(0);
   if (!textContent) {
     // Set up a textnode with our value
-    nsCOMPtr<nsIContent> textNode;
-    nsresult rv = NS_NewTextNode(getter_AddRefs(textNode),
-                                 mContent->NodeInfo()->NodeInfoManager());
-    NS_ENSURE_SUCCESS(rv, rv);
+    nsRefPtr<nsTextNode> textNode =
+      new nsTextNode(mContent->NodeInfo()->NodeInfoManager());
 
     NS_ASSERTION(textNode, "Must have textcontent!\n");
 
@@ -1456,7 +1395,7 @@ nsTextControlFrame::PeekOffset(nsPeekOffsetStruct *aPos)
   return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP
+void
 nsTextControlFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
                                      const nsRect&           aDirtyRect,
                                      const nsDisplayListSet& aLists)
@@ -1472,22 +1411,50 @@ nsTextControlFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
   nsCOMPtr<nsITextControlElement> txtCtrl = do_QueryInterface(GetContent());
   NS_ASSERTION(txtCtrl, "Content not a text control element!");
 
-  nsresult rv = DisplayBorderBackgroundOutline(aBuilder, aLists);
-  NS_ENSURE_SUCCESS(rv, rv);
+  DisplayBorderBackgroundOutline(aBuilder, aLists);
 
   nsIFrame* kid = mFrames.FirstChild();
-  nsDisplayListSet set(aLists, aLists.Content());
+  // Redirect all lists to the Content list so that nothing can escape, ie
+  // opacity creating stacking contexts that then get sorted with stacking
+  // contexts external to us.
+  nsDisplayList* content = aLists.Content();
+  nsDisplayListSet set(content, content, content, content, content, content);
 
   while (kid) {
     // If the frame is the placeholder frame, we should only show it if the
     // placeholder has to be visible.
     if (kid->GetContent() != txtCtrl->GetPlaceholderNode() ||
         txtCtrl->GetPlaceholderVisibility()) {
-      nsresult rv = BuildDisplayListForChild(aBuilder, kid, aDirtyRect, set, 0);
-      NS_ENSURE_SUCCESS(rv, rv);
+      BuildDisplayListForChild(aBuilder, kid, aDirtyRect, set, 0);
     }
     kid = kid->GetNextSibling();
   }
+}
 
+NS_IMETHODIMP
+nsTextControlFrame::EditorInitializer::Run()
+{
+  if (!mFrame) {
+    return NS_OK;
+  }
+
+  // Need to block script to avoid bug 669767.
+  nsAutoScriptBlocker scriptBlocker;
+
+  nsCOMPtr<nsIPresShell> shell =
+    mFrame->PresContext()->GetPresShell();
+  bool observes = shell->ObservesNativeAnonMutationsForPrint();
+  shell->ObserveNativeAnonMutationsForPrint(true);
+  // This can cause the frame to be destroyed (and call Revoke()).
+  mFrame->EnsureEditorInitialized();
+  shell->ObserveNativeAnonMutationsForPrint(observes);
+
+  // The frame can *still* be destroyed even though we have a scriptblocker,
+  // bug 682684.
+  if (!mFrame) {
+    return NS_ERROR_FAILURE;
+  }
+
+  mFrame->FinishedInitializer();
   return NS_OK;
 }
