@@ -5,7 +5,7 @@
 MOZ_APP_BASENAME=Fennec
 MOZ_APP_VENDOR=Mozilla
 
-MOZ_APP_VERSION=19.0
+MOZ_APP_VERSION=27.0
 MOZ_APP_UA_NAME=Firefox
 
 MOZ_BRANDING_DIRECTORY=mobile/android/branding/unofficial
@@ -14,13 +14,13 @@ MOZ_OFFICIAL_BRANDING_DIRECTORY=mobile/android/branding/official
 
 MOZ_SAFE_BROWSING=1
 
-MOZ_DISABLE_DOMCRYPTO=1
+MOZ_DISABLE_CRYPTOLEGACY=1
 
 # Enable getUserMedia
 MOZ_MEDIA_NAVIGATOR=1
 
-MOZ_SERVICES_COMMON=1
-MOZ_SERVICES_METRICS=1
+# Enable NFC permission
+MOZ_ANDROID_BEAM=1
 
 if test "$LIBXUL_SDK"; then
 MOZ_XULRUNNER=1
@@ -31,7 +31,9 @@ fi
 MOZ_CAPTURE=1
 MOZ_RAW=1
 MOZ_PLACES=
+MOZ_SOCIAL=
 MOZ_ANDROID_HISTORY=1
+MOZ_DISABLE_EXPORT_JS=1
 
 # Needed for building our components as part of libxul
 MOZ_APP_COMPONENT_LIBS="browsercomps"
@@ -40,12 +42,27 @@ MOZ_APP_COMPONENT_INCLUDE=nsBrowserComponents.h
 # use custom widget for html:select
 MOZ_USE_NATIVE_POPUP_WINDOWS=1
 
-# dispatch only touch events (no mouse events)
-MOZ_ONLY_TOUCH_EVENTS=1
-
 MOZ_APP_ID={aa3c5121-dab2-40e2-81ca-7ea25febc110}
 
 MOZ_ANDROID_OMTC=1
 MOZ_EXTENSION_MANAGER=1
 MOZ_APP_STATIC_INI=1
 
+# Enable on-demand decompression
+MOZ_ENABLE_SZIP=1
+
+MOZ_FOLD_LIBS=1
+
+# Enable navigator.mozPay
+MOZ_PAY=1
+
+# Enable UI for healthreporter
+MOZ_SERVICES_HEALTHREPORT=1
+
+# Wifi-AP/cell tower data reporting is enabled on non-release builds.
+if test ! "$RELEASE_BUILD"; then
+MOZ_DATA_REPORTING=1
+fi
+
+# Enable the production cert for verifying signed packaged apps.
+MOZ_B2G_CERTDATA=1
