@@ -12,6 +12,7 @@
 #include "nsStaticAtom.h"
 #include "nsUnicharUtils.h"
 #include "mozilla/HashFunctions.h"
+#include <algorithm>
 
 using namespace mozilla;
 
@@ -34,10 +35,8 @@ static const PRUnichar sHTMLTagUnicodeName_article[] =
   {'a', 'r', 't', 'i', 'c', 'l', 'e', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_aside[] =
   {'a', 's', 'i', 'd', 'e', '\0'};
-#if defined(MOZ_MEDIA)
 static const PRUnichar sHTMLTagUnicodeName_audio[] =
   {'a', 'u', 'd', 'i', 'o', '\0'};
-#endif
 static const PRUnichar sHTMLTagUnicodeName_b[] =
   {'b', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_base[] =
@@ -50,8 +49,6 @@ static const PRUnichar sHTMLTagUnicodeName_bgsound[] =
   {'b', 'g', 's', 'o', 'u', 'n', 'd', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_big[] =
   {'b', 'i', 'g', '\0'};
-static const PRUnichar sHTMLTagUnicodeName_blink[] =
-  {'b', 'l', 'i', 'n', 'k', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_blockquote[] =
   {'b', 'l', 'o', 'c', 'k', 'q', 'u', 'o', 't', 'e', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_body[] =
@@ -74,8 +71,8 @@ static const PRUnichar sHTMLTagUnicodeName_col[] =
   {'c', 'o', 'l', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_colgroup[] =
   {'c', 'o', 'l', 'g', 'r', 'o', 'u', 'p', '\0'};
-static const PRUnichar sHTMLTagUnicodeName_counter[] =
-  {'c', 'o', 'u', 'n', 't', 'e', 'r', '\0'};
+static const PRUnichar sHTMLTagUnicodeName_data[] =
+  {'d', 'a', 't', 'a', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_datalist[] =
   {'d', 'a', 't', 'a', 'l', 'i', 's', 't', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_dd[] =
@@ -160,6 +157,8 @@ static const PRUnichar sHTMLTagUnicodeName_link[] =
   {'l', 'i', 'n', 'k', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_listing[] =
   {'l', 'i', 's', 't', 'i', 'n', 'g', '\0'};
+static const PRUnichar sHTMLTagUnicodeName_main[] =
+  {'m', 'a', 'i', 'n', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_map[] =
   {'m', 'a', 'p', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_mark[] =
@@ -220,12 +219,8 @@ static const PRUnichar sHTMLTagUnicodeName_select[] =
   {'s', 'e', 'l', 'e', 'c', 't', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_small[] =
   {'s', 'm', 'a', 'l', 'l', '\0'};
-#if defined(MOZ_MEDIA)
 static const PRUnichar sHTMLTagUnicodeName_source[] =
   {'s', 'o', 'u', 'r', 'c', 'e', '\0'};
-#endif
-static const PRUnichar sHTMLTagUnicodeName_spacer[] =
-  {'s', 'p', 'a', 'c', 'e', 'r', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_span[] =
   {'s', 'p', 'a', 'n', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_strike[] =
@@ -252,10 +247,16 @@ static const PRUnichar sHTMLTagUnicodeName_th[] =
   {'t', 'h', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_thead[] =
   {'t', 'h', 'e', 'a', 'd', '\0'};
+static const PRUnichar sHTMLTagUnicodeName_template[] =
+  {'t', 'e', 'm', 'p', 'l', 'a', 't', 'e', '\0'};
+static const PRUnichar sHTMLTagUnicodeName_time[] =
+  {'t', 'i', 'm', 'e', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_title[] =
   {'t', 'i', 't', 'l', 'e', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_tr[] =
   {'t', 'r', '\0'};
+static const PRUnichar sHTMLTagUnicodeName_track[] =
+  {'t', 'r', 'a', 'c', 'k', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_tt[] =
   {'t', 't', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_u[] =
@@ -264,10 +265,8 @@ static const PRUnichar sHTMLTagUnicodeName_ul[] =
   {'u', 'l', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_var[] =
   {'v', 'a', 'r', '\0'};
-#if defined(MOZ_MEDIA)
 static const PRUnichar sHTMLTagUnicodeName_video[] =
   {'v', 'i', 'd', 'e', 'o', '\0'};
-#endif
 static const PRUnichar sHTMLTagUnicodeName_wbr[] =
   {'w', 'b', 'r', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_xmp[] =
@@ -287,7 +286,7 @@ const PRUnichar* const nsHTMLTags::sTagUnicodeTable[] = {
 // static array of tag atoms
 nsIAtom* nsHTMLTags::sTagAtomTable[eHTMLTag_userdefined - 1];
 
-PRInt32 nsHTMLTags::gTableRefCount;
+int32_t nsHTMLTags::gTableRefCount;
 PLHashTable* nsHTMLTags::gTagTable;
 PLHashTable* nsHTMLTags::gTagAtomTable;
 
@@ -299,7 +298,7 @@ HTMLTagsHashCodeUCPtr(const void *key)
   return HashString(static_cast<const PRUnichar*>(key));
 }
 
-static PRIntn
+static int
 HTMLTagsKeyCompareUCPtr(const void *key1, const void *key2)
 {
   const PRUnichar *str1 = (const PRUnichar *)key1;
@@ -350,19 +349,19 @@ nsHTMLTags::AddRefTable(void)
 
     gTagTable = PL_NewHashTable(64, HTMLTagsHashCodeUCPtr,
                                 HTMLTagsKeyCompareUCPtr, PL_CompareValues,
-                                nsnull, nsnull);
+                                nullptr, nullptr);
     NS_ENSURE_TRUE(gTagTable, NS_ERROR_OUT_OF_MEMORY);
 
     gTagAtomTable = PL_NewHashTable(64, HTMLTagsHashCodeAtom,
                                     PL_CompareValues, PL_CompareValues,
-                                    nsnull, nsnull);
+                                    nullptr, nullptr);
     NS_ENSURE_TRUE(gTagAtomTable, NS_ERROR_OUT_OF_MEMORY);
 
     // Fill in gTagTable with the above static PRUnichar strings as
     // keys and the value of the corresponding enum as the value in
     // the table.
 
-    PRInt32 i;
+    int32_t i;
     for (i = 0; i < NS_HTML_TAG_MAX; ++i) {
       PL_HashTableAdd(gTagTable, sTagUnicodeTable[i],
                       NS_INT32_TO_PTR(i + 1));
@@ -373,7 +372,7 @@ nsHTMLTags::AddRefTable(void)
 
 
 
-#if defined(DEBUG) && defined(NS_STATIC_ATOM_USE_WIDE_STRINGS)
+#if defined(DEBUG)
     {
       // let's verify that all names in the the table are lowercase...
       for (i = 0; i < NS_HTML_TAG_MAX; ++i) {
@@ -392,10 +391,10 @@ nsHTMLTags::AddRefTable(void)
       }
 
       // let's verify that NS_HTMLTAG_NAME_MAX_LENGTH is correct
-      PRUint32 maxTagNameLength = 0;
+      uint32_t maxTagNameLength = 0;
       for (i = 0; i < NS_HTML_TAG_MAX; ++i) {
-        PRUint32 len = NS_strlen(sTagUnicodeTable[i]);
-        maxTagNameLength = NS_MAX(len, maxTagNameLength);        
+        uint32_t len = NS_strlen(sTagUnicodeTable[i]);
+        maxTagNameLength = std::max(len, maxTagNameLength);        
       }
       NS_ASSERTION(maxTagNameLength == NS_HTMLTAG_NAME_MAX_LENGTH,
                    "NS_HTMLTAG_NAME_MAX_LENGTH not set correctly!");
@@ -416,8 +415,8 @@ nsHTMLTags::ReleaseTable(void)
 
       PL_HashTableDestroy(gTagTable);
       PL_HashTableDestroy(gTagAtomTable);
-      gTagTable = nsnull;
-      gTagAtomTable = nsnull;
+      gTagTable = nullptr;
+      gTagAtomTable = nullptr;
     }
   }
 }
@@ -426,7 +425,7 @@ nsHTMLTags::ReleaseTable(void)
 nsHTMLTag
 nsHTMLTags::LookupTag(const nsAString& aTagName)
 {
-  PRUint32 length = aTagName.Length();
+  uint32_t length = aTagName.Length();
 
   if (length > NS_HTMLTAG_NAME_MAX_LENGTH) {
     return eHTMLTag_userdefined;
@@ -435,7 +434,7 @@ nsHTMLTags::LookupTag(const nsAString& aTagName)
   PRUnichar buf[NS_HTMLTAG_NAME_MAX_LENGTH + 1];
 
   nsAString::const_iterator iter;
-  PRUint32 i = 0;
+  uint32_t i = 0;
   PRUnichar c;
 
   aTagName.BeginReading(iter);
