@@ -27,12 +27,10 @@
 #include "nsNetUtil.h"
 #include "nsIDocument.h"
 #include "nsIContent.h"
-#include "nsIFrame.h"
-#include "nsIView.h"
+#include "nsView.h"
 #include "gfxASurface.h"
 #include "gfxContext.h"
-
-#import <Cocoa/Cocoa.h>
+#include "nsCocoaUtils.h"
 
 #ifdef PR_LOGGING
 extern PRLogModuleInfo* sCocoaLog;
@@ -47,7 +45,7 @@ extern bool gUserCancelledDrag;
 
 // This global makes the transferable array available to Cocoa's promised
 // file destination callback.
-nsISupportsArray *gDraggedTransferables = nsnull;
+nsISupportsArray *gDraggedTransferables = nullptr;
 
 NSString* const kWildcardPboardType = @"MozillaWildcard";
 NSString* const kCorePboardType_url  = @"CorePasteboardFlavorType 0x75726C20"; // 'url '  url
@@ -73,12 +71,12 @@ static nsresult SetUpDragClipboard(nsISupportsArray* aTransferableArray)
   if (!aTransferableArray)
     return NS_ERROR_FAILURE;
 
-  PRUint32 count = 0;
+  uint32_t count = 0;
   aTransferableArray->Count(&count);
 
   NSPasteboard* dragPBoard = [NSPasteboard pasteboardWithName:NSDragPboard];
 
-  for (PRUint32 i = 0; i < count; i++) {
+  for (uint32_t i = 0; i < count; i++) {
     nsCOMPtr<nsISupports> currentTransferableSupports;
     aTransferableArray->GetElementAt(i, getter_AddRefs(currentTransferableSupports));
     if (!currentTransferableSupports)
@@ -117,7 +115,8 @@ static nsresult SetUpDragClipboard(nsISupportsArray* aTransferableArray)
       else if (currentKey == NSTIFFPboardType) {
         [dragPBoard setData:currentValue forType:currentKey];
       }
-      else if (currentKey == NSFilesPromisePboardType) {
+      else if (currentKey == NSFilesPromisePboardType ||
+               currentKey == NSFilenamesPboardType) {
         [dragPBoard setPropertyList:currentValue forType:currentKey];        
       }
     }
@@ -135,29 +134,36 @@ nsDragService::ConstructDragImage(nsIDOMNode* aDOMNode,
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
 
-  NSPoint screenPoint = [[gLastDragView window] convertBaseToScreen:[gLastDragMouseDownEvent locationInWindow]];
+  NSPoint screenPoint =
+    [[gLastDragView window] convertBaseToScreen:
+      [gLastDragMouseDownEvent locationInWindow]];
   // Y coordinates are bottom to top, so reverse this
-  if ([[NSScreen screens] count] > 0)
-    screenPoint.y = NSMaxY([[[NSScreen screens] objectAtIndex:0] frame]) - screenPoint.y;
+  screenPoint.y = nsCocoaUtils::FlippedScreenY(screenPoint.y);
+
+  CGFloat scaleFactor = nsCocoaUtils::GetBackingScaleFactor(gLastDragView);
 
   nsRefPtr<gfxASurface> surface;
   nsPresContext* pc;
   nsresult rv = DrawDrag(aDOMNode, aRegion,
-                         NSToIntRound(screenPoint.x), NSToIntRound(screenPoint.y),
+                         NSToIntRound(screenPoint.x),
+                         NSToIntRound(screenPoint.y),
                          aDragRect, getter_AddRefs(surface), &pc);
   if (!aDragRect->width || !aDragRect->height) {
     // just use some suitable defaults
-    aDragRect->SetRect(NSToIntRound(screenPoint.x), NSToIntRound(screenPoint.y), 20, 20);
+    int32_t size = nsCocoaUtils::CocoaPointsToDevPixels(20, scaleFactor);
+    aDragRect->SetRect(nsCocoaUtils::CocoaPointsToDevPixels(screenPoint.x, scaleFactor),
+                       nsCocoaUtils::CocoaPointsToDevPixels(screenPoint.y, scaleFactor),
+                       size, size);
   }
 
   if (NS_FAILED(rv) || !surface)
     return nil;
 
-  PRUint32 width = aDragRect->width;
-  PRUint32 height = aDragRect->height;
+  uint32_t width = aDragRect->width;
+  uint32_t height = aDragRect->height;
 
   nsRefPtr<gfxImageSurface> imgSurface = new gfxImageSurface(
-    gfxIntSize(width, height), gfxImageSurface::ImageFormatARGB32);
+    gfxIntSize(width, height), gfxImageFormatARGB32);
   if (!imgSurface)
     return nil;
 
@@ -169,43 +175,46 @@ nsDragService::ConstructDragImage(nsIDOMNode* aDOMNode,
   context->SetSource(surface);
   context->Paint();
 
-  PRUint32* imageData = (PRUint32*)imgSurface->Data();
-  PRInt32 stride = imgSurface->Stride();
+  uint32_t* imageData = (uint32_t*)imgSurface->Data();
+  int32_t stride = imgSurface->Stride();
 
-  NSBitmapImageRep* imageRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:NULL
-                                                                       pixelsWide:width
-                                                                       pixelsHigh:height
-                                                                    bitsPerSample:8
-                                                                  samplesPerPixel:4
-                                                                         hasAlpha:YES
-                                                                         isPlanar:NO
-                                                                   colorSpaceName:NSDeviceRGBColorSpace
-                                                                      bytesPerRow:width * 4
-                                                                     bitsPerPixel:32];
+  NSBitmapImageRep* imageRep =
+    [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:NULL
+                                            pixelsWide:width
+                                            pixelsHigh:height
+                                         bitsPerSample:8
+                                       samplesPerPixel:4
+                                              hasAlpha:YES
+                                              isPlanar:NO
+                                        colorSpaceName:NSDeviceRGBColorSpace
+                                           bytesPerRow:width * 4
+                                          bitsPerPixel:32];
 
-  PRUint8* dest = [imageRep bitmapData];
-  for (PRUint32 i = 0; i < height; ++i) {
-    PRUint8* src = (PRUint8 *)imageData + i * stride;
-    for (PRUint32 j = 0; j < width; ++j) {
+  uint8_t* dest = [imageRep bitmapData];
+  for (uint32_t i = 0; i < height; ++i) {
+    uint8_t* src = (uint8_t *)imageData + i * stride;
+    for (uint32_t j = 0; j < width; ++j) {
       // Reduce transparency overall by multipying by a factor. Remember, Alpha
       // is premultipled here. Also, Quartz likes RGBA, so do that translation as well.
 #ifdef IS_BIG_ENDIAN
-      dest[0] = PRUint8(src[1] * DRAG_TRANSLUCENCY);
-      dest[1] = PRUint8(src[2] * DRAG_TRANSLUCENCY);
-      dest[2] = PRUint8(src[3] * DRAG_TRANSLUCENCY);
-      dest[3] = PRUint8(src[0] * DRAG_TRANSLUCENCY);
+      dest[0] = uint8_t(src[1] * DRAG_TRANSLUCENCY);
+      dest[1] = uint8_t(src[2] * DRAG_TRANSLUCENCY);
+      dest[2] = uint8_t(src[3] * DRAG_TRANSLUCENCY);
+      dest[3] = uint8_t(src[0] * DRAG_TRANSLUCENCY);
 #else
-      dest[0] = PRUint8(src[2] * DRAG_TRANSLUCENCY);
-      dest[1] = PRUint8(src[1] * DRAG_TRANSLUCENCY);
-      dest[2] = PRUint8(src[0] * DRAG_TRANSLUCENCY);
-      dest[3] = PRUint8(src[3] * DRAG_TRANSLUCENCY);
+      dest[0] = uint8_t(src[2] * DRAG_TRANSLUCENCY);
+      dest[1] = uint8_t(src[1] * DRAG_TRANSLUCENCY);
+      dest[2] = uint8_t(src[0] * DRAG_TRANSLUCENCY);
+      dest[3] = uint8_t(src[3] * DRAG_TRANSLUCENCY);
 #endif
       src += 4;
       dest += 4;
     }
   }
 
-  NSImage* image = [[NSImage alloc] initWithSize:NSMakeSize((float)width, (float)height)];
+  NSImage* image =
+    [[NSImage alloc] initWithSize:NSMakeSize(width / scaleFactor,
+                                             height / scaleFactor)];
   [image addRepresentation:imageRep];
   [imageRep release];
 
@@ -219,7 +228,7 @@ nsDragService::ConstructDragImage(nsIDOMNode* aDOMNode,
 // stack when InvokeDragSession gets called.
 NS_IMETHODIMP
 nsDragService::InvokeDragSession(nsIDOMNode* aDOMNode, nsISupportsArray* aTransferableArray,
-                                 nsIScriptableRegion* aDragRgn, PRUint32 aActionType)
+                                 nsIScriptableRegion* aDragRgn, uint32_t aActionType)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
@@ -255,12 +264,10 @@ nsDragService::InvokeDragSession(nsIDOMNode* aDOMNode, nsISupportsArray* aTransf
     [image unlockFocus];
   }
 
-  NSPoint point;
-  point.x = dragRect.x;
-  if ([[NSScreen screens] count] > 0)
-    point.y = NSMaxY([[[NSScreen screens] objectAtIndex:0] frame]) - dragRect.YMost();
-  else
-    point.y = dragRect.y;
+  nsIntPoint pt(dragRect.x, dragRect.YMost());
+  CGFloat scaleFactor = nsCocoaUtils::GetBackingScaleFactor(gLastDragView);
+  NSPoint point = nsCocoaUtils::DevPixelsToCocoaPoints(pt, scaleFactor);
+  point.y = nsCocoaUtils::FlippedScreenY(point.y);
 
   point = [[gLastDragView window] convertScreenToBase: point];
   NSPoint localPoint = [gLastDragView convertPoint:point fromView:nil];
@@ -294,7 +301,7 @@ nsDragService::InvokeDragSession(nsIDOMNode* aDOMNode, nsISupportsArray* aTransf
 }
 
 NS_IMETHODIMP
-nsDragService::GetData(nsITransferable* aTransferable, PRUint32 aItemIndex)
+nsDragService::GetData(nsITransferable* aTransferable, uint32_t aItemIndex)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
@@ -307,7 +314,7 @@ nsDragService::GetData(nsITransferable* aTransferable, PRUint32 aItemIndex)
   if (NS_FAILED(rv))
     return NS_ERROR_FAILURE;
 
-  PRUint32 acceptableFlavorCount;
+  uint32_t acceptableFlavorCount;
   flavorList->Count(&acceptableFlavorCount);
 
   // if this drag originated within Mozilla we should just use the cached data from
@@ -318,7 +325,7 @@ nsDragService::GetData(nsITransferable* aTransferable, PRUint32 aItemIndex)
     if (currentTransferableSupports) {
       nsCOMPtr<nsITransferable> currentTransferable(do_QueryInterface(currentTransferableSupports));
       if (currentTransferable) {
-        for (PRUint32 i = 0; i < acceptableFlavorCount; i++) {
+        for (uint32_t i = 0; i < acceptableFlavorCount; i++) {
           nsCOMPtr<nsISupports> genericFlavor;
           flavorList->GetElementAt(i, getter_AddRefs(genericFlavor));
           nsCOMPtr<nsISupportsCString> currentFlavor(do_QueryInterface(genericFlavor));
@@ -328,7 +335,7 @@ nsDragService::GetData(nsITransferable* aTransferable, PRUint32 aItemIndex)
           currentFlavor->ToString(getter_Copies(flavorStr));
 
           nsCOMPtr<nsISupports> dataSupports;
-          PRUint32 dataSize = 0;
+          uint32_t dataSize = 0;
           rv = currentTransferable->GetTransferData(flavorStr, getter_AddRefs(dataSupports), &dataSize);
           if (NS_SUCCEEDED(rv)) {
             aTransferable->SetTransferData(flavorStr, dataSupports, dataSize);
@@ -340,7 +347,7 @@ nsDragService::GetData(nsITransferable* aTransferable, PRUint32 aItemIndex)
   }
 
   // now check the actual clipboard for data
-  for (PRUint32 i = 0; i < acceptableFlavorCount; i++) {
+  for (uint32_t i = 0; i < acceptableFlavorCount; i++) {
     nsCOMPtr<nsISupports> genericFlavor;
     flavorList->GetElementAt(i, getter_AddRefs(genericFlavor));
     nsCOMPtr<nsISupportsCString> currentFlavor(do_QueryInterface(genericFlavor));
@@ -367,7 +374,7 @@ nsDragService::GetData(nsITransferable* aTransferable, PRUint32 aItemIndex)
       PRUnichar* clipboardDataPtr = (PRUnichar*)malloc(dataLength);
       if (!clipboardDataPtr)
         return NS_ERROR_OUT_OF_MEMORY;
-      [filePath getCharacters:clipboardDataPtr];
+      [filePath getCharacters:reinterpret_cast<unichar*>(clipboardDataPtr)];
       clipboardDataPtr[stringLength] = 0; // null terminate
 
       nsCOMPtr<nsIFile> file;
@@ -376,9 +383,7 @@ nsDragService::GetData(nsITransferable* aTransferable, PRUint32 aItemIndex)
       if (NS_FAILED(rv))
         continue;
 
-      nsCOMPtr<nsISupports> genericDataWrapper;
-      genericDataWrapper = do_QueryInterface(file);
-      aTransferable->SetTransferData(flavorStr, genericDataWrapper, dataLength);
+      aTransferable->SetTransferData(flavorStr, file, dataLength);
       
       break;
     }
@@ -401,7 +406,7 @@ nsDragService::GetData(nsITransferable* aTransferable, PRUint32 aItemIndex)
       [stringData getBytes:clipboardDataPtr];
 
       // The DOM only wants LF, so convert from MacOS line endings to DOM line endings.
-      PRInt32 signedDataLength = dataLength;
+      int32_t signedDataLength = dataLength;
       nsLinebreakHelpers::ConvertPlatformToDOMLinebreaks(flavorStr, &clipboardDataPtr, &signedDataLength);
       dataLength = signedDataLength;
 
@@ -450,7 +455,7 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor, bool *_retval)
 
   // first see if we have data for this in our cached transferable
   if (mDataItems) {
-    PRUint32 dataItemsCount;
+    uint32_t dataItemsCount;
     mDataItems->Count(&dataItemsCount);
     for (unsigned int i = 0; i < dataItemsCount; i++) {
       nsCOMPtr<nsISupports> currentTransferableSupports;
@@ -467,9 +472,9 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor, bool *_retval)
       if (NS_FAILED(rv))
         continue;
 
-      PRUint32 flavorCount;
+      uint32_t flavorCount;
       flavorList->Count(&flavorCount);
-      for (PRUint32 j = 0; j < flavorCount; j++) {
+      for (uint32_t j = 0; j < flavorCount; j++) {
         nsCOMPtr<nsISupports> genericFlavor;
         flavorList->GetElementAt(j, getter_AddRefs(genericFlavor));
         nsCOMPtr<nsISupportsCString> currentFlavor(do_QueryInterface(genericFlavor));
@@ -509,7 +514,7 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor, bool *_retval)
 }
 
 NS_IMETHODIMP
-nsDragService::GetNumDropItems(PRUint32* aNumItems)
+nsDragService::GetNumDropItems(uint32_t* aNumItems)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
@@ -555,7 +560,7 @@ nsDragService::EndDragSession(bool aDoneDrag)
   mUserCancelled = gUserCancelledDrag;
 
   nsresult rv = nsBaseDragService::EndDragSession(aDoneDrag);
-  mDataItems = nsnull;
+  mDataItems = nullptr;
   return rv;
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
