@@ -6,13 +6,14 @@
 #include "nsLookAndFeel.h"
 #include <windows.h>
 #include <shellapi.h>
-#include "nsWindow.h"
 #include "nsStyleConsts.h"
 #include "nsUXThemeData.h"
 #include "nsUXThemeConstants.h"
 #include "gfxFont.h"
+#include "gfxWindowsPlatform.h"
 #include "WinUtils.h"
 #include "mozilla/Telemetry.h"
+#include "gfxFontConstants.h"
 
 using namespace mozilla::widget;
 using mozilla::LookAndFeel;
@@ -388,22 +389,50 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
     case eIntID_WindowsThemeIdentifier:
         aResult = nsUXThemeData::GetNativeThemeId();
         break;
+
+    case eIntID_OperatingSystemVersionIdentifier:
+    {
+        switch(WinUtils::GetWindowsVersion()) {
+            case WinUtils::WINXP_VERSION:
+            case WinUtils::WIN2K3_VERSION:
+                aResult = LookAndFeel::eOperatingSystemVersion_WindowsXP;
+                break;
+            case WinUtils::VISTA_VERSION:
+                aResult = LookAndFeel::eOperatingSystemVersion_WindowsVista;
+                break;
+            case WinUtils::WIN7_VERSION:
+                aResult = LookAndFeel::eOperatingSystemVersion_Windows7;
+                break;
+            case WinUtils::WIN8_VERSION:
+                aResult = LookAndFeel::eOperatingSystemVersion_Windows8;
+                break;
+            default:
+                aResult = LookAndFeel::eOperatingSystemVersion_Unknown;
+                break;
+        }
+        break;
+    }
+
     case eIntID_MacGraphiteTheme:
     case eIntID_MacLionTheme:
-    case eIntID_MaemoClassic:
         aResult = 0;
         res = NS_ERROR_NOT_IMPLEMENTED;
         break;
     case eIntID_DWMCompositor:
         aResult = nsUXThemeData::CheckForCompositor();
         break;
+    case eIntID_WindowsGlass:
+        // Aero Glass is only available prior to Windows 8 when DWM is used.
+        aResult = (nsUXThemeData::CheckForCompositor() &&
+                   WinUtils::GetWindowsVersion() < WinUtils::WIN8_VERSION);
+        break;
     case eIntID_AlertNotificationOrigin:
         aResult = 0;
         {
           // Get task bar window handle
-          HWND shellWindow = FindWindowW(L"Shell_TrayWnd", NULL);
+          HWND shellWindow = FindWindowW(L"Shell_TrayWnd", nullptr);
 
-          if (shellWindow != NULL)
+          if (shellWindow != nullptr)
           {
             // Determine position
             APPBARDATA appBarData;
@@ -450,6 +479,24 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
     case eIntID_ScrollbarButtonAutoRepeatBehavior:
         aResult = 0;
         break;
+    case eIntID_SwipeAnimationEnabled:
+        aResult = 0;
+        break;
+    case eIntID_UseOverlayScrollbars:
+        aResult = (XRE_GetWindowsEnvironment() == WindowsEnvironmentType_Metro);
+        break;
+    case eIntID_AllowOverlayScrollbarsOverlap:
+        aResult = 0;
+        break;
+    case eIntID_ScrollbarDisplayOnMouseMove:
+        aResult = 1;
+        break;
+    case eIntID_ScrollbarFadeBeginDelay:
+        aResult = 2500;
+        break;
+    case eIntID_ScrollbarFadeDuration:
+        aResult = 350;
+        break;
     default:
         aResult = 0;
         res = NS_ERROR_FAILURE;
@@ -484,7 +531,7 @@ GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
                nsString &aFontName,
                gfxFontStyle &aFontStyle)
 {
-  LOGFONTW* ptrLogFont = NULL;
+  LOGFONTW* ptrLogFont = nullptr;
   LOGFONTW logFont;
   NONCLIENTMETRICSW ncm;
   HGDIOBJ hGDI;
@@ -551,18 +598,19 @@ GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
     break;
   }
 
-  // FIXME?: mPixelScale is currently hardcoded to 1.
-  float mPixelScale = 1.0f;
+  // Get scaling factor from physical to logical pixels
+  float pixelScale = 1.0f / gfxWindowsPlatform::GetPlatform()->GetDPIScale();
 
   // The lfHeight is in pixels, and it needs to be adjusted for the
   // device it will be displayed on.
   // Screens and Printers will differ in DPI
   //
   // So this accounts for the difference in the DeviceContexts
-  // The mPixelScale will be a "1" for the screen and could be
-  // any value when going to a printer, for example mPixleScale is
+  // The pixelScale will typically be 1.0 for the screen
+  // (though larger for hi-dpi screens where the Windows resolution
+  // scale factor is 125% or 150% or even more), and could be
+  // any value when going to a printer, for example pixelScale is
   // 6.25 when going to a 600dpi printer.
-  // round, but take into account whether it is negative
   float pixelHeight = -ptrLogFont->lfHeight;
   if (pixelHeight < 0) {
     HFONT hFont = ::CreateFontIndirectW(ptrLogFont);
@@ -575,7 +623,7 @@ GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
     ::DeleteObject(hFont);
     pixelHeight = tm.tmAscent;
   }
-  pixelHeight *= mPixelScale;
+  pixelHeight *= pixelScale;
 
   // we have problem on Simplified Chinese system because the system
   // report the default font size is 8 points. but if we use 8, the text
@@ -612,9 +660,11 @@ nsLookAndFeel::GetFontImpl(FontID anID, nsString &aFontName,
                            gfxFontStyle &aFontStyle,
                            float aDevPixPerCSSPixel)
 {
-  HDC tdc = GetDC(NULL);
+  HDC tdc = GetDC(nullptr);
   bool status = GetSysFontInfo(tdc, anID, aFontName, aFontStyle);
-  ReleaseDC(NULL, tdc);
+  ReleaseDC(nullptr, tdc);
+  // now convert the logical font size from GetSysFontInfo into device pixels for layout
+  aFontStyle.size *= aDevPixPerCSSPixel;
   return status;
 }
 
