@@ -15,13 +15,12 @@
  */
 
 #include <string.h>
-#include "prmem.h"
 #include "prlog.h"
+#include "prlock.h"
 #include "nsHashtable.h"
-#include "nsReadableUtils.h"
 #include "nsIObjectInputStream.h"
 #include "nsIObjectOutputStream.h"
-#include "nsCRT.h"
+#include "nsCRTGlue.h"
 #include "mozilla/HashFunctions.h"
 
 using namespace mozilla;
@@ -120,7 +119,7 @@ nsHashKey::Write(nsIObjectOutputStream* aStream) const
 }
 
 nsHashtable::nsHashtable(uint32_t aInitSize, bool threadSafe)
-  : mLock(NULL), mEnumerating(false)
+  : mLock(nullptr), mEnumerating(false)
 {
     MOZ_COUNT_CTOR(nsHashtable);
 
@@ -135,10 +134,10 @@ nsHashtable::nsHashtable(uint32_t aInitSize, bool threadSafe)
     
     if (threadSafe) {
         mLock = PR_NewLock();
-        if (mLock == NULL) {
+        if (mLock == nullptr) {
             // Cannot create a lock. If running on a multiprocessing system
             // we are sure to die.
-            PR_ASSERT(mLock != NULL);
+            PR_ASSERT(mLock != nullptr);
         }
     }
 }
@@ -172,7 +171,7 @@ bool nsHashtable::Exists(nsHashKey *aKey)
 
 void *nsHashtable::Put(nsHashKey *aKey, void *aData)
 {
-    void *res =  NULL;
+    void *res =  nullptr;
 
     if (!mHashtable.ops) return nullptr;
     
@@ -300,7 +299,7 @@ hashEnumerateRemove(PLDHashTable*, PLDHashEntryHdr* hdr, uint32_t i, void *arg)
 }
 
 void nsHashtable::Reset() {
-    Reset(NULL);
+    Reset(nullptr);
 }
 
 void nsHashtable::Reset(nsHashtableEnumFunc destroyFunc, void* aClosure)
@@ -532,8 +531,8 @@ nsCStringKey::Clone() const
 
     uint32_t len = mStrLen * sizeof(char);
     char* str = (char*)nsMemory::Alloc(len + sizeof(char));
-    if (str == NULL)
-        return NULL;
+    if (str == nullptr)
+        return nullptr;
     memcpy(str, mStr, len);
     str[len] = 0;
     return new nsCStringKey(str, mStrLen, OWN);
@@ -655,8 +654,8 @@ nsStringKey::Clone() const
 
     uint32_t len = (mStrLen+1) * sizeof(PRUnichar);
     PRUnichar* str = (PRUnichar*)nsMemory::Alloc(len);
-    if (str == NULL)
-        return NULL;
+    if (str == nullptr)
+        return nullptr;
     memcpy(str, mStr, len);
     return new nsStringKey(str, mStrLen, OWN);
 }
